///////////////
// IanniX
///////////////

////////////////////////////
// NxScheduler.cpp
////////////////////////////

#include "NxScheduler.h"

// Time lib
#include <unistd.h>
#include <math.h>

//////////////////////////////////
// THREAD CLASS
//////////////////////////////////

// Timing Scheduler for IanniX
/////////////////////////////////////////////////////

// THREADING PART
//////////////////////////////////////

///////////////////
// Constructor
///////////////////
NxScheduler::NxScheduler( NxGLScore* inGLBox )
{
	mainGLBox = inGLBox;
	
	nbPackets 	= 0 ;
	
  	pCur 		= NULL;
	finished 	= true;
  	playing 	= false;

	mainTempo = (float)((NxScore*)mainGLBox->getPartition())->getEnvironment().globalTempo ;
	
	clock = QTime(0,0,0,0) ;
	useCirclesAlgo = false ;
	sendInitialFinalMessage = false ;
	advanceCursors = true ;
	
	addTrigger = addCurve = false ;
	readyToPlay = false ;

	//sTimer = new QTimer( this );
}

//////////////////
// Destructor
//////////////////
NxScheduler::~NxScheduler()
{}
////////////////////////////
// Playback Parameters
////////////////////////////
void NxScheduler::useAutoSchedulerStep( bool chooser )
{
	autoSchedulerStep = chooser ;
}
bool NxScheduler::isAutoSchedulerStep() 
{
	return autoSchedulerStep ;
}
void NxScheduler::useCirclesAlgorithm( bool chooser, bool withReset )
{
	if ( chooser != useCirclesAlgo && withReset ) resetCurves() ;
	useCirclesAlgo = chooser ;
}
void NxScheduler::sendIFmessages( bool chooser )
{
	sendInitialFinalMessage = chooser ;
}
void NxScheduler::setOSCOutActive( bool chooser )	
{
	oscActive = chooser;
}
void NxScheduler::useBroadcastMode( bool inBroad )
{
	broadcastMode = inBroad ;
}
void NxScheduler::moveCursors( bool chooser )
{
	advanceCursors = chooser ;
}
////////////////////////////
// Time Settings
////////////////////////////
void NxScheduler::setPlaying( bool chooser )
{
	playing = chooser;
}
bool NxScheduler::isPlaying()
{
	return playing ;
}
void NxScheduler::changeTempo( float inTempo )
{
	mainTempo = inTempo;
}
void NxScheduler::setCurrentTime( float inTime )
{
	currentTime = inTime ;
}
void NxScheduler::setInitialTime( float inStartTime )	
{
	initialTime = inStartTime;
}
void NxScheduler::setFinalTime( float inStopTime )	
{
	finalTime = inStopTime;
}
void NxScheduler::resetTime()
{
	currentTime = initialTime ;
}
float NxScheduler::getCurrentTime()
{
	return currentTime ;
}
float NxScheduler::getInitialTime()
{
	return initialTime ;
}
float NxScheduler::getFinalTime()
{
	return finalTime ;
}

///////////////////////////////////////////////////////////////////////////////////////////////////
///////////// UPDATE OBJECTS NUMBERS
///////////////////////////////////////////////////////////////////////////////////////////////////
void NxScheduler::updateObjectsIndexes()
{
	QList<int> cursInd = score->cursorsIndexes() ;
	QList<int> curvInd = score->curvesIndexes() ;
	QList<int> trigInd = score->triggersIndexes() ;
	nbCursors	= cursInd.count() ;
	nbCurves	= curvInd.count() ;
	nbTriggers	= trigInd.count() ;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////// INITIALISATION
///////////////////////////////////////////////////////////////////////////////////////////////////
bool NxScheduler::isReadyToPlay() 
{
	return readyToPlay ;
}
void NxScheduler::setReadyToPlay( bool chooser ) 
{
	readyToPlay = chooser ;
}
void NxScheduler::initScheduler() // Called only at the begining and reset
{	
	readyToPlay = false ;
	currentTime = initialTime ;
	
	/////////////// INITIAL TIME SET - Here we shift all the cursors
	if(currentTime!=0.0 && mainTempo>0.0) {
		NxCursor* cur ;
		float len, initO, finalO ;
		bool forw ;
		float tmpOffset, Offset ;
	
		QList<int> indexes = score->cursorsIndexes() ;
		for(int i=0; i<indexes.count(); i++) {
			cur = score->getCursor( indexes.at(i) ) ;
			len = ((NxTraject*)cur->getParentTrajectory())->getLength() ;
			initO = cur->getInitOffset() ;
			finalO = cur->getFinalOffset() ;
			forw = cur->getForward() ;
			
			tmpOffset = initO + initialTime * cur->getSpeed() * fabs(mainTempo) ;
			if( tmpOffset < finalO )
				Offset = tmpOffset ;
			else
				Offset = finalO ;
			
			if(forw) cur->setOffset( Offset ) ;
			else cur->setOffset( len - Offset ) ;
		}
	}
	/////////////////////////////////////////////////////////////////
	
	// Initializing OSC buffer
 	b = &myBuf;
 	OSC_initBuffer(b, 1000, bytes);	
	
	resetSchedule() ;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////// LIVE OBJECT ADDING
///////////////////////////////////////////////////////////////////////////////////////////////////
void NxScheduler::addNewCursor()
{
	resetSchedule() ;
}
void NxScheduler::addNewCurve()
{
	if(!playing) resetSchedule() ;
	else addCurve = true ;
}
void NxScheduler::doAddNewCurve()
{
	QList<int> indexes = score->cursorsIndexes() ;
	mutex.lock() ; // useful ?
	for(int i=0; i<indexes.count(); i++) {	
		QList<float>	tmpValCTimers		= curvesTimers.at(i) ;	
		QList<float>	tmpValCDurations	= curvesDurations.at(i) ;
		QList<bool>		tmpValCCollisions	= curvesCollisions.at(i) ;	
		QList<float>	tmpValCLocalStep	= curvesLocalStep.at(i) ;			
		tmpValCTimers		<< 0.0 ;
		tmpValCDurations	<< 5.0 ;
		tmpValCCollisions	<< false ;
		tmpValCLocalStep	<< 0.0 ;
		curvesTimers.replace( i , tmpValCTimers ) ;
		curvesDurations.replace( i , tmpValCDurations ) ;
		curvesCollisions.replace( i , tmpValCCollisions ) ;
		curvesLocalStep.replace( i , tmpValCLocalStep ) ;
	}
	updateObjectsIndexes() ; // nbCurves actualized (+= 1)
	mutex.unlock() ; // useful ?
}
void NxScheduler::addNewTrigger()
{
	if(!playing) resetSchedule() ;
	else addTrigger = true ;
}
void NxScheduler::doAddNewTrigger()
{
	QList<int> indexes = score->cursorsIndexes() ;
	mutex.lock() ; // useful ?
  	for(int i=0; i<indexes.count(); i++) {	
		QList<bool> tmpList1 = triggersSign1.at(i) ;
		QList<bool> tmpList2 = triggersSign2.at(i) ;
		tmpList1 << true ; // adding a value
		tmpList2 << true ; // adding a value
		triggersSign1.replace( i , tmpList1 ) ;
		triggersSign2.replace( i , tmpList2 ) ;
	}
	updateObjectsIndexes() ; // nbTriggers actualized (+= 1)
	triggersFirstShot << true ; // To avoid banging
	firstShot = true ;
	mutex.unlock() ; // useful ?
}
void NxScheduler::setFirstShot( bool chooser )
{
	QList<int> tindexes = score->triggersIndexes() ;
	triggersFirstShot.clear() ;
	
	for(int i=0; i<tindexes.count(); i++) { // for all triggers
		triggersFirstShot << chooser ;
	}
	firstShot = chooser ;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////// RESET ALL
///////////////////////////////////////////////////////////////////////////////////////////////////
void NxScheduler::resetSchedule() // Reinitialize arrays
{
	nbPackets = 0 ;

	score = mainGLBox->getPartition();
	
	updateObjectsIndexes() ;
	
	mutex.lock() ; // useful ?
	resetCursors() ;
	resetCurves() ;
	resetTriggers() ;
	resetOSC() ;
	mutex.unlock() ; // useful ?
	
	readyToPlay = true ;
}
////////////////////////////////////// Reset Cursors
void NxScheduler::resetCursors()
{	
	curDirections.clear() ;
	cursorsFinished.clear() ;
		
	QList<int> indexes = score->cursorsIndexes() ;
	for(int i=0; i<indexes.count(); i++) {
		curDirections << ((NxCursor*)score->getCursor( indexes.at(i) ))->getForward() ; // direction at time t
		cursorsFinished << false ;
	}
}
////////////////////////////////////// Reset Curves
void NxScheduler::resetCurves()
{	
	curvesTimers.clear() ;
	curvesDurations.clear() ;
	curvesCollisions.clear() ;
	curvesLocalStep.clear() ;
	
	resetSplines() ;
	
	QList<int> indexes = score->cursorsIndexes() ;
	QList<int> cindexes = score->curvesIndexes() ;
	for(int i=0; i<indexes.count(); i++) {	
		QList<float>	valCTimers ;	
		QList<float>	valCDurations ;
		QList<bool>		valCCollisions ;	
		QList<float>	valCLocalStep ;			
		for (int j=0; j<cindexes.count(); j++) {
			valCTimers		<< 0.0 ;
			valCDurations	<< 5.0 ;
			valCCollisions	<< false ;
			valCLocalStep	<< 0.0 ;
		}
		curvesTimers		<< valCTimers ;
		curvesDurations		<< valCDurations ;
		curvesCollisions	<< valCCollisions ;
		curvesLocalStep		<< valCLocalStep ;
	}
}
////////////////////////////////////// Reset Splines
void NxScheduler::resetSplines()
{	
	QList<int> ind = score->curvesIndexes() ;
	for( int i=0; i< score->getEnvironment().nbCurves;i++ ) {		
		if( score->getCurve(ind.at(i))->getDrawType() == SplineStrip )
			score->getCurve(ind.at(i))->getSpline()->reset();
	}
}
////////////////////////////////////// Reset Triggers
void NxScheduler::resetTriggers()
{	
	triggersSign1.clear() ; /////////// trigger collisions / sign tests
	triggersSign2.clear() ; /////////// trigger collisions / sign tests
	triggersFirstShot.clear() ; // triggers firstShot banging
	
	QList<int> indexes = score->cursorsIndexes() ;
	QList<int> tindexes = score->triggersIndexes() ;
	
	for(int i=0; i<tindexes.count(); i++) { // for all triggers
		triggersFirstShot << true ;
	}
	
  	for(int i=0; i<indexes.count(); i++) {	// for all cursors
		QList<bool> valTBool1 ;
		QList<bool> valTBool2 ;	
		for(int k=0; k<tindexes.count(); k++) { // for all triggers
			valTBool1 << true ;
			valTBool2 << true ;	
		}
		triggersSign1 << valTBool1  ;
		triggersSign2 << valTBool2  ;
	}
}
////////////////////////////////////// Reset OSC
void NxScheduler::resetOSC()
{	
	triggersMessages.clear() ;
	curvesMessages.clear() ;
}
///////////////////////////////////////////////////////////////////////////////////////////
///////// CONFIG / INITIAL / FINAL OSC MESSAGES
///////////////////////////////////////////////////////////////////////////////////////////
void NxScheduler::sendOSCConfigMessage()
{
	NxEnv tmpEnv = ((NxScore*)mainGLBox->getPartition())->getEnvironment() ;
	
	printf("Sending OSC Configuration Message (%s:%d) ...\n", tmpEnv.OSCConfigMessageIP.toAscii().constData(), tmpEnv.UDPout) ;
	
	char* charOSC1 = new char[100];
	QString configOSC = tmpEnv.OSCConfigMessageIP  ;
	strncpy(charOSC1, configOSC.toAscii(), 99);
	envClientConfigUDP = new NxNetwork( charOSC1, tmpEnv.UDPout );
	OSC_resetBuffer( b );
	///////////////////////////////////////////////////////////////////////////////////////////
	OSC_writeAddressAndTypes(b, "/iannix/score/dimensions",",ii");
	OSC_writeIntArg( b, tmpEnv.length ) ;
	OSC_writeIntArg( b, tmpEnv.height ) ;
	envClientConfigUDP->sendPacket(OSC_getPacket(b), OSC_packetSize(b), false ) ;
	OSC_resetBuffer( b ) ;
	///////////////////////////////////////////////////////////////////////////////////////////
	OSC_writeAddressAndTypes(b, "/iannix/score/time",",f");
	OSC_writeFloatArg( b, currentTime ) ;		
	envClientConfigUDP->sendPacket(OSC_getPacket(b), OSC_packetSize(b), false ) ;
	OSC_resetBuffer( b );
	///////////////////////////////////////////////////////////////////////////////////////////
	QList<int> cursInd = score->cursorsIndexes() ;
	QList<int> trajInd = score->trajectsIndexes() ;
	QList<int> curvInd = score->curvesIndexes() ;
	QList<int> trigInd = score->triggersIndexes() ;
	///////////////////////////////////////////////////////////////////////////////////////////
	OSC_writeAddressAndTypes(b, "/iannix/score/cursors",",i");
	OSC_writeIntArg( b, tmpEnv.nbCursors ) ;
	envClientConfigUDP->sendPacket(OSC_getPacket(b), OSC_packetSize(b), false );
	OSC_resetBuffer( b );
	NxCursor* curs ;
	for(int k=0 ; k<tmpEnv.nbCursors ; k++) { // CURSORS
		curs = score->getCursor( cursInd.at(k) ) ;
		int forwInt ;
		if(curs->getForward()) forwInt = 1 ;
		else forwInt = 0 ;
		OSC_writeAddressAndTypes(b, "/iannix/score/cursor",",iiiffii");
		OSC_writeIntArg( b, curs->getID() ) ;			// ID
		OSC_writeIntArg( b, curs->getCustomID() ) ;		// Custom ID
		OSC_writeIntArg( b, curs->getGroupID() ) ;		// Group ID
		OSC_writeFloatArg( b, curs->getSpeed() ) ;		// Speed
		OSC_writeFloatArg( b, curs->getWidth() ) ;		// Width
		OSC_writeIntArg( b, forwInt ) ;					// Forward or Backward
		OSC_writeIntArg( b, curs->getMode() ) ;			// Repeat Mode
		envClientConfigUDP->sendPacket(OSC_getPacket(b), OSC_packetSize(b), false );
		OSC_resetBuffer( b );
	}
	///////////////////////////////////////////////////////////////////////////////////////////
	OSC_writeAddressAndTypes(b, "/iannix/score/trajects",",i");	
	OSC_writeIntArg( b, tmpEnv.nbTrajects ) ;
	envClientConfigUDP->sendPacket(OSC_getPacket(b), OSC_packetSize(b), false );
	OSC_resetBuffer( b );
	NxTraject* traj ;
	for(int k=0 ; k<tmpEnv.nbTrajects ; k++) { // TRAJECTS
		traj = score->getTraject( trajInd.at(k) ) ;
		OSC_writeAddressAndTypes(b, "/iannix/score/traject",",iiif");	
		OSC_writeIntArg( b, traj->getID() ) ;			// ID
		OSC_writeIntArg( b, traj->getCustomID() ) ;		// Custom ID
		OSC_writeIntArg( b, traj->getGroupID() ) ;		// Group ID
		OSC_writeFloatArg( b, traj->getLength() ) ;		// Length
		envClientConfigUDP->sendPacket(OSC_getPacket(b), OSC_packetSize(b), false );
		OSC_resetBuffer( b );
	}
	///////////////////////////////////////////////////////////////////////////////////////////
	OSC_writeAddressAndTypes(b, "/iannix/score/curves",",i");
	OSC_writeIntArg( b, tmpEnv.nbCurves ) ;	
	envClientConfigUDP->sendPacket(OSC_getPacket(b), OSC_packetSize(b), false );
	OSC_resetBuffer( b );
	NxCurve *curv ;
	for(int k=0 ; k<tmpEnv.nbCurves ; k++) { // CURVES
		curv = score->getCurve( curvInd.at(k) ) ;
		OSC_writeAddressAndTypes(b, "/iannix/score/curve",",iiifff");
		OSC_writeIntArg( b, curv->getID() ) ;			// ID
		OSC_writeIntArg( b, curv->getCustomID() ) ;		// Custom ID
		OSC_writeIntArg( b, curv->getGroupID() ) ;		// Group ID
		OSC_writeFloatArg( b, curv->getRealMin() ) ;	// Real Min
		OSC_writeFloatArg( b, curv->getRealMax() ) ;	// Real Max
		OSC_writeFloatArg( b, curv->getXRange() ) ;		// Duration
		envClientConfigUDP->sendPacket(OSC_getPacket(b), OSC_packetSize(b), false );
		OSC_resetBuffer( b );
	}
	///////////////////////////////////////////////////////////////////////////////////////////
	OSC_writeAddressAndTypes(b, "/iannix/score/triggers",",i");
	OSC_writeIntArg( b, tmpEnv.nbTriggers ) ;
	envClientConfigUDP->sendPacket(OSC_getPacket(b), OSC_packetSize(b), false );
	OSC_resetBuffer( b );	
	NxTrigger *trig ;
	for(int k=0 ; k<tmpEnv.nbTriggers ; k++) { // TRIGGERS
		trig = score->getTrigger( trigInd.at(k) ) ;
		OSC_writeAddressAndTypes(b, "/iannix/score/trigger",",iiii");
		OSC_writeIntArg( b, trig->getID() ) ;			// ID
		OSC_writeIntArg( b, trig->getCustomID() ) ;		// Custom ID
		OSC_writeIntArg( b, trig->getGroupID() ) ;		// Group ID
		OSC_writeIntArg( b, trig->getValue() ) ;		// Int Value
		envClientConfigUDP->sendPacket(OSC_getPacket(b), OSC_packetSize(b), false );
		OSC_resetBuffer( b );	
	}
	///////////////////////////////////////////////////////////////////////////////////////////
	delete(envClientConfigUDP) ;
}
///////////////////////////////////////////////////////////////////////////////////////////
///////// INITIAL and FINAL OSC MESSAGES
///////////////////////////////////////////////////////////////////////////////////////////
void NxScheduler::sendOSCFirstMessage()
{
	NxEnv tmpEnv = ((NxScore*)mainGLBox->getPartition())->getEnvironment() ;
	
	printf("Sending OSC Initial Message (255.255.255.255:%d) ...\n", tmpEnv.UDPout) ;
	
	char* charOSC1 = new char[100];
	QString firstOSC = QString("255.255.255.255")  ;
	strncpy(charOSC1, firstOSC.toAscii(), 99);
	envClientFirstUDP = new NxNetwork( charOSC1, tmpEnv.UDPout );
	OSC_resetBuffer( b );
	///////////////////////////////////////////////////////////////////////////////////////////
	OSC_writeAddressAndTypes(b, "/iannix/score/play",",f");
	OSC_writeFloatArg( b, currentTime ) ;		
	envClientFirstUDP->sendPacket(OSC_getPacket(b), OSC_packetSize(b), false );
	OSC_resetBuffer( b );
	///////////////////////////////////////////////////////////////////////////////////////////
	delete(envClientFirstUDP) ;
}
void NxScheduler::sendOSCLastMessage()
{
	NxEnv tmpEnv = ((NxScore*)mainGLBox->getPartition())->getEnvironment() ;
	
	printf("Sending OSC Final Message (255.255.255.255:%d) ...\n", tmpEnv.UDPout) ;

	char* charOSC2 = new char[100];
	QString lastOSC = QString("255.255.255.255")  ;
	strncpy(charOSC2, lastOSC.toAscii(), 99);
	envClientLastUDP = new NxNetwork( charOSC2, tmpEnv.UDPout );
	OSC_resetBuffer( b );
	///////////////////////////////////////////////////////////////////////////////////////////
	OSC_writeAddressAndTypes(b, "/iannix/score/stop",",f");
	OSC_writeFloatArg( b, currentTime ) ;		
	envClientLastUDP->sendPacket(OSC_getPacket(b), OSC_packetSize(b), false );
	OSC_resetBuffer( b );
	///////////////////////////////////////////////////////////////////////////////////////////
	OSC_writeAddressAndTypes(b, "/iannix/score/packets",",i");
	OSC_writeIntArg( b, nbPackets ) ;		
	envClientLastUDP->sendPacket(OSC_getPacket(b), OSC_packetSize(b), false );
	OSC_resetBuffer( b );
	///////////////////////////////////////////////////////////////////////////////////////////
	delete(envClientLastUDP) ;
}
/////////////////////
// MAIN TIMING METHOD
/////////////////////
void NxScheduler::run()
{	
	NxEnv tmpEnv = ((NxScore*)mainGLBox->getPartition())->getEnvironment() ;
	NxCursor* curs ;
	
	printf("//////////////////////////////\n") ;
	if(useCirclesAlgo) printf("We use the Circles Algorithm\n") ;
	else printf("We use the Segment Algorithm\n") ;
	
	////////////////////// OSC ENVIRONMENT FIRST MESSAGE //////////////////////
	if (oscActive && sendInitialFinalMessage) sendOSCFirstMessage() ;

	///////////// Curves / Triggers UDP CLIENTS Initialisation ///////////
	curvesOSCmF = new char[100] ;
	triggersOSCmF = new char[100] ;
	QString cOSC = tmpEnv.OSCCurvesDefaultMFormat ;
	QString tOSC = tmpEnv.OSCTriggersDefaultMFormat ;
	strncpy(curvesOSCmF, cOSC.toAscii(), 99);
	strncpy(triggersOSCmF, tOSC.toAscii(), 99);
	//////////////////////////////////////////////////////////////////////
	finished = false ;
	next = true ;
	firstShot = true ;

	////////////////// TIME CONTROL //////////////////
	gPrecision = tmpEnv.schedulerStep ;		/// in ms
	dtime = gPrecision ;
	if(autoSchedulerStep) {
		gPrecision = 4 ;
		dtime = 4 ;
		printf("Auto Scheduler TimeStep Starting with: %d ms\n", gPrecision ) ;
	}
	else printf("Manual Scheduler TimeStep : %d ms\n", gPrecision ) ;

	float xStep ;			///// advance step of cursor	(in ms)
	int timeLeft = 0 ;		///// sleep time				(in ms)
	int loopCount = 50 ;

	clock.start() ;
	QTime tt0 = QTime(0,0,0,0) ;
	QTime tt1 = QTime(0,0,0,0) ;
	QTime tt2 = QTime(0,0,0,0) ;
	
	//connect( sTimer, SIGNAL(timeout()), this, SLOT(nextStep()) );
	//sTimer->start(10) ;
	//next = true ;
	///////////////////////////////////////////////////
	
	updateObjectsIndexes() ; // Update Objects NB (nbCursors / nbTriggers / ...)
	
    QList<int> indexes = score->cursorsIndexes();
	QList<int> curveindexes = score->curvesIndexes();
	
	QList<float> curveTempStep ;
	
	while (finished == false)
    {
		tt0 = clock.currentTime() ; //// get init calcul Time
		//////////////////////////////////////////////////////////////////////////////////////////
		for (int i=0; i<nbCursors; i++) {
			curs = score->getCursor( indexes.at(i) ) ;
			if(advanceCursors) updateCursorPos( i ) ;
			if (oscActive) {
				curveTempStep = curvesLocalStep.at(i) ;
				
				if( curs->getPlayFinished() || !cursorsFinished.at(i) ) triggersCollisionTest( i ) ;

				for (int k=0; k<nbCurves; k++) {
					float customPeriod = score->getCurve( curveindexes.at(k) )->getSpatialPeriod() ;
					/////// get the spatial step advance (in ms)
					xStep = dtime * fabs(mainTempo) * curs->getSpeed() ;
					//if ( (xStep >= customPeriod) && (loopCount == 50) ) {
						//mainGLBox->setCPUWarning(true) ;
						//mainGLBox->setToolBarInfosTxt( QString("At Least one Cursor is Going too Fast\n( Cursor Step > Curve Spatial Period )") ) ;
						//}
					curveTempStep.replace(k, curveTempStep.at(k) + xStep ) ;
					if( (float)curveTempStep.at(k) >= customPeriod ) {
						//printf("////////// Test COLLISION\n") ;
						curveTempStep.replace(k, 0.0) ;
						
						if( curs->getPlayFinished() || !cursorsFinished.at(i) ) curvesCollisionTest( i, k ) ;
					} // end test collision
				} // end for (curves)
				curvesLocalStep.replace( i , curveTempStep ) ;
				cursorsFinished.replace( i , false ) ;
			} // end if (oscActive)
		} // end for (cursors)

		if(firstShot) setFirstShot( false ) ; // put all triggers firstShot && firstShot bools to false
		//////////////////////////////////////////////////////////////////////////////////////////
		
		//////////////////////////////////////////////////////////////////////////////////////////
		tt1 = clock.currentTime() ;
		//printf("timestep : %d\n", tt1.msecsTo(tt0)) ;
		timeLeft = gPrecision + tt1.msecsTo(tt0) ;
		if (timeLeft>0) usleep( timeLeft*1000 ) ;
		else {
			//mainGLBox->setCPUWarning(true) ;
			if (autoSchedulerStep) { ////// INCREASE SCHEDULER STEP //////
				if ( gPrecision == (int)floor(gPrecision*(1.4)) )
					gPrecision += 2 ;
				else
					gPrecision = (int)floor( gPrecision*(1.4) )  ;
				//mainGLBox->setCPUTimeStep( gPrecision ) ;
			}
		}
		
		//tt2 = clock.currentTime() ;
		//printf("time : %d\n", tt2.msecsTo(tt0)  ) ;
		//////////////////////////////////////////////////////////////////////////////////////////
		if(loopCount == 50) {
			loopCount = 0 ;
			if (timeLeft>2 && gPrecision>1 && autoSchedulerStep) { ////// REDUCE SCHEDULER STEP //////
				if ( gPrecision == (int)floor( gPrecision*(0.9)) )
					gPrecision -= 1 ;
				else
					gPrecision = (int)floor( gPrecision*(0.9) )  ;
			}
			//mainGLBox->setCPUWarning(false) ;
			//mainGLBox->setCPUTimeStep( gPrecision ) ; // in ms
		}//////////////////////////////////////////////////////////////////////////////////////////
		
		loopCount++ ;
		
		sendSavedOSC() ;
		
		/////////// LIVE ADDING OBJECT TEST ///////////
		if(addTrigger) {
			addTrigger = false ;
			doAddNewTrigger() ;
		}
		if(addCurve) {
			addCurve = false ;
			doAddNewCurve() ;
			curveindexes = score->curvesIndexes() ;
		}
		////////////// LIVE ADDING END ////////////////
		
		tt2 = clock.currentTime() ;		
		dtime = (float)fabs( tt2.msecsTo(tt0) ) ;
		currentTime += (float)dtime * 0.001 ;
		if (finalTime!=0 && currentTime >= finalTime) {
			currentTime = finalTime ;
			stop() ;
		}
		//////////////////////////////////////////////////////////////////////////////////////////
	}
	
	////////////////////// OSC ENVIRONMENT LAST MESSAGE //////////////////////
	if (oscActive && sendInitialFinalMessage) sendOSCLastMessage() ;

	////////////////////////////////////////////////////////////////////
	
	printf("//////////////////////////////\n") ;
	printf("Playing finished !\n");
	printf("UDP messages sent : %d \n", nbPackets );
	
	//mainGLBox->setToolBarInfosTxt( QString(" ") ) ;
	//mainGLBox->setCPUWarning(false) ;
	
	delete(curvesOSCmF) ;
	delete(triggersOSCmF) ;
	//sTimer->stop() ;
	finished = false ;
	playing = false ;

	nbPackets = 0 ;
}
void NxScheduler::updateCursorPos( int i )
{
	bool forwCur ;	// forward or backward
	int typeCur ;	// single / repeat / fwd_back
	bool dir ;
	float initOffs, finalOffs ; // initial offset
	QList<int> curinds = score->cursorsIndexes() ;
	
	NxCursor* cur = score->getCursor( curinds.at(i) ) ;
	initOffs	= cur->getInitOffset() ;		// initial offset
	finalOffs	= cur->getFinalOffset() ;		// final offset
	forwCur		= cur->getForward() ;			// forward / backward
	typeCur		= cur->getMode() ;				// 0 / 1 / 2 : single / loop / b_f
	dir			= curDirections.at(i) ;			// direction of the cursor i at time t
	
	////// To allow direction inversion during playback
	if(cur->hasChangedDirection()) {
		curDirections.replace(i, !dir ) ;
		dir = !dir ;
		cur->setChangedDirection( false ) ;
	}
		
	float a = initOffs ;
	float b = finalOffs ;
	// If backward, offsets are calculated from the end of parent trajectory
	float inva = ((NxTraject*)cur->getParentTrajectory())->getLength() - b ;
	float invb = ((NxTraject*)cur->getParentTrajectory())->getLength() - a ;
	if (!forwCur) { a = inva ; b = invb ; } // Get real offsets bounds (if backward)
			
	bool tooMuch = false ;
	bool tooLess = false ;
	if ( cur->getOffset() >= b ) tooMuch = true ;
	if ( cur->getOffset() <= a ) tooLess = true ;
				
	if (mainTempo < 0.0) dir = !dir ;
		
	switch( typeCur ) {
		case 0 : // single
			if( dir )	advance( cur ) ;
			if( !dir )	regress( cur ) ;
			if ( dir && tooMuch )	{ cur->setOffset( b ) ; cursorsFinished.replace( i, true ) ; }
			if ( !dir && tooLess )	{ cur->setOffset( a ) ; cursorsFinished.replace( i, true ) ; }
		break;
		case 1 : // loop
			if( dir )	advance( cur ) ;
			if( !dir )	regress( cur ) ;
			if( dir && tooMuch )	{ cur->setOffset( a ) ; setFirstShot( true ) ; }
			if( !dir && tooLess )	{ cur->setOffset( b ) ; setFirstShot( true ) ; }
		break;											
		case 2 : // for/bck
			if( dir )	advance( cur ) ;
			if( !dir )	regress( cur ) ;		
			if( dir && tooMuch )	{ cur->setOffset( b ) ; curDirections.replace(i, !(mainTempo>=0.0) ) ; }
			if( !dir && tooLess )	{ cur->setOffset( a ) ; curDirections.replace(i, (mainTempo>=0.0) ) ; }
		break;				
	}
	if( (cur->getParentTrajectory()->getType() == 2) || typeCur!=1 ) { // non circle loop
		if( cur->getOffset() < a ) cur->setOffset( a ) ;
		if( cur->getOffset() > b ) cur->setOffset( b ) ;
	}
}
void NxScheduler::advance( NxCursor* cur ) 
{
	cur->setOffset( cur->getOffset() + ( 0.001 * dtime * cur->getSpeed() * fabs(mainTempo) ) ) ;
}

void NxScheduler::regress( NxCursor* cur) 
{
	cur->setOffset( cur->getOffset() - ( 0.001 * dtime * cur->getSpeed() * fabs(mainTempo) ) ) ;
}
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
void NxScheduler::nextStep() ////// UNUSED
{
	mutex.lock() ;
	//currentTime += dtime;
	next = true ;
	mutex.unlock() ;
	//printf("nextstep : time : %f\n", currentTime) ;
}

//////////////////////////// Stopping Method
void NxScheduler::stop()
{
	mutex.lock();
	finished = true ;
	mutex.unlock();
}
/////////////////////////////////////////////////////////////////////////////
/////////////////////////// TRIGGERS COLLISION TEST /////////////////////////
/////////////////////////////////////////////////////////////////////////////
void NxScheduler::triggersCollisionTest( int i ) // For one cursor
{	
	NxCursor 	*cursor ;
	NxTrigger 	*trigger ; 
	NxTraject	*traj ;
	
	NxPoint trigP ;		// trigger position
	NxPoint cursP ;		// cursor position
	
	int layer ;			// Interactions only if objects in the same layer
	
	float wid, ang ;	// width and angle of the cursor
	float cosV, sinV ;	// projections of object center on the cursor
	float off ;			// offset
	float distT	;		// distance between cursor and trigger
	
	QList<int> cursorindexes	= score->cursorsIndexes();
	QList<int> curveindexes 	= score->curvesIndexes();
	QList<int> triggerindexes 	= score->triggersIndexes();
	
	cursor = score->getCursor( cursorindexes.at(i) );
	layer = cursor->getLayer() ;
	if( cursor->isActive() ) { // only if cursor is active
		ang		= ( cursor->getAngle() )*PI/180.0 ;	// get angle in radians
		wid		= cursor->getWidth() ;				// get width
		traj	= cursor->getParentTrajectory() ;		// get parent
		off		= cursor->getOffset() ;				// get offset
		
		if(traj->getType() == 2) { /////////////  Line
			cursP.x = traj->getPosition().x + off*cos(ang) ;
			cursP.y = traj->getPosition().y + off*sin(ang) ;
		}
		else ///////////////////////////////////  Circle
		{
			float rd = traj->getCircle().radius ;
			cursP.x = traj->getPosition().x +rd*cos(ang) ; 
			cursP.y = traj->getPosition().y +rd*sin(ang) ;
			ang += PI/2.0 ; // get the real angle of the cursor
		}
			
		//////////////////////////////////////////////////////////
		//////////////////////// TRIGGERS ////////////////////////
		//////////----------------------------------------////////
		/////// arrays of triggers booleans
		//printf("debut1\n") ;
		// crash 1 ici
		QList<bool> tmpTBool1 = triggersSign1.at(i) ; // array of cursor i
		QList<bool> tmpTBool2 = triggersSign2.at(i) ; // array of cursor i
		//printf("debut2\n") ;
		/////// Trigger Loop
		for (int j=0; j<nbTriggers; j++) {				
			trigger = score->getTrigger( triggerindexes.at(j) ) ;	
			if( (trigger->isActive()) && (trigger->getLayer()==layer) ) { // only if in the same layer and active
				trigP = trigger->getPosition() ; // new collision test
				//printf("-++- TRIGGER ID %d\n", trigP.ID ) ;
				//////// COS projection /////////////////////////////////////////////////
				// vector C :	X    - sin(ang)		////////// (normalized)
				//				Y    cos(ang)		////////// (normalized)
				// vector T :	X    trigP.x - cursP.x
				//				Y    trigP.y - cursP.y
				//printf("/////////////////////////////////////////////////////////////\n") ;
				//printf("Positions: trigg %f %f curs: %f %f\n", trigP.x, trigP.y, cursP.x, cursP.y ) ; 
				//printf("Angle: %f\n", ang ) ;
				float c1, c2, t1, t2 ;
				c1 = - sin(ang) ;
				c2 = cos(ang) ;
				t1 = trigP.x - cursP.x ;
				t2 = trigP.y - cursP.y ;
				cosV = (c1*t1 + c2*t2 ) ; //---------// distance to the traject (perpendicular to cursor direction)
		
				distT = sqrt(t1*t1 + t2*t2)  ; //---------// distance between cursor center and trigger
				//printf("distance : %f\n",distT ) ; 
				//printf("cursor c1: %f c2: %f\n", c1, c2 ) ; 
				//printf("trigger t1: %f t2: %f\n", t1, t2 ) ; 
				//printf("Distance to the traject = %f \n", cosV) ;
				///////////////////////////////////////
				if( fabs(cosV) <=  (wid/2) ) { // only if trigger 'viewable'
					//////// SIN projection ///////////////////
					sinV = (c1*t2 - c2*t1) ; // (wid/2) ; //---------// distance to the cursor
					//////////////////////////////////////////
					//printf("Distance to the cursor = %f \n\n", sinV) ;
					//printf("midle1\n") ;
					if(sinV>0) tmpTBool1.replace( j, true ) ; // SIN projection
					else tmpTBool1.replace( j, false ) ;
					//printf("midle2\n") ;
					/////////// Collision Test - need to test :
					// sign change
					// avoid first shot banging
					// avoid circles problem
					if( ( tmpTBool1.at(j) != tmpTBool2.at(j) ) && !triggersFirstShot.at(j) && (distT<=wid/2) ) {	
						///////////////////////////////////////////////////////////////////////////// COLLISION !!!!
						//// SEND ////////
						//sendTriggerOSC( cursor, trigger ) ;
						triggerOSC mess ;
						mess.host		= trigger->getHost() ;
						mess.port		= trigger->getPort() ;
						mess.cursGID	= cursor->getGroupID() ;
						mess.cursID		= cursor->getCustomID() ;
						mess.GID		= trigger->getGroupID() ;
						mess.ID			= trigger->getCustomID() ;
						mess.val		= trigger->getValue() ;
						triggersMessages.append( mess ) ;

					} // end if() -> collision
					tmpTBool2.replace( j, tmpTBool1.at(j) ) ;
					//printf("after1\n") ;
					/// crash 2 ici
					triggersSign1.replace( i , tmpTBool1 ) ;
					//printf("after2\n") ;
					triggersSign2.replace( i , tmpTBool2 ) ;
					//printf("after3\n") ;
				} // end if( trigger viewable )	
			} // end if( active and same layer )
		} // end for ( triggers )
	} // end if ( cursor active and same layer)
}
////////////////////////////////////////////////////////////////////////
////////////////     TRIGGERS COLLISION TEST END       /////////////////
////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////
////////////////     CURVES COLLISION TEST      ////////////////////////
////////////////////////////////////////////////////////////////////////
void NxScheduler::curvesCollisionTest( int i, int k ) // For one cursor
{	
	NxCursor	*cursor ;
	NxCurve		*curve ;
	NxSpline	*spline ;
	NxTraject	*traj ;
	
	NxPoint curvP ;		// curve position
	NxPoint cursP ;		// cursor position
	
	int layer ; // Interactions only if objects in the same layer
	
	float wid, ang ;	// width and angle of the cursor
	float cosV, sinV ;	// projections of object center on the cursor
	float off ;			// offset
	float distC ;		// distance between cursor and curve
	
	float solx, soly, a1, a2, b1, b2 ; // local variables for algo 2
	
	QList<int> cursorindexes	= score->cursorsIndexes();
	QList<int> curveindexes 	= score->curvesIndexes();
	QList<int> triggerindexes 	= score->triggersIndexes();
	
	cursor = score->getCursor( cursorindexes.at(i) );
	layer = cursor->getLayer() ;
	if( cursor->isActive() ) { // only if cursor is active
		ang		= ( cursor->getAngle() )*PI/180.0 ;	// get angle in radians
		wid		= cursor->getWidth() ;				// get width
		traj	= cursor->getParentTrajectory() ;	// get parent
		off		= cursor->getOffset() ;				// get offset
		
		if(traj->getType() == 2) { /////////////  Line
			cursP.x = traj->getPosition().x + off*cos(ang) ;
			cursP.y = traj->getPosition().y + off*sin(ang) ;
		}
		else ///////////////////////////////////  Circle
		{
			float rd = traj->getCircle().radius ;
			cursP.x = traj->getPosition().x +rd*cos(ang) ; 
			cursP.y = traj->getPosition().y +rd*sin(ang) ;
			ang += PI/2 ; // get the real angle of the cursor
		}

		if(useCirclesAlgo) {
			//////////////////////////////////////////////////////////	
			//////////////////////// CURVES ALGO 1 ///////////////////
			///////////-----------------------------------////////////	
			/////// arrays for curves collision test
			QList <float> tmpDurations	= curvesDurations.at(i) ;	/////// collision durations	
			QList <float> tmpTimers		= curvesTimers.at(i) ;			/////// local timers
			QList <bool> tmpCollisions	= curvesCollisions.at(i) ;	/////// in-collision booleans		
			////// Curves Loop
			//for (int k=0; k<curveindexes.count(); k++) {
				float splineVal, splineAngle ;
				splineVal = 0 ; // initialisation
				
				curve 	= score->getCurve( curveindexes.at(k) ) ;
				if( (curve->isActive()) && (curve->getLayer()==layer) ) { // only if same layer and active
					curvP = curve->getCenter() ;
					float cRadius = curve->getRadius() ;
		
					if( tmpCollisions.at(k) == false ) { //----------------------// test the collision 
						//printf("/////// Curve Radius : %f \n", cRadius) ;
						//////// COS projection /////////////////////////////////////////////////
						// vector C :	X    - sin(ang)		////////// (normalized)
						//				Y    cos(ang)		////////// (normalized)
						// vector T :	X    curvP.x - cursP.x
						//				Y    curvP.y - cursP.y
						//printf("/////////////////////////////////////////////////////////////\n") ;
						//printf("Positions: curv %f %f curs: %f %f\n", curvP.x, curvP.y, cursP.x, cursP.y ) ; 
						//printf("Angle: %f\n", ang ) ;
						float c1, c2, t1, t2 ;
						c1 = - sin(ang) ;
						c2 = cos(ang) ;
						t1 = curvP.x - cursP.x ;
						t2 = curvP.y - cursP.y ;
						cosV = (c1*t1 + c2*t2 ) ; //---------// distance to the traject (perpendicular to cursor direction)
			
						distC = sqrt(t1*t1 + t2*t2)  ; // distance between cursor center and curve center
						//printf("distance : %f\n",distC ) ; 
						//printf("cursor c1: %f c2: %f\n", c1, c2 ) ; 
						//printf("curve t1: %f t2: %f\n", t1, t2 ) ; 
						//printf("Distance to the traject = %f \n", cosV) ;
						///////////////////////////////////////
						if( fabs(cosV) <=  ((wid/2) + cRadius) ) { // only if the curve circle is "viewable" ...
							sinV = (c1*t2 - c2*t1) ; //---------// distance to the cursor
							//printf("Distance to the cursor = %f \n", sinV) ;
							float d ;
							if( fabs(cosV) > (wid/2) ) { //// the circle is not read entirely, we have to calculate the real duration
								///////////////// Calculate duration...
								//		(cosV-(wid/2))^2 + (duration/2)^2 = cRadius^2
								//		d = duration/2
								d = sqrt( pow(cRadius, 2) - pow(fabs(cosV)-(wid/2), 2) ) ;
							}
							else d = cRadius ;
							//printf("Real duration of event : %f\n", 2*d) ;
				
							if( distC < ((wid/2)+cRadius) ) { // test only if not too far one from another
								//if( (sinV>0.0) && ((sinV-d)>0) && collisions.at(k) )	
								//if( (sinV>0.0) && ((sinV-d)>0) && !collisions.at(k) )	
								//if( (sinV>0.0) && ((sinV-d)<0) && collisions.at(k) )	
								//if( (sinV<0.0) && ((sinV+d)<0) && collisions.at(k) )		
								//if( (sinV<0.0) && ((sinV+d)<0) && !collisions.at(k) )								
								//if( (sinV<0.0) && ((sinV+d)>0) && collisions.at(k) )	
								if( firstShot && (fabs(sinV) < cRadius) ) { // cursor already int the curve circle when playing
									//printf("Already in the Curve Circle !!\n") ;
									tmpCollisions.replace( k , true ) ; 
									tmpTimers.replace( k , cRadius + sinV ) ; /// TO DO : depends on cursor position
									float realDur = cRadius + sqrt( pow( cRadius ,2) - pow( fabs(cosV) - wid/2 ,2) ) ;
									//printf("TimeIn of event (already in circle) = %f\n", cRadius + sinV ) ;
									//printf("TimeOut of event (already in circle) = %f\n", realDur ) ;
									tmpDurations.replace( k , realDur  ) ; 						
								}
								else { // To simplify...
									if( (sinV>0.0) && ((sinV-d)<0) && !tmpCollisions.at(k) ) { 
										tmpCollisions.replace( k , true ) ; 
										tmpTimers.replace( k , cRadius - d ) ; 
										tmpDurations.replace( k , cRadius + d ) ; 
									}
									if( (sinV<0.0) && ((sinV+d)>0) && !tmpCollisions.at(k) ) {	
										tmpCollisions.replace( k , true ) ; 
										tmpTimers.replace( k , cRadius - d ) ; 
										tmpDurations.replace( k , cRadius + d ) ; 
									}
								}
							} // end if( distC < ... )			
						} /// end if( 'viewable' )
					} /// end if( collision=false )
	
					if( tmpCollisions.at(k) == true ) { //----------------------// Already in collision
						///////////////////////////////////////////////////////////////////////////////////// COLLISION !!!!!		
						////// get curve value
						if(curve->getDrawType() == LineStrip) { /////----- LINES -----//////
							splineVal = curve->getLinearValue( tmpTimers.at(k) ) ;
						}
						if(curve->getDrawType() == SplineStrip) { /////----- SPLINES -----///////
							spline = curve->getSpline() ;
							splineVal = (float)spline->getValue( tmpTimers.at(k) );
						}
						splineAngle = curve->getAngle() - cursor->getAngle() ; ////// get real angle in degrees

						curveOSC mess ;
						mess.host		= curve->getHost() ;
						mess.port		= curve->getPort() ;
						mess.cursGID	= cursor->getGroupID() ;
						mess.cursID		= cursor->getCustomID() ;
						mess.GID		= curve->getGroupID() ;
						mess.ID			= curve->getCustomID() ;
						mess.angle		= splineAngle ;
						mess.val		= splineVal ;
						curvesMessages.append( mess ) ;
						if(curve->getDrawType() == SplineStrip) spline->reset() ;
												
						//-------/// TO DO WARNING : it is not the same timeStep with circle trajects ....
						//float ddtimer = tmpTimers.at(k) + 0.001 * dtime * cursor->getSpeed() * fabs(mainTempo) ; 
						float ddtimer = tmpTimers.at(k) + curve->getSpatialPeriod() * 0.001 ; 
						tmpTimers.replace( k ,  ddtimer  ) ; ///// advance local time
						//printf( "Timer Value : %f -- Max duration : %f\n", tmpTimers.at(k) , tmpDurations.at(k));
						if( tmpTimers.at(k) > tmpDurations.at(k) ) { //// end of collision
							tmpCollisions.replace( k , false ) ;
							if(curve->getDrawType() == SplineStrip) spline->reset() ;
							tmpTimers.replace( k , 0.0 ) ;
							tmpDurations.replace( k , 999.0 ) ;
						}
					} /// end if( collision=true )
					
					curvesDurations.replace( i , tmpDurations ) ;
					curvesTimers.replace( i , tmpTimers ) ;
					curvesCollisions.replace( i , tmpCollisions ) ;
				} // end if (active and same layer )
			//} /// end for( curves ) 
		//////////////////////////////////// END ALGO 1
		} // end algo1
		else {
			//////////////////////////////////////////////////////////	
			//////////////////////// CURVES ALGO 2 ///////////////////
			///////////-----------------------------------////////////		
			//for (int k=0; k<curveindexes.count(); k++) {
				float splineVal, splineAngle ;
				splineVal = 0 ; // initialisation
				
				NxSpline *spline ;
				curve = score->getCurve( curveindexes.at(k) ) ;
				if( (curve->isActive()) && (curve->getLayer()==layer) ) {
					bool collisionIntersect = false ; 
					bool paralel = false ;
					bool curvVertical = false ;
					bool cursVertical = false ;
					float localTime ;
					curvP = curve->getCenter() ;
					float cRadius = curve->getXRange()/2 ;
					//printf("cRadius is : %f\n", cRadius) ;
					float cAngle = curve->getAngle()*PI/180.0 ; // in radians
					ang += PI/2 ; // 'ang' is the real angle of the cursor in radians
					if( sqrt( pow(cursP.x-curvP.x,2) + pow(cursP.y-curvP.y,2) ) <= (cRadius+wid/2) ) { // test only if centers are not too far one from another ...
						//a1 = a2 = b1 = b2 = 0.0 ;
						a1 = 0 ; // Initialisation (avoid automatic paralell)
						a2 = 1 ; // Initialisation (avoid automatic paralell)
						if( fabs(cos(cAngle))>0.01 ) { // avoid Curve vertical direction
							a1 = sin(cAngle) / cos(cAngle) ;
							b1 = curvP.y - a1*curvP.x ;
							}
						else curvVertical = true ;
						if(fabs(cos(ang))>0.01 ) { // avoid Cursor vertical direction
							a2 = sin(ang) / cos(ang) ; 
							b2 = cursP.y - a2*cursP.x ;
							}
						else cursVertical = true ;
						if(curvVertical) {
							solx = curvP.x ;
							soly = a2*solx + b2 ;
							//printf("Curve is vertical !\n") ;
							}						
						if(cursVertical) {
							solx = cursP.x ;
							soly = a1*solx + b1 ;
							//printf("Cursor is vertical !\n") ;
							}
						if( ( a1==a2 && !(cursVertical || curvVertical) ) || (cursVertical && curvVertical) ) { 
							paralel = true ;
							//printf("Parallel !\n") ;
							}
						//printf("angles : curve %f cursor : %f \n", cAngle*180.0/PI,  ang*180.0/PI ) ;
						//printf("a1 a2 : %f %f\n", a1, a2) ;
						//printf("b1 b2 : %f %f\n", b1, b2) ;
						////////// now we solve the intersection :
						if( !paralel && !cursVertical && !curvVertical) { // avoid paralellism and vertical direction !
							solx = (b2-b1)/(a1-a2) ;
							soly = a1*solx + b1 ;
						}
						//printf("Intersection calculated : %f et %f\n", solx, soly) ;
						
						// Test if solution is on the segments (cursor and curve segment)
						if(	!paralel && ( sqrt( pow(solx-curvP.x,2) + pow(soly-curvP.y,2) )<=cRadius ) && ( sqrt( pow(solx-cursP.x,2) + pow(soly-cursP.y,2) )<=wid/2 )) {
							//printf("Collision!!\n") ;
							collisionIntersect = true ;
							NxPoint initPos = curve->getCenter() ;
							initPos.x -= cRadius*cos(cAngle) ;
							initPos.y -= cRadius*sin(cAngle) ;
							localTime = sqrt( pow(solx-initPos.x,2) + pow(soly-initPos.y,2) ) ;
						}
					
						if( !paralel && collisionIntersect ) { //----------------------// 
							///////////////////////////////////////////////////////////////////////////////////// COLLISION !!!!!		
							////// Get curve value
							if(curve->getDrawType() == LineStrip) { /////----- LINES -----//////
								splineVal = curve->getLinearValue( localTime ) ;
							}
							if(curve->getDrawType() == SplineStrip) { /////----- SPLINES -----/////
								spline = curve->getSpline() ;
								splineVal = spline->getValue( localTime );
							}
							splineAngle = curve->getAngle() - cursor->getAngle() ; ////// get real angle in degrees
							
							curveOSC mess ;
							mess.host		= curve->getHost() ;
							mess.port		= curve->getPort() ;
							mess.cursGID	= cursor->getGroupID() ;
							mess.cursID		= cursor->getCustomID() ;
							mess.GID		= curve->getGroupID() ;
							mess.ID			= curve->getCustomID() ;
							mess.angle		= splineAngle ;
							mess.val		= splineVal ;
							curvesMessages.append( mess ) ;
							if( curve->getDrawType() == SplineStrip ) curve->getSpline()->reset() ;	
							
						} // end if( paralel && collision ) (collision) 
					} // end if (not too far from onother)
				} // end if (active and same layer)
			//} /// end for(curves)
		} // and else[ ] (algo 2)
	} // end if (cursor active)
}
////////////////////////////////////////////////////////////////////////
////////////////     CURVES COLLISION TEST END       ///////////////////
////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////
////////////////     OSC SEND FONCTIONS       ///////////////////////
/////////////////////////////////////////////////////////////////////
void NxScheduler::updateSendingOptions()
{
	NxEnv tmpEnv = ((NxScore*)mainGLBox->getPartition())->getEnvironment() ;
	scCID	= tmpEnv.scCID ;
	scCGID	= tmpEnv.scCGID ;
	scID	= tmpEnv.scID ;
	scGID	= tmpEnv.scGID ;
	scA		= tmpEnv.scA ;
	scV		= tmpEnv.scV ;
	stCID	= tmpEnv.stCID ;
	stCGID	= tmpEnv.stCGID ;
	stID	= tmpEnv.stID ;
	stGID	= tmpEnv.stGID ;
	stV		= tmpEnv.stV ;
}
void NxScheduler::sendSavedOSC()
{
	for (int j=0; j< triggersMessages.size(); j++)
		sendTriggerOSC( triggersMessages.at(j) ) ;
	for (int j=0; j< curvesMessages.size(); j++)
		sendCurveOSC( curvesMessages.at(j) ) ;

	resetOSC() ;
}
void NxScheduler::sendCurveOSC( curveOSC mess ) 
{
	QString cC( "," ) ;
	if(scCGID)	cC.append("i") ;
	if(scCID)	cC.append("i") ;
	if(scGID)	cC.append("i") ;
	if(scID)	cC.append("i") ;
	if(scA)		cC.append("f") ;
	if(scV)		cC.append("f") ;
	char curvChar[ cC.size()+1 ];
	strncpy(curvChar, cC.toAscii().data(), sizeof(curvChar)-1);
	curvChar[sizeof(curvChar)-1] = '\0' ;
	
	OSC_writeAddressAndTypes(b, curvesOSCmF, curvChar) ;
	if(scCGID)	OSC_writeIntArg( b, mess.cursGID );		/// OSC Argument 1 - Cursor Group ID
	if(scCID)	OSC_writeIntArg( b, mess.cursID );		/// OSC Argument 2 - Cursor Custom ID
	if(scGID)	OSC_writeIntArg( b, mess.GID );			/// OSC Argument 3 - Curve Group ID
	if(scID)	OSC_writeIntArg( b, mess.ID );			/// OSC Argument 4 - Curve Custom ID
	if(scA)		OSC_writeFloatArg( b, mess.angle );		/// OSC Argument 5 - Curve Angle
	if(scV)		OSC_writeFloatArg( b, mess.val );		/// OSC Argument 6 - Curve Value
	curveClientUDP = new NxNetwork( mess.host, mess.port );
	curveClientUDP->sendPacket(OSC_getPacket(b), OSC_packetSize(b), broadcastMode);
	//printf("//////////////////////////////////////////////////////\n") ;
	//printf("Collision CURVE // ID %d (%d) // val %f\n", mess.ID, mess.cursID, mess.val);
	OSC_resetBuffer( b );
	delete(curveClientUDP) ;
	nbPackets++;
}
void NxScheduler::sendTriggerOSC( triggerOSC mess ) 
{
	QString tC( "," ) ;
	if(stCGID)	tC.append("i") ;
	if(stCID)	tC.append("i") ;
	if(stGID)	tC.append("i") ;
	if(stID)	tC.append("i") ;
	if(stV)		tC.append("i") ;
	char trigChar[ tC.size()+1 ];
	strncpy(trigChar, tC.toAscii().data(), sizeof(trigChar)-1);
	trigChar[sizeof(trigChar)-1] = '\0' ;
	
	OSC_writeAddressAndTypes(b, triggersOSCmF , trigChar);
	if(stCGID)	OSC_writeIntArg( b, mess.cursGID );		/// OSC Argument 1 - Cursor Group ID
	if(stCID)	OSC_writeIntArg( b, mess.cursID );		/// OSC Argument 2 - Cursor Custom ID
	if(stGID)	OSC_writeIntArg( b, mess.GID );			/// OSC Argument 3 - Trigger Group ID
	if(stID)	OSC_writeIntArg( b, mess.ID );			/// OSC Argument 4 - Trigger Custom ID
	if(stV)		OSC_writeIntArg( b, mess.val );			/// OSC Argument 5 - Trigger Int Value
	triggerClientUDP = new NxNetwork( mess.host, mess.port );
	//printf("////////////////////////////////////////////////////\n") ;
	//printf("Collision TRIGGER %d : ID:%d / Val:%d / Cursor:%d\n", nbPackets, trigger->getCustomID(), trigger->getValue(), cursor->getCustomID() );
	triggerClientUDP->sendPacket(OSC_getPacket(b), OSC_packetSize(b), broadcastMode);
	OSC_resetBuffer( b );
	delete(triggerClientUDP) ;
	nbPackets++;
}

/////////////////////////////////////////////////////////////////////
// Function for displaying OSC packets
/////////////////////////////////////////////////////////////////////
void NxScheduler::PrintPacket(OSCbuf *b) 
{
	char *p = OSC_getPacket(b);
	int size = OSC_packetSize(b);
	unsigned int *intp;
	int i;

	printf("PrintPacket: packet at %p, size %d\n", p, size);
	if (p == 0 || size == 0) return;

	printf("Hex version:");
	for (i = 0, intp = (unsigned int *)p; i < size; i += 4, intp++) 
	{
		if (i % 40 == 0) printf("\n");
		printf("%x ", *intp);
	}

	printf("\n\nString version:");
	for (i = 0; i < size; i++) 
	{
		if (i % 40 == 0) printf("\n");
		if (isprint(p[i])) 
			printf("%c", p[i]); 
		else 
			printf("\\%x", p[i] & 0x000000ff);
	}
	printf("\n");
}
