///////////////
// IanniX
///////////////

////////////////////////////
// NxControlCenter.cpp
////////////////////////////

#include "NxControlCenter.h"
#include <cstdlib>

///////////////////////////////////
// Main Constructor
///////////////////////////////////
NxControlCenter::NxControlCenter( NxGLScore* parent )
  : QDialog( parent )
{
	//QFont f( "Lucida Grande", 13);
	//setFont( f );
	
	backgColor = QColor(0,0,0,3) ;
  	
	score			= parent->getPartition();
  	parentGLBox 	= parent;
	
	nbCurves		= score->curvesIndexes().count();
	nbTrajects		= score->trajectsIndexes().count();
	nbTriggers		= score->triggersIndexes().count();
	nbCursors		= score->cursorsIndexes().count();
		
	// Widgets
  	controlBox 		= new QTabWidget;
  	curvePage 		= new QWidget ;
  	curveList 		= new QTableWidget( nbCurves, 11, curvePage );
  	trajectPage 	= new QWidget ;
  	trajectList 	= new QTableWidget( nbTrajects, 7, trajectPage );
  	triggerPage 	= new QWidget ;
  	triggerList 	= new QTableWidget( nbTriggers, 8, triggerPage );
  	cursorPage 		= new QWidget ;
  	cursorList 		= new QTableWidget( nbCursors, 10, cursorPage );

	// Settings
  	updateIndexes();
  	controlBox->setCurrentIndex( 0 );
	
  	// Adding Sections
  	controlBox->addTab( curvePage, "Curves" );
  	controlBox->addTab( trajectPage, "Trajectories" );
  	controlBox->addTab( cursorPage, "Cursors" );
	controlBox->addTab( triggerPage, "Triggers" );
 
	this->setTabOrder( curvePage, trajectPage ) ;
	this->setTabOrder( trajectPage, triggerPage ) ;
	this->setTabOrder( triggerPage, cursorPage ) ;
	this->setTabOrder( cursorPage, curvePage ) ;
	
	// First time filling tables
	addItems() ;
	updateIndexes() ;
	
	// Initializing signals
	connect( curveList,  	SIGNAL(cellChanged(int, int)), this, SLOT(updateCurve(int, int)) );
	connect( trajectList,  	SIGNAL(cellChanged(int, int)), this, SLOT(updateTrajectory(int, int)) );
	connect( cursorList,  	SIGNAL(cellChanged(int, int)), this, SLOT(updateCursor(int, int)) );
	connect( triggerList,  	SIGNAL(cellChanged(int, int)), this, SLOT(updateTrigger(int, int)) );
	
  	// Layout	
  	mainLayout = new QVBoxLayout( this );
  	mainLayout->addWidget( controlBox );
	
	// Sizes
  	//controlBox->setMaximumSize( 860, 450 );
	cursorList->setMinimumSize( 920, 610 );
	curveList->setMinimumSize( 920, 610 );
	trajectList->setMinimumSize( 920, 610 );
	triggerList->setMinimumSize( 920, 610 );
	// Labels
	curveList->setHorizontalHeaderLabels( QStringList() << "ID" << "Name" << "Custom ID" << "Group ID" 
														<< "Layer" << "Duration" << "Style" << "Real Min" 
														<< "Real Max" << "Host IP" << "UDP Port" );
	for(int n = 0; n<11; n++) {
		if(n!=6) curveList->resizeColumnToContents(n) ;
	}
	trajectList->setHorizontalHeaderLabels( QStringList() << "ID" << "Name" << "Custom ID" << "Group ID" << "Layer"
														  << "Type" << "Length" ); 
	for(int n = 0; n<7; n++) {
		trajectList->resizeColumnToContents(n) ;
	}
	triggerList->setHorizontalHeaderLabels( QStringList() << "ID" << "Name" << "Custom ID" << "Group ID" << "Layer"
														  << "Host IP" << "UDP Port" << "Value" );
	for(int n = 0; n<8; n++) {
		triggerList->resizeColumnToContents(n) ;
	}													  
	cursorList->setHorizontalHeaderLabels( QStringList() << "ID" << "Name" << "Custom ID" << 
															"Group ID" << "Layer" << "Parent ID" << 
															"Speed" << "Width" << "Playback" << "Mode" );
	for(int n = 0; n<8; n++) {
		cursorList->resizeColumnToContents(n) ;
	}														
  	// Global settings
  	languageChange();
  	resize( QSize( 950, 660 ).expandedTo(minimumSizeHint()) );
  	setMaximumSize( 950, 660 );
	
	//setWindowModality( Qt::ApplicationModal ) ;
}

//////////////////////
// Destructor
//////////////////////
NxControlCenter::~NxControlCenter()
{
	// Done by Qt...
}

////////////////////////////////////////
// Item setting method (OPEN EXISTING SCORE)
////////////////////////////////////////
void NxControlCenter::addItems()
{
  	QString strings;
		
	//////////////////////////////// CURVES
  	QList<int> indexes = score->curvesIndexes();
	for(int i=0;i<indexes.count(); i++)
	{
		NxCurve *curve = score->getCurve( indexes.at(i) );
		
		strings = strings.setNum( curve->getID() ) ;
		curveList->setItem( i, 0, new QTableWidgetItem(strings) );
		curveList->item(i,0)->setBackgroundColor(backgColor) ;
		curveList->item(i,0)->setFlags(!Qt::ItemIsEditable) ;
		
		strings = curve->getName();
		curveList->setItem( i, 1, new QTableWidgetItem(strings) );
		
		strings = strings.setNum( curve->getCustomID() );
		curveList->setItem( i, 2, new QTableWidgetItem(strings) );
		
		strings = strings.setNum( curve->getGroupID() );
		curveList->setItem( i, 3, new QTableWidgetItem(strings) );
		
		strings = strings.setNum( curve->getLayer() );
		curveList->setItem( i, 4, new QTableWidgetItem(strings) );		
		
		strings = strings.setNum( curve->getRadius() * 2.0, 'f', 4 );
		curveList->setItem( i, 5, new QTableWidgetItem(strings) );
		curveList->item(i,5)->setBackgroundColor(backgColor) ;
		curveList->item(i,5)->setFlags(!Qt::ItemIsEditable) ;			
					
		QStringList strList;
		strList << "Lines" << "Splines";
	
		QComboBox *combo = new QComboBox;
		combo->addItems(strList);
		if ( curve->getDrawType() == LineStrip )
			combo->setCurrentIndex(0);
		else
			combo->setCurrentIndex(1);
		curveList->setCellWidget( i, 6, combo );
		connect( combo, SIGNAL(currentIndexChanged(int)), this, SLOT(updateCurvesDrawTypes()) ) ;
		
		strings = strings.setNum( curve->getRealMin() );
		curveList->setItem( i, 7, new QTableWidgetItem(strings) );
		
		strings = strings.setNum( curve->getRealMax() );
  		curveList->setItem( i, 8, new QTableWidgetItem(strings) );
		
		strings = curve->getHost() ;
  		curveList->setItem( i, 9, new QTableWidgetItem(strings) );
		
		strings = strings.setNum( curve->getPort() );
  		curveList->setItem( i, 10, new QTableWidgetItem(strings) );
		
		curveList->resizeRowToContents(i) ;
	}
	
	//////////////////////////////// TRAJECTS
	float len = 0.0;
	indexes.clear();
	indexes = score->trajectsIndexes();
	for(int i=0; i<indexes.count(); i++)
	{
		NxTraject *traject = score->getTraject( indexes.at(i) );
		
		strings = strings.setNum( traject->getID() ) ;
		trajectList->setItem( i, 0, new QTableWidgetItem(strings) );		
		trajectList->item(i,0)->setBackgroundColor(backgColor) ;
		trajectList->item(i,0)->setFlags(!Qt::ItemIsEditable) ;
				
		strings = traject->getName();
		trajectList->setItem( i, 1, new QTableWidgetItem(strings) );
		
		strings = strings.setNum( traject->getCustomID() );
		trajectList->setItem( i, 2, new QTableWidgetItem(strings) );
		
		strings = strings.setNum( traject->getGroupID() );
		trajectList->setItem( i, 3, new QTableWidgetItem(strings) );
		
		strings = strings.setNum( traject->getLayer() );
		trajectList->setItem( i, 4, new QTableWidgetItem(strings) );
		
		switch( traject->getType() ) {
			case 1 : // circle
				strings = "Circle";
			break;
			case 2 : // line
				strings = "Line";
			break;
		}
		len = traject->getLength() ;
		trajectList->setItem( i, 5, new QTableWidgetItem(strings) );
		strings = strings.setNum( len, 'f', 4 );
		trajectList->setItem( i, 6, new QTableWidgetItem(strings) );		
		trajectList->item(i,5)->setBackgroundColor(backgColor) ;
		trajectList->item(i,6)->setBackgroundColor(backgColor) ;
		trajectList->item(i,5)->setFlags(!Qt::ItemIsEditable) ;
		trajectList->item(i,6)->setFlags(!Qt::ItemIsEditable) ;
		
		trajectList->resizeRowToContents(i) ;
	}
	
	//////////////////////////////// TRIGGERS
	indexes.clear();
	indexes = score->triggersIndexes() ;	
  	for(int i=0; i<indexes.count(); i++)
	{
		NxTrigger *trigger = score->getTrigger( indexes.at(i) );
		
		strings = strings.setNum( trigger->getID() ) ;
		triggerList->setItem( i, 0, new QTableWidgetItem(strings) );
		triggerList->item(i, 0)->setBackgroundColor(backgColor) ;
		triggerList->item(i, 0)->setFlags(!Qt::ItemIsEditable) ;
		
		strings = trigger->getName();
		triggerList->setItem( i, 1, new QTableWidgetItem(strings) );
		
		strings = strings.setNum( trigger->getCustomID() );
		triggerList->setItem( i, 2, new QTableWidgetItem(strings) );
		
		strings = strings.setNum( trigger->getGroupID() );
		triggerList->setItem( i, 3, new QTableWidgetItem(strings) );
		
		strings = strings.setNum( trigger->getLayer() );
		triggerList->setItem( i, 4, new QTableWidgetItem(strings) );
		
		strings = trigger->getHost() ;
		triggerList->setItem( i, 5, new QTableWidgetItem(strings) );		
		
		strings = strings.setNum( trigger->getPort() );
		triggerList->setItem( i, 6, new QTableWidgetItem(strings) );
		
		strings = strings.setNum( trigger->getValue() );
		triggerList->setItem( i, 7, new QTableWidgetItem(strings) );
		
		triggerList->resizeRowToContents(i) ;
	}
	
	//////////////////////////////// CURSORS
	indexes.clear();
	indexes = score->cursorsIndexes();
  	for(int i=0; i<indexes.count(); i++)
	{
		NxCursor *cursor = score->getCursor( indexes.at(i) );
		
		strings = strings.setNum( cursor->getID() ) ;
		cursorList->setItem( i, 0, new QTableWidgetItem(strings) );
		cursorList->item(i,0)->setBackgroundColor(backgColor) ;
		cursorList->item(i,0)->setFlags(!Qt::ItemIsEditable) ;
		
		strings = cursor->getName();
		cursorList->setItem( i, 1, new QTableWidgetItem(strings) );
		
		strings = strings.setNum( cursor->getCustomID() );
		cursorList->setItem( i, 2, new QTableWidgetItem(strings) );
		
		strings = strings.setNum( cursor->getGroupID() );
		cursorList->setItem( i, 3, new QTableWidgetItem(strings) );
		
		strings = strings.setNum( cursor->getLayer() );
		cursorList->setItem( i, 4, new QTableWidgetItem(strings) );
		
		strings = strings.setNum( cursor->getParentTrajectory()->getID() );
		cursorList->setItem( i, 5, new QTableWidgetItem(strings) );
		cursorList->item(i,5)->setBackgroundColor(backgColor) ;
		cursorList->item(i,5)->setFlags(!Qt::ItemIsEditable) ;
		
		strings = strings.setNum( cursor->getSpeed(), 'f', 4 );
		cursorList->setItem( i, 6, new QTableWidgetItem(strings) );
	
		strings = strings.setNum( cursor->getWidth(), 'f', 4 );
		cursorList->setItem( i, 7, new QTableWidgetItem(strings) );	
		
		// Playback
		QStringList strList2;
		strList2 << "Forward" << "Backward";
		QComboBox *combo2 = new QComboBox;
		combo2->addItems(strList2);
		if ( cursor->getForward() )
			combo2->setCurrentIndex(0);
		else
			combo2->setCurrentIndex(1);
		cursorList->setCellWidget( i, 8, combo2 );
		connect( combo2, SIGNAL(currentIndexChanged(int)), this, SLOT(updateCursorsBackward()) ) ;			
		
		// Mode
		QStringList strList3;
		strList3 << "Single" << "Repeat" << "Fwd/Bckwd" ;
		QComboBox *combo3 = new QComboBox;
		combo3->addItems(strList3);
		combo3->setCurrentIndex( (int)cursor->getMode() );
		cursorList->setCellWidget( i, 9, combo3 );	
		connect( combo3, SIGNAL(currentIndexChanged(int)), this, SLOT(updateCursorsMode()) ) ;	
		
		cursorList->resizeRowToContents(i) ;	
	}
}

void NxControlCenter::clearTable()
{
	cursorList->clear();
}


/////////////////////////////////////////////////////
// Dynamic additions to tables
/////////////////////////////////////////////////////
void NxControlCenter::updateItems( int choice, bool add, int inID )
{
	QString strings;
		
	score = parentGLBox->getPartition() ; // met a jour la partition en local
	updateIndexes() ; // pour mettre a jour les index locaux du Control Center (nbTriggers, etc...)
	
	switch (choice) 
	{
    	case Curve :
      		if (add)
			{
				NxCurve *tCurve = score->getCurve(inID);
	  			curveList->setRowCount( nbCurves );
				
				strings = strings.setNum( tCurve->getID() );
	  			curveList->setItem( nbCurves-1, 0, new QTableWidgetItem(strings) );	
				curveList->item(nbCurves-1,0)->setBackgroundColor(backgColor) ;			
				curveList->item(nbCurves-1,0)->setFlags(!Qt::ItemIsEditable) ;
				
				strings = tCurve->getName();
	  			curveList->setItem( nbCurves-1, 1, new QTableWidgetItem(strings) );
				
				strings = strings.setNum( tCurve->getCustomID() );
	  			curveList->setItem( nbCurves-1, 2, new QTableWidgetItem(strings) );
		
	  			strings = strings.setNum( tCurve->getGroupID() );
	  			curveList->setItem( nbCurves-1, 3, new QTableWidgetItem(strings) );
				
				strings = strings.setNum( tCurve->getLayer() );
	  			curveList->setItem( nbCurves-1, 4, new QTableWidgetItem(strings) );
				
				strings = strings.setNum( tCurve->getRadius() * 2.0, 'f', 4 );
				curveList->setItem( nbCurves-1, 5, new QTableWidgetItem(strings) );
				curveList->item(nbCurves-1,5)->setBackgroundColor(backgColor) ;
				curveList->item(nbCurves-1,5)->setFlags(!Qt::ItemIsEditable) ;
				
				QStringList strList;
				strList << "Lines" << "Splines";
				QComboBox *comboStyle = new QComboBox;
				comboStyle->addItems(strList);
				if ( (tCurve->getDrawType()) == LineStrip )
					comboStyle->setCurrentIndex(0);
				else
					comboStyle->setCurrentIndex(1);
				curveList->setCellWidget( nbCurves-1, 6, comboStyle );
				connect( comboStyle, SIGNAL(currentIndexChanged(int)), this, SLOT(updateCurvesDrawTypes()) ) ;
								
				strings = strings.setNum( tCurve->getRealMin() );
	     		curveList->setItem( nbCurves-1, 7, new QTableWidgetItem(strings) );
	
				strings = strings.setNum( tCurve->getRealMax() );
				curveList->setItem( nbCurves-1, 8, new QTableWidgetItem(strings) );
				
				strings = tCurve->getHost() ;
				curveList->setItem( nbCurves-1, 9, new QTableWidgetItem(strings) );
				
				strings = strings.setNum( tCurve->getPort() );
				curveList->setItem( nbCurves-1, 10, new QTableWidgetItem(strings) );
				
				curveList->resizeRowToContents( nbCurves-1 ) ;
			}
      		else
				for (int i=0; i<curveList->rowCount(); i++)
					if ( curveList->item(i, 0)->text().toInt() == inID )
						curveList->removeRow(i);
			break;
		
    	case Trajectory :	
      		if (add)
			{
				NxTraject *traject = score->getTraject(inID);
	  			trajectList->setRowCount( nbTrajects );

				strings = strings.setNum( traject->getID() ) ;
	  			trajectList->setItem( nbTrajects-1, 0, new QTableWidgetItem(strings) );
				trajectList->item(nbTrajects-1,0)->setBackgroundColor(backgColor) ;
				trajectList->item(nbTrajects-1,0)->setFlags(!Qt::ItemIsEditable) ;
								
				strings = traject->getName();
	  			trajectList->setItem( nbTrajects-1, 1, new QTableWidgetItem(strings) );
				
				strings = strings.setNum( traject->getCustomID() );
	  			trajectList->setItem( nbTrajects-1, 2, new QTableWidgetItem(strings) );

	  			strings = strings.setNum( traject->getGroupID() );
	  			trajectList->setItem( nbTrajects-1, 3, new QTableWidgetItem(strings) );
				
				strings = strings.setNum( traject->getLayer() );
	  			trajectList->setItem( nbTrajects-1, 4, new QTableWidgetItem(strings) );

				switch( traject->getType() ) {
					case 1 : // Circle
						strings = "Circle";
					break;
					case 2 : // Line
						strings = "Line";
					break;
				}
				trajectList->setItem( nbTrajects-1, 5, new QTableWidgetItem(strings) );
				
				strings = strings.setNum( traject->getLength(), 'f', 4 );
				trajectList->setItem( nbTrajects-1, 6, new QTableWidgetItem(strings) );
				
				trajectList->item(nbTrajects-1,5)->setBackgroundColor(backgColor) ;
				trajectList->item(nbTrajects-1,6)->setBackgroundColor(backgColor) ;
				trajectList->item(nbTrajects-1,5)->setFlags(!Qt::ItemIsEditable) ;
				trajectList->item(nbTrajects-1,6)->setFlags(!Qt::ItemIsEditable) ;
				
				trajectList->resizeRowToContents( nbTrajects-1 ) ;
			}
      		else	
				for (int i=0; i<trajectList->rowCount(); i++)
					if ( trajectList->item(i, 0)->text().toInt() == inID )
						trajectList->removeRow(i);
			break;	
		
			
    	case Cursor :	
      		if (add)
			{
	  			cursorList->setRowCount( nbCursors );
				NxCursor *cursor = score->getCursor(inID);
				
	  			strings = strings.setNum( cursor->getID() );
	  			cursorList->setItem( nbCursors-1, 0, new QTableWidgetItem(strings) );
				cursorList->item(nbCursors-1,0)->setBackgroundColor(backgColor) ;
				cursorList->item(nbCursors-1,0)->setFlags(!Qt::ItemIsEditable) ;
								
				strings = cursor->getName();
	  			cursorList->setItem( nbCursors-1, 1, new QTableWidgetItem(strings) );
			
				strings = strings.setNum( cursor->getCustomID() );
	  			cursorList->setItem( nbCursors-1, 2, new QTableWidgetItem(strings) );
			
	  			strings = strings.setNum( cursor->getGroupID() );
	  			cursorList->setItem( nbCursors-1, 3, new QTableWidgetItem(strings) );
				
				strings = strings.setNum( cursor->getLayer() );
	  			cursorList->setItem( nbCursors-1, 4, new QTableWidgetItem(strings) );
				
	  			strings = strings.setNum( cursor->getParentTrajectory()->getID() );
	  			cursorList->setItem( nbCursors-1, 5, new QTableWidgetItem(strings) );
				cursorList->item(nbCursors-1,5)->setBackgroundColor(backgColor) ;
				cursorList->item(nbCursors-1,5)->setFlags(!Qt::ItemIsEditable) ;
				
				strings = strings.setNum( cursor->getSpeed(), 'f', 4 );
				cursorList->setItem( nbCursors-1, 6, new QTableWidgetItem(strings) );
			
				strings = strings.setNum( cursor->getWidth(), 'f', 4 );
				cursorList->setItem( nbCursors-1, 7, new QTableWidgetItem(strings) );	
				
				// Playback
				QStringList strList2;
				strList2 << "Forward" << "Backward";
				QComboBox *combo2 = new QComboBox;
				combo2->addItems(strList2);
				if ( cursor->getForward() )
					combo2->setCurrentIndex(0);
				else
					combo2->setCurrentIndex(1);
				cursorList->setCellWidget( nbCursors-1, 8, combo2 );
				connect( combo2, SIGNAL(currentIndexChanged(int)), this, SLOT(updateCursorsBackward()) ) ;	
				// Mode	
				QStringList strList3;
				strList3 << "Single" << "Repeat" << "Fwd/Bckwd" ;
				QComboBox *combo3 = new QComboBox;
				combo3->addItems(strList3);
				combo3->setCurrentIndex( (int)cursor->getMode() );
				cursorList->setCellWidget( nbCursors-1, 9, combo3 );	
				connect( combo3, SIGNAL(currentIndexChanged(int)), this, SLOT(updateCursorsMode()) ) ;		
				
				cursorList->resizeRowToContents( nbCursors-1 ) ;	 
			}
      		else
				for (int i=0; i<cursorList->rowCount(); i++)
					if ( cursorList->item(i, 0)->text().toInt() == inID )
						cursorList->removeRow(i);
			break;	
		
			
    	case Trigger :
      		if (add)
			{
		
	  			triggerList->setRowCount( nbTriggers );	
	  			NxTrigger *trigger = score->getTrigger(inID);
				
				strings = strings.setNum( trigger->getID() ) ;
	  			triggerList->setItem(nbTriggers-1, 0, new QTableWidgetItem(strings) );
				triggerList->item(nbTriggers-1,0)->setBackgroundColor( backgColor ) ;
				triggerList->item(nbTriggers-1,0)->setFlags(!Qt::ItemIsEditable) ;
				
				strings = trigger->getName();
	  			triggerList->setItem(nbTriggers-1, 1, new QTableWidgetItem(strings) );
				
				strings = strings.setNum( trigger->getCustomID() );
				triggerList->setItem(nbTriggers-1, 2, new QTableWidgetItem(strings) );
			
	  			strings = strings.setNum( trigger->getGroupID() );
	  			triggerList->setItem( nbTriggers-1, 3, new QTableWidgetItem(strings) );
				
				strings = strings.setNum( trigger->getLayer() );
	  			triggerList->setItem( nbTriggers-1, 4, new QTableWidgetItem(strings) );
				
	  			strings = trigger->getHost() ;
	  			triggerList->setItem( nbTriggers-1, 5, new QTableWidgetItem(strings) );
				
	  			strings = strings.setNum( trigger->getPort() );
	  			triggerList->setItem( nbTriggers-1, 6, new QTableWidgetItem(strings) );		
				
				strings = strings.setNum( trigger->getValue() );
	  			triggerList->setItem( nbTriggers-1, 7, new QTableWidgetItem(strings) );		
				
				triggerList->resizeRowToContents( nbTriggers-1 ) ;							
			}
      		else 
				for (int i=0; i<triggerList->rowCount(); i++)
					if ( triggerList->item(i, 0)->text().toInt() == inID )
						triggerList->removeRow(i);
		break;	
	}	
}


////////////////////////////////////
// Updating Indexes
////////////////////////////////////
void NxControlCenter::updateIndexes()
{
	nbCurves 	= score->curvesIndexes().count();
  	nbCursors 	= score->cursorsIndexes().count();
  	nbTrajects 	= score->trajectsIndexes().count();
  	nbTriggers 	= score->triggersIndexes().count();
	
	controlBox->setTabText ( 0, QString("%1 Curves").arg(nbCurves) ) ;
	controlBox->setTabText ( 1, QString("%1 Trajects").arg(nbTrajects) ) ;
	controlBox->setTabText ( 2, QString("%1 Cursors").arg(nbCursors) ) ;
	controlBox->setTabText ( 3, QString("%1 Triggers").arg(nbTriggers) ) ;
}


///////////////////////////
// Text Settings
///////////////////////////
void NxControlCenter::languageChange()
{   
	setWindowTitle( tr( "IanniX Control Center" ) );
}
// ****************************************************
// Parameters updates (if changed from another place)
// ****************************************************
void NxControlCenter::updateParameters( int choice, int inID )
{	
	QList<int> curvInd	= score->curvesIndexes();
	QList<int> trajInd	= score->trajectsIndexes();
	QList<int> cursInd	= score->cursorsIndexes();
	QList<int> trigInd	= score->triggersIndexes();
	NxCurve		*curv ;
	NxTraject	*traj ;
	NxCursor	*curs ;
	NxTrigger	*trig ;
	int idx = 0	; // Initialisation
	int cforw ;
	int cmode ;
	
	switch (choice) {
		case Curve : ///////////////////////// CURVE
			idx = curvInd.indexOf(inID) ;
			curv = score->getCurve(inID) ;
			// Name
			curveList->item(idx,1)->setText( curv->getName() ) ;
			curveList->resizeColumnToContents(1) ;
			// CID
			curveList->item(idx,2)->setText( QString().setNum( curv->getCustomID() ) ) ;
			curveList->resizeColumnToContents(2) ;
			// GID
			curveList->item(idx,3)->setText( QString().setNum( curv->getGroupID() ) ) ;
			curveList->resizeColumnToContents(3) ;
			// Layer
			curveList->item(idx,4)->setText( QString().setNum( curv->getLayer() ) ) ;
			curveList->resizeColumnToContents(4) ;
			// Duration
 			curveList->item(idx,5)->setText( QString().setNum( curv->getXRange() ) ) ;
			curveList->resizeColumnToContents(5) ;
			// Real Min
			curveList->item(idx,7)->setText( QString().setNum( curv->getRealMin() ) ) ;
			curveList->resizeColumnToContents(7) ;
			// Real Max
			curveList->item(idx,8)->setText( QString().setNum( curv->getRealMax() ) ) ;
			curveList->resizeColumnToContents(8) ;
			// Host
			curveList->item(idx,9)->setText( curv->getHost() ) ;
			curveList->resizeColumnToContents(9) ;
			// Port
			curveList->item(idx,10)->setText( QString().setNum( curv->getPort() ) ) ;
			curveList->resizeColumnToContents(10) ;
			
			curveList->resizeRowToContents(idx) ;
		break;			
		case Trajectory : ////////////////////// TRAJECTORY
			idx = trajInd.indexOf(inID) ;
			traj = score->getTraject(inID) ;
			// Name
			trajectList->item(idx,1)->setText( traj->getName() ) ;
			trajectList->resizeColumnToContents(1) ;
			// CID
			trajectList->item(idx,2)->setText( QString().setNum( traj->getCustomID() ) ) ;
			trajectList->resizeColumnToContents(2) ;
			// GID
			trajectList->item(idx,3)->setText( QString().setNum( traj->getGroupID() ) ) ;
			trajectList->resizeColumnToContents(3) ;
			// Layer
			trajectList->item(idx,4)->setText( QString().setNum( traj->getLayer() ) ) ;
			trajectList->resizeColumnToContents(4) ;
			// Length
			trajectList->item(idx,6)->setText( QString().setNum( traj->getLength() ) ) ;
			trajectList->resizeColumnToContents(6) ;

			trajectList->resizeRowToContents(idx) ;
		break;
		case Cursor : ///////////////////////// CURSOR
			idx = cursInd.indexOf(inID) ;
			curs = score->getCursor(inID) ;
			// Name
			cursorList->item(idx,1)->setText( curs->getName() ) ;
			cursorList->resizeColumnToContents(1) ;
			// CID
			cursorList->item(idx,2)->setText( QString().setNum( curs->getCustomID() ) ) ;
			cursorList->resizeColumnToContents(2) ;
			// GID
			cursorList->item(idx,3)->setText( QString().setNum( curs->getGroupID() ) ) ;
			cursorList->resizeColumnToContents(3) ;
			// Layer
			cursorList->item(idx,4)->setText( QString().setNum( curs->getLayer() ) ) ;
			cursorList->resizeColumnToContents(4) ;
			// Speed
			cursorList->item(idx,6)->setText( QString().setNum( curs->getSpeed() ) ) ;
			cursorList->resizeColumnToContents(6) ;
			// Width
			cursorList->item(idx,7)->setText( QString().setNum( curs->getWidth() ) ) ;
			cursorList->resizeColumnToContents(7) ;
			// Forward
			if(curs->getForward()) cforw = 0 ;
			else cforw = 1 ;
			( (QComboBox*)cursorList->cellWidget(idx,8) )->setCurrentIndex( cforw ) ;
			// Mode
			cmode = (int)curs->getMode() ;
			( (QComboBox*)cursorList->cellWidget(idx,9) )->setCurrentIndex( cmode ) ;
			
			cursorList->resizeRowToContents(idx) ;
		break;
		case Trigger : ///////////////////////// TRIGGER
			idx = trigInd.indexOf(inID) ;
			trig = score->getTrigger(inID) ;
			// Name
			triggerList->item(idx,1)->setText( trig->getName() ) ;
			triggerList->resizeColumnToContents(1) ;
			// CID
			triggerList->item(idx,2)->setText( QString().setNum( trig->getCustomID() ) ) ;
			triggerList->resizeColumnToContents(2) ;
			// GID
			triggerList->item(idx,3)->setText( QString().setNum( trig->getGroupID() ) ) ;
			triggerList->resizeColumnToContents(3) ;
			// Layer
			triggerList->item(idx,4)->setText( QString().setNum( trig->getLayer() ) ) ;
			triggerList->resizeColumnToContents(4) ;
			// Host
			triggerList->item(idx,5)->setText( trig->getHost() ) ;
			triggerList->resizeColumnToContents(5) ;
			// Port
			triggerList->item(idx,6)->setText( QString().setNum( trig->getPort() ) ) ;
			triggerList->resizeColumnToContents(6) ;
			// Value
			triggerList->item(idx,7)->setText( QString().setNum( trig->getValue() ) ) ;
			triggerList->resizeColumnToContents(7) ;
			
			triggerList->resizeRowToContents(idx) ;
		break;
	}
}

// **************************
// Dynamic Table Updates
// **************************
void NxControlCenter::updateCurve( int inRow, int inCol )
{
	if(inCol!=6) curveList->resizeColumnToContents(inCol) ;
	curveList->resizeRowToContents(inRow) ;
	QList<int> indexes = score->curvesIndexes();
	QString str, str2;
	str 			= curveList->item( inRow, inCol )->text();
	int idx 		= indexes.at( inRow ) ;
	NxCurve *curve 	= score->getCurve(idx);
	
	switch (inCol)
	{
		case 1 : 
			curve->setName( str );
		break;			
		case 2 : // Custom ID
			curve->setCustomID( str.toInt() );
		break;
		case 3 : // GroupID
			curve->setGroupID( str.toInt() );
		break;
		case 4 : // Layer
			curve->setLayer( str.toInt() );
		break;
		case 7 : // real min
			curve->setRealMin( str.toFloat() );
			curveList->item(inRow, inCol)->setText( QString("%1").arg(curve->getRealMin()) ) ;
			if(curve->getDrawType() == SplineStrip) curve->splineGenerate() ;
		break; 
		case 8 : // real max
			curve->setRealMax( str.toFloat() );
			curveList->item(inRow, inCol)->setText( QString("%1").arg(curve->getRealMax()) ) ;
			if(curve->getDrawType() == SplineStrip) curve->splineGenerate() ;
		break;		
		case 9 : // host
			curve->setHost( str );
		break;
		case 10 : // port
			curve->setPort( str.toFloat() );
		break;
	}
	//updateItems( Curve, true, idx ) ;
}

void NxControlCenter::updateTrajectory( int inRow, int inCol )
{
	trajectList->resizeColumnToContents(inCol) ;
	trajectList->resizeRowToContents(inRow) ;
	QList<int> indexes = score->trajectsIndexes();
	QString str;
	str 				= trajectList->item( inRow, inCol )->text();
	int idx 			= indexes.at( inRow ) ;
	NxTraject *traject 	= score->getTraject(idx);
	
	switch (inCol)
	{
		case 1 :
			traject->setName( str );
		break;			
		case 2 : // Custom ID
			traject->setCustomID( str.toInt() );
		break;
		case 3 : // GroupID
			traject->setGroupID( str.toInt() );
		break;	
		case 4 : // Layer
			traject->setLayer( str.toInt() );
		break;		
	}
}

void NxControlCenter::updateCursor( int inRow, int inCol )
{
	cursorList->resizeColumnToContents(inCol) ;
	cursorList->resizeRowToContents(inRow) ;
	QList<int> indexes = score->cursorsIndexes();
	QString str;
	str 				= cursorList->item( inRow, inCol )->text();
	int idx				= indexes.at( inRow ) ;
	NxCursor *cursor 	= score->getCursor(idx);
	
	switch (inCol)
	{
		case 1 :
			cursor->setName( str );
		break;
		case 2 : // Custom ID
			cursor->setCustomID( str.toInt() );
		break;
		case 3 : // GroupID
			cursor->setGroupID( str.toInt() );
		break;	
		case 4 : // Layer
			cursor->setLayer( str.toInt() );
		break;	
		case 6 :
			cursor->setSpeed( str.toFloat() );
		break;
		case 7 :
			cursor->setWidth( str.toFloat() );
		break;	
	}
}

void NxControlCenter::updateTrigger( int inRow, int inCol )
{
	triggerList->resizeColumnToContents(inCol) ;
	triggerList->resizeRowToContents(inRow) ;
	QList<int> indexes = score->triggersIndexes();
	QString str;
	str 				= triggerList->item( inRow, inCol )->text();
	int idx 			= indexes.at( inRow ) ;
	NxTrigger *trigger 	= score->getTrigger(idx);
	
	switch (inCol)
	{
		case 1 :
			trigger->setName( str );
		break;			
		case 2 : // Custom ID
			trigger->setCustomID( str.toInt() );
		break;
		case 3 : // Group ID
			trigger->setGroupID( str.toInt() );
		break;
		case 4 : // Layer
			trigger->setLayer( str.toInt() );
		break;
		case 5 :
			trigger->setHost( str );
		break;		
		case 6 :
			trigger->setPort( str.toInt() );
		break;	
		case 7 :
			trigger->setValue( str.toInt() );
		break;	
	}
}

////////////////////////////
// Closing Event
////////////////////////////
void NxControlCenter::closeEvent( QCloseEvent* )
{
	//this->releaseKeyboard();
  	//parentGLBox->grabKeyboard();
  	//this->hide();
}

void NxControlCenter::updateCurvesDrawTypes()		// COULD BE BETTER ...
{
	int nbC = score->getEnvironment().nbCurves ;
	int tempID = 0 ;
	int tempDrawT = 0 ;
	for(int i=0 ; i<nbC ; i++) {
		tempID = (curveList->item(i,0))->text().toInt() ;
		tempDrawT =  8 - ( ( (QComboBox*)curveList->cellWidget(i,6) )->currentIndex() ) ;
		score->getCurve( tempID )->setDrawType( tempDrawT ) ;
		if(tempDrawT == SplineStrip) { // Generate Spline if Line->Spline
			score->getCurve( tempID )->splineGenerate() ; 
			score->getCurve( tempID )->setSplineMinMax() ;
		}
	}
}
void NxControlCenter::updateCursorsBackward()		// COULD BE BETTER ...
{
	int nbC = score->getEnvironment().nbCursors ;
	int tempID = 0 ;
	bool tempBack = false ;
	for(int i=0 ; i<nbC ; i++) {
		tempID = (cursorList->item(i,0))->text().toInt() ;
		if( ( (QComboBox*)cursorList->cellWidget(i,8) )->currentIndex() == 0 ) tempBack = true ;
			else tempBack = false ;
		score->getCursor( tempID )->setForward( tempBack ) ;
	}
}
void NxControlCenter::updateCursorsMode()		// COULD BE BETTER ...
{
	int nbC = score->getEnvironment().nbCursors ;
	int tempID = 0 ;
	int tempMode = 0 ;
	for(int i=0 ; i<nbC ; i++) {
		tempID = (cursorList->item(i,0))->text().toInt() ;
		tempMode = ( (QComboBox*)cursorList->cellWidget(i,9) )->currentIndex() ;
		score->getCursor( tempID )->setMode( tempMode ) ;
	}
}

///////////////////////
// Key Events
///////////////////////
void NxControlCenter::keyReleaseEvent( QKeyEvent* e )
{
	NxObject* obj ;
	int rID ;
	switch ( e->key() )
	{
		case Qt::Key_Backspace :
			for (int i = nbCurves-1 ; i >=0; i-- )
				if ( curveList->isItemSelected( curveList->horizontalHeaderItem(i) ) )
				{
					rID = curveList->item(i,0)->text().toInt() ;
					obj = parentGLBox->getPartition()->getCurve( rID ) ;
					curveList->removeRow( i );
					parentGLBox->destroyObject( obj );
				}
			for (int i = nbTrajects-1 ; i >=0; i-- )
				if ( trajectList->isItemSelected( trajectList->horizontalHeaderItem(i) ) )
				{
					rID = trajectList->item(i,0)->text().toInt() ;
					obj = parentGLBox->getPartition()->getTraject( rID ) ;
					trajectList->removeRow( i );
					parentGLBox->destroyObject( obj );
				}
			for (int i = nbCursors-1 ; i >=0; i-- )
				if ( cursorList->isItemSelected( cursorList->horizontalHeaderItem(i) ) )
				{
					rID = cursorList->item(i,0)->text().toInt() ;
					obj = parentGLBox->getPartition()->getCursor( rID ) ;
					cursorList->removeRow( i );
					parentGLBox->destroyObject( obj );
				}
			for (int i = nbTriggers-1 ; i >=0; i-- )
				if ( triggerList->isItemSelected( triggerList->horizontalHeaderItem(i) ) )
				{
					rID = triggerList->item(i,0)->text().toInt() ;
					obj = parentGLBox->getPartition()->getTrigger( rID ) ;
					triggerList->removeRow( i );
					parentGLBox->destroyObject( obj );
				}
			parentGLBox->updateIndexes();
			updateIndexes();
		break;
	}
}

