///////////////
// IanniX
///////////////

////////////////////////////
// NxGLScore.cpp
////////////////////////////

// C standard libs
#include <cstdlib>
#include <iostream>

// Class Header
#include "NxGLScore.h"

////////////////////////////////
// Warning disabling
///////////////////////////////
#if defined(Q_CC_MSVC)
#pragma warning(disable:4305) 
#endif

//////////////////////
// Constructor
//////////////////////
NxGLScore::NxGLScore( QWidget* parent, NxView* daddy, QString name, QString inFile, int inL, int inH, bool isNew )
  : QGLWidget( parent )
{
	mParent = (NxView*)daddy;
	
	neverSaved = firstClick = lineRuleSetFirstPt = true ;
	frustum = onClick = onDrag = firstPoint = zoomMode = false;
  	selectionMode = curveEdition = curveCreation = trajectCreation = cursorMode = cursorMoving = mouseZoom = trajectEdition = false;
	curvEdPtChoosed = false ;	// pt choosed to be moved (on a curve)
	curvEdOnlyClic = true ;		// to avoid 'only clic' bug
	lockedSelection = false ;	// for selection lock by contextual menu
	snapLockX = snapLockY = lineRuleMode = gridMove = false ;
	gridDecX = 0 ;		// Grid X translation
	gridDecY = 0 ;		// Grid Y translation
	
	//setAccessibleName( name ) ;
	
	selected = NULL ;
	UDPListenerCreated = false ;
	
	mouseXPixPos = mouseYPixPos = 0 ;
	timePosPix.x = timePosPix.y = timePosPix.z = 0 ;
	posX = posY = offX = offY = transX = transY = selX = selY = 0.0;
  	cursX = cursY = diffX = diffY = 0.0;
  	mouseX = mouseY = 0;
  	IDselected = 0 ;
	currentPoints = 1 ;			// Counter for the curve creation
  	pointID = -1 ;				// curves points edition
	selTrajectPoint = -1 ;		// traject lines edition
	drawMode = LineStrip ;
	curveXMinStep = 0.001 ;		// x-step between values on a curve
	
  	//////////////////// COLORS /////////////////////////////
	selectedColor.ID = BBColor.ID = inactiveColor.ID = 0 ;
	selectedColor.c		= QColor(255,255,255,255) ;
	BBColor.c			= QColor(255,127,0,255) ;
	inactiveColor.c		= QColor(90, 90, 90, 150) ;	
	/////////////////////////////////////////////////////////
	//// Contextual Color Menus ////
	dialogColorPicked = false ;
	objectPickedColor.ID = 0 ;
	///////////////////////////////
	
  	setMouseTracking( true ) ;
	
  	/////////// Opening score
  	if (inFile.contains(".xml")) {
		score = new NxScore(inFile);
		neverSaved = false ;
	}
  	else score = new NxScore(inFile, 0, inL, inH);	
	
	////// XML VERSION TEST //////
	QString xmlVersion = score->getEnvironment().xmlVersion ;
	if(xmlVersion=="0.631b" || xmlVersion=="0.635b" || xmlVersion=="0.636b" || xmlVersion=="0.637b" || xmlVersion=="0.638b" || xmlVersion=="0.639b") printf("Opening Score - XML Version(v%s) OK\n", xmlVersion.toAscii().constData()) ;
	else QMessageBox::about(this, "Warning !", "The XML score seems to be from an older\nversion of IanniX (before 0.631b).\nPlease check the XML and re-open the score.\n") ;
		
	int numb = inFile.lastIndexOf("/") ;
	inFile.truncate(numb+1) ;
	NxEnv tmpEnv = score->getEnvironment() ;
  	tmpEnv.path = inFile ;
	score->setEnvData( tmpEnv ) ;
	
  	////////// Retrieving global indexes
  	nbCurves 	= score->getEnvironment().nbCurves;
  	nbTrajects 	= score->getEnvironment().nbTrajects;
  	nbTriggers 	= score->getEnvironment().nbTriggers;
  	nbCursors 	= score->getEnvironment().nbCursors;
  	
	if (isNew) {
		length = inL ;
  		height = inH ;
	}
	else {
		length = score->getEnvironment().length ;
		height = score->getEnvironment().height ;
	}
	///////////////////////////////////////////////////////////////////////////
	///////////////// ENVIRONMENT VARIABLES INITIALISATION ////////////////////
	updateScoreColors() ;
	////////////////////////////// MODES
	visualMode		= score->getEnvironment().visualMode ;
	oscOutMode		= score->getEnvironment().oscOutMode ;
	oscInMode		= score->getEnvironment().oscInMode ;
	showTrajects	= score->getEnvironment().showTrajects ;
	showCursors		= score->getEnvironment().showCursors ;
	showCurves		= score->getEnvironment().showCurves ;
	showTriggers	= score->getEnvironment().showTriggers ;
	////////////////////////////// DISPLAY
	crossesDraw		= score->getEnvironment().crossesDraw ;	
	scoreDraw		= score->getEnvironment().scoreDraw ;	
	displayOn		= score->getEnvironment().displayOn ;
	circlesDraw		= score->getEnvironment().circlesDraw ;	
	rectDraw		= score->getEnvironment().rectDraw ;
	showCursorsArea = score->getEnvironment().showCursorsArea ;
	antialiasing	= true ;
	// non Env
	gridOpacity = 0.3 ;
	cursZonesOpacity = 0.2 ;
	// Grid
	showGrid		= score->getEnvironment().showGrid ;
	snapGrid		= score->getEnvironment().snapGrid ;		
	autoGridStep	= score->getEnvironment().autoGridStep ;
	gridStep		= score->getEnvironment().gridStep ;
	if(gridStep > 0.01) gridStepInt = int(floor( gridStep*100 )) ;
	else gridStepInt = 1 ;
	////////////////////////////// TIME
	useCirclesAlgo	= score->getEnvironment().useCirclesAlgo ;
	broadcastMode	= score->getEnvironment().broadcastMode ;
	sendIFmessages	= score->getEnvironment().sendIFmessages ;
	// non Env
	moveCursors		= true ;

	timeScheduler = new NxScheduler( this );
	
	timeScheduler->moveCursors( moveCursors ) ;
	timeScheduler->useAutoSchedulerStep( true ) ;
	timeScheduler->useBroadcastMode( broadcastMode ) ;
	timeScheduler->setOSCOutActive( oscOutMode ) ;
	timeScheduler->setInitialTime( score->getEnvironment().initialTime ) ;
	timeScheduler->setFinalTime( score->getEnvironment().finalTime ) ;
	timeScheduler->useCirclesAlgorithm( useCirclesAlgo, false ) ;
	timeScheduler->updateSendingOptions() ;
	
	timeScheduler->initScheduler();	
	//////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////
	
	updateCheckedMenus() ;
	
  	//////////////////////// Menus Initialization ////////////////////////
  	objectMenu			= new NxObjectEditor( this );
  	controlCenter		= new NxControlCenter( this );
	timeSpeedEditor		= new NxTime( this, 1 ) ; 
	timeWidthEditor		= new NxTime( this, 0 ) ;
	fileChooser			= new QFileDialog;
	
  	/////////////////////////////
  	// OpenGL Section
  	////////////////////////////
	scaleX = scaleY = scaleZ = 0.17 ;
	RScaleX = RScaleY = 1 ;
	initSelfH = 504 ;
	initSelfW = 795 ;
  	nHeight = (float)length*initSelfH/initSelfW ; // depends of initial selfH and selfW (Qmainwindow)
	NbSLICES = 60 ;
  	slice = 360 / NbSLICES ;
  	nbCercles = 0 ;
	
	xRot = yRot = zRot = 0.0 ;
	angleX = angleY = oldAngleX = oldAngleY = 0.0 ;
	mouseRotation = false ;
  	decX = decY = 0 ;
  	bX = length / (RScaleX*scaleX*10.0) ;
  	bY = nHeight / (RScaleY*scaleY*10.0) ;
  	order = 4 ;
		
  	/// Initializing randomizer seed with current time
  	randomizer = static_cast<unsigned>(time(NULL)) ;

	// Retrieving GL Widget size
	selfW = frameSize().width();
	selfH = frameSize().height();

	glTimer = new QTimer( this );

	connect(glTimer, SIGNAL(timeout()), this, SLOT(refreshView()));
	
	mParent->setWindowTitle( QString("IanniX Viewer - %1 %2").arg( getZoomScale() ).arg('%') );
	
	////////////////////////// UDP RECEIVER //////////////////////////
	//printf("Create UDP Listener\n") ;
	oscCurveCreation = false ;
	oscCurveListening = false ;
	listener = new NxListener( this ) ;
	if(oscInMode) updateUDPListener( true ) ; // Listener Creation
	//////////////////////////////////////////////////////////////////
	
	BBsize = (float)length/(600*scaleX) ; // init first time -> initial cursor selection
	
	assignCursors() ;
}
void NxGLScore::updateScoreColors()
{
	bColor = score->getEnvironment().backColor ;
	sColor = score->getEnvironment().scoreColor ;
	gColor = score->getEnvironment().gridColor ;
}

////////////////////
// Destructor
///////////////////
NxGLScore::~NxGLScore()
{
	//delete objectMenu;
  	//delete controlCenter;
  	//delete fileChooser;
}
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
//////////////////// UDP OSC SERVER
///////////////////////////////////////////////////////////////////////////////////////////////
void NxGLScore::updateUDPListener( bool chooser ) {
	if( chooser && !UDPListenerCreated ) {
		printf("Create and Connect OSC UDP Listener Slot (port=%d) \n", score->getEnvironment().UDPin ) ;
		udpReceiver = new QUdpSocket( this );
		udpReceiver->bind( score->getEnvironment().UDPin );
		connect(udpReceiver, SIGNAL(readyRead()), this, SLOT(processUDPMessage()) ) ;
		UDPListenerCreated = true ;
	}
	if( !chooser && UDPListenerCreated ) {
		printf("Disconnect and Delete OSC UDP Listener\n") ;
		disconnect(udpReceiver, SIGNAL(readyRead()), this, SLOT(processUDPMessage()) ) ;
		delete( udpReceiver ) ;
		UDPListenerCreated = false ;
	}
}
void NxGLScore::processUDPMessage()
{
	//printf("Message Received OK !! - Process receiver...\n") ;	
	while ( udpReceiver->hasPendingDatagrams() ) {
		QByteArray		datagram ;
		datagram.resize( udpReceiver->pendingDatagramSize() ) ;
		QHostAddress	sender ;
		quint16			senderPort ;
		udpReceiver->readDatagram( datagram.data(), datagram.size(), &sender, &senderPort ) ;

		//printf("MESSAGE UDP Datagram : %s \n", datagram.data() ) ;
		
		osc::ReceivedPacket packetOSC( datagram.data(), datagram.size() ) ;
		if( !packetOSC.IsBundle() ) {
			//printf("OSC - Message non Bundle\n") ;
			osc::ReceivedMessage messageOSC( packetOSC ) ;
			listener->ProcessMessage( messageOSC, 0 ) ;
		}
		else {
			//printf("OSC - Message Bundle\n") ;
			listener->ProcessBundle( osc::ReceivedBundle(packetOSC), 0 ) ;
		}
	}
}
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////// SCORE
void NxGLScore::oscSendInfos()
{
	timeScheduler->sendOSCConfigMessage() ;
}
void NxGLScore::oscSetTempo( float inTempo )
{
	mParent->changeTempo( (int)floor(100 * inTempo) ) ;
}
void NxGLScore::oscMoveCursors( int chooser )
{	
	moveCursors = (chooser==1) ;
	timeScheduler->moveCursors( moveCursors ) ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::oscBroadcastMode( int chooser )
{
	broadcastMode = (chooser==1) ;
	timeScheduler->useBroadcastMode( broadcastMode ) ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::oscOSCMode( int chooser ) 
{
	oscOutMode = (chooser==1) ;
	timeScheduler->setOSCOutActive( oscOutMode ) ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////// TRIGGER CREATION
void NxGLScore::oscNewTrigger( float inX, float inY, float inZ, int inCID, int inGID, int inIntValue, int colR, int colG, int colB  )
{	
	//printf("OSC Creation : TRIGGER ADDED !\n") ;
	// Position
	NxPoint inPos ;
	inPos.ID = 0 ;
	inPos.x = inX ;
	inPos.y = inY ;
	inPos.z = inZ ;
	// Color
	QColor col = QColor(colR, colG, colB) ;
	Color inColor ;
	inColor.ID = 0 ;
	inColor.c = col ;
	// Others
	int inLay = 0 ;
	QString inName = QString("oscTrigger") ;
	QString inHost = QString("localhost") ;
	int inID = score->getCptTrigger() ;
	int inPort = 1979 ;
	
	addNewTrigger( inPos, inID, inCID, inGID, inLay, inName, inColor, inHost, inPort, inIntValue ) ;
	score->maxOfIndexes() ;
}
void NxGLScore::oscTriggerValue( int inID, int inValue )
{
	if( score->isThereTrigger( inID ) ) { // Test of existence
		((NxTrigger*)score->getTrigger( inID ))->setValue( inValue ) ;
		controlCenter->updateParameters( Trigger, inID ) ;
	}
	else printf("OSC Trigger Value - Cursor doesn't exist\n") ;
}
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////// CURVES CREATION
void NxGLScore::oscCurveCreationBegin( float inPx, float inPy, float inPz, int inCID, int inGID, int c1R, int c1G, int c1B )
{
	if(!oscCurveCreation) {
		printf("Curve Creation\n") ;
		// Initialisation
		oscCurveRealMin = oscCurveRealMax = 0 ;
		oscNbPoints = 0 ;
		
		// Position
		oscCurve1R = c1R ;
		oscCurve1G = c1G ;
		oscCurve1B = c1B ;
		oscCurvePos.ID = 0 ;
		oscCurvePos.x = inPx ;
		oscCurvePos.y = inPy ;
		oscCurvePos.z = inPz ;
		// Parameters
		oscCurveCID = inCID ;
		oscCurveGID = inGID ;
		
		oscCurveCreation = true ;
		oscCurveListening = true ;
	}
}
void NxGLScore::oscCurveCreationAddPt( float inPx, float inPy )
{
	if(oscCurveListening) {
		NxPoint oscNewPt ;
		oscNewPt.z = 0 ;
		oscNewPt.ID = oscNbPoints ;
		oscNewPt.x = inPx ;
		oscNewPt.y = inPy ;
		// Force inPx to advance
		float lastx = 0 ;
		if(oscNbPoints != 0) {
			lastx = ( oscEditedCurve.last() ).x ;
			if(inPx<lastx+curveXMinStep) oscNewPt.x = lastx + curveXMinStep ;
		}
		else oscNewPt.x = 0 ;
		
		//printf("Add point %d // x: %f y : %f\n", oscNbPoints, oscNewPt.x, oscNewPt.y ) ;
		
		oscEditedCurve.append( oscNewPt ) ;
		
		// Curve Real Bounds update
		if(inPy>oscCurveRealMax) oscCurveRealMax = inPy ;
		if(inPy<oscCurveRealMin) oscCurveRealMin = inPy ;
		
		oscNbPoints++ ;
	}
}
void NxGLScore::oscCurveCreationEnd()
{
	if(oscNbPoints>=2) {
		
		//pause() ; // Stop Playback

		// Fixed Parameters
		int realID = score->getCptCurve() ;
		int inLay = 0 ;
		float inScale = 1 ;
		float inAngle = 0 ;
		int inSpatialPeriod = 20 ;
		int inPort = 1979 ;
		QString inHost = "localhost" ;
		QString savName = "oscCurve" ;
		Color inColor1, inColor2 ;
		QColor col1 = QColor(oscCurve1R, oscCurve1G, oscCurve1B) ;
		QColor col2 = QColor(50, 200, 50) ;
		inColor1.ID = inColor2.ID =  0 ;
		inColor1.c = col1 ;
		inColor2.c = col2 ;
	
		// ADD CURVE
		printf("End : add curve\n") ;
	
		score->addCurve( oscCurvePos, realID, oscCurveCID, oscCurveGID, inLay, savName, inColor1, inColor2, oscEditedCurve, oscEditedCurve.count(), oscCurveRealMin, oscCurveRealMax, inScale, inAngle, inSpatialPeriod, inHost, inPort ) ;
		score->getCurve(realID)->setDrawType( LineStrip ) ;
		score->getCurve(realID)->evalBoundingBox( oscCurvePos, 0 ) ;
		score->maxOfIndexes() ;
		updateIndexes() ;
		controlCenter->updateItems( Curve, true, realID ) ;
		timeScheduler->addNewCurve() ;
		
		oscCurveListening = oscCurveCreation = false ;
		oscEditedCurve.clear() ;
	}
	else {
		oscCurveListening = oscCurveCreation = false ;
		if(oscNbPoints!=0) oscEditedCurve.clear() ;
	}
}
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////// CURVE POINT MODIFICATION
void NxGLScore::oscCurveModifPt( int inID, int nb, float inX, float inY )
{
	if( score->isThereCurve( inID ) ) { // Test Curve existence
		printf("Curve Point Modif\n") ;
		NxCurve *curv = score->getCurve( inID ) ;
		int initialSize = curv->size() ;
		if( nb < initialSize && nb>0 ) { // Test Point existence
			QVector<NxPoint> oscSelCurve = curv->getCurve() ;
			NxPoint cvP		= curv->getPosition() ;
			float cvMin		= curv->getRealMin() ;
			float cvMax		= curv->getRealMax() ;
			NxPoint newP = oscSelCurve[nb] ;
			newP.x = inX ;
			newP.y = inY ;
			
			///// Constrain the position
			if( nb != 0 )
				if (newP.x < oscSelCurve[nb-1].x + curveXMinStep )	newP.x = oscSelCurve[nb-1].x + curveXMinStep ;
			if( nb!=initialSize-1 )
				if (newP.x > oscSelCurve[nb+1].x - curveXMinStep )	newP.x = oscSelCurve[nb+1].x - curveXMinStep ;
			if( newP.y > cvMax )	curv->setRealMax( newP.y ) ;
			if( newP.y < cvMin )	curv->setRealMin( newP.y ) ;

			oscSelCurve[nb] = newP ;
			curv->setCurve( oscSelCurve ) ;
			curv->evalBoundingBox( cvP, BBsize ) ;
			controlCenter->updateParameters( Curve, inID ) ;
		}
	}
	else printf("Curve doesn't exist\n") ;
}
void NxGLScore::oscCurveAddPt( int inID, float inX, float inY )
{
	if( score->isThereCurve( inID ) ) { // Test of existence
		printf("Curve Point Add\n") ;
		if( inX>0 ) { // we don't do anything if x < 0
			NxCurve *curv = score->getCurve( inID ) ;
			QVector<NxPoint> oscSelCurve = curv->getCurve() ;
		
			NxPoint newP ;
			newP.z = 0 ;
			int initialSize = curv->size() ;
			int newID ;
			int localID = -1 ;
			bool IDfound = false ;
			float cvMin		= curv->getRealMin() ;
			float cvMax		= curv->getRealMax() ;
			NxPoint cvP		= curv->getPosition() ;
		
			while(!IDfound) { /////// Search new point location
				localID++ ;
				if( (inX < oscSelCurve[localID].x ) || (localID == initialSize) ) IDfound = true ;
			}
			
			////// we don't do anything if there isn't any x-space left
			if( fabs( oscSelCurve[localID-1].x-oscSelCurve[localID].x )>=curveXMinStep*2 ) 
			{
				oscSelCurve.append(newP) ; // enlarge the array size
			
				newID = localID ;
		
				for(int l=initialSize-1 ; l>=newID ; l--) {
					oscSelCurve[l+1] = oscSelCurve[l];
					oscSelCurve[l+1].ID = l+1 ;
				}

				newP.x = inX ;
				newP.y = inY ;
		
				//////////////////// Constrain the position
				if( newID != 0 )
					if (newP.x < oscSelCurve[newID-1].x + curveXMinStep )	newP.x = oscSelCurve[newID-1].x + curveXMinStep ;
				if( (newID!=initialSize-1) && (newID!=initialSize) )
					if (newP.x > oscSelCurve[newID+1].x - curveXMinStep )	newP.x = oscSelCurve[newID+1].x - curveXMinStep ;
				if( newP.y > cvMax )	curv->setRealMax( newP.y ) ;
				if( newP.y < cvMin )	curv->setRealMin( newP.y ) ;
		
				newP.ID = newID ;
				oscSelCurve[newID] = newP ;
		
				curv->setCurve( oscSelCurve );
				curv->evalBoundingBox( cvP, BBsize ) ;
				controlCenter->updateParameters( Curve, inID ) ;
			}
			else printf("No space left for that x - point wasn't added on the curve\n") ;
		}
		else printf("The point has a negative abscisse - it wasn't added on the curve\n") ;
	}
	else printf("Curve doesn't exist\n") ;
}
void NxGLScore::oscCurveRmPt( int inID, int inN )
{
	if( score->isThereCurve( inID ) ) { // Test of existence
		printf("Curve Point Removing\n") ;
		NxCurve *curv = score->getCurve( inID ) ;
		
		if ( curv->size() > 2 && inN>0 ) { // do not remove if nb < 2 or first point
			QVector<NxPoint> oscSelCurve = curv->getCurve() ;
			NxPoint newPt	= curv->getPosition() ; // curve position
			int initialSize = curv->size() ;
			
			for(int l=inN ; l<initialSize-1 ; l++) {
				oscSelCurve[l] = oscSelCurve[l+1];
				oscSelCurve[l].ID = l ;
			}
			
			oscSelCurve.remove(initialSize-1) ;
			curv->setCurve( oscSelCurve );
			curv->evalBoundingBox( newPt, BBsize ) ;
			controlCenter->updateParameters( Curve, inID ) ;
		}
	}
	else printf("Curve doesn't exist\n") ;
}
void NxGLScore::oscCurveAngle( int inID, float inAngle )
{
	if( score->isThereCurve( inID ) ) { // Test of existence
		((NxCurve*)score->getCurve( inID ))->setAngle( inAngle ) ;
		controlCenter->updateParameters( Curve, inID ) ;
	}
	else printf("OSC Curve Angle - Curve doesn't exist\n") ;
}
void NxGLScore::oscCurveScale( int inID, float inScale )
{
	if( score->isThereCurve( inID ) ) { // Test of existence
		NxCurve *scCurv = score->getCurve( inID ) ;
		scCurv->setScale( inScale ) ;
		scCurv->evalBoundingBox( scCurv->getPosition() ,  BBsize  ) ;
		controlCenter->updateParameters( Curve, inID ) ;
	}
	else printf("OSC Curve Scale - Curve doesn't exist\n") ;
}
void NxGLScore::oscCurveStyle( int inID, int inStyle )
{
	if( score->isThereCurve( inID ) ) { // Test of existence
		NxCurve *scCurv = score->getCurve( inID ) ;
		if(inStyle==1)	scCurv->setDrawType( SplineStrip ) ;
		else			scCurv->setDrawType( LineStrip ) ;
		controlCenter->updateParameters( Curve, inID ) ;
	}
	else printf("OSC Curve Drawing Style - Curve doesn't exist\n") ;
}
void NxGLScore::oscCurveMin( int inID, float inMin )
{
	if( score->isThereCurve( inID ) ) { // Test of existence
		((NxCurve*)score->getCurve( inID ))->setRealMin( inMin ) ;
		controlCenter->updateParameters( Curve, inID ) ;
	}
	else printf("OSC Curve Min - Curve doesn't exist\n") ;
}
void NxGLScore::oscCurveMax( int inID, float inMax )
{
	if( score->isThereCurve( inID ) ) { // Test of existence
		((NxCurve*)score->getCurve( inID ))->setRealMax( inMax ) ;
		controlCenter->updateParameters( Curve, inID ) ;
	}
	else printf("OSC Curve Max - Curve doesn't exist\n") ;
}
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////// TRAJECTS PARAMETERS
void NxGLScore::oscNewCircle( float inX, float inY, float inZ, float inRadius, int inCID, int inGID, int inR, int inG, int inB )
{
	NxPoint inPos ;
	inPos.ID = 0 ;
	inPos.x = inX ;
	inPos.y = inY ;
	inPos.z = inZ ;
	Color inColor ;
	inColor.ID = 0 ;
	inColor.c = QColor(inR,inG,inB) ;
	
	int inID = score->getCptTraject() ;
	int inLay = 0 ;
	QString inName = QString("oscCircle") ;
	
	addNewTraject( inPos, inID, inCID, inGID, inLay, inName, inColor, 1, 0, inRadius ) ;
	score->maxOfIndexes() ;
}
void NxGLScore::oscNewLine( float inX1, float inY1, float inZ1, float inX2, float inY2, float inZ2, int inCID, int inGID, int inR, int inG, int inB )
{
	NxPoint inPos1, inPos2 ;
	inPos1.ID = inPos2.ID = 0 ;
	inPos1.x = inX1 ;
	inPos1.y = inY1 ;
	inPos1.z = inZ1 ;
	inPos2.x = inX2 ;
	inPos2.y = inY2 ;
	inPos2.z = inZ2 ;
	Color inColor ;
	inColor.ID = 0 ;
	inColor.c = QColor(inR,inG,inB) ;

	int inID = score->getCptTraject() ;
	int inLay = 0 ;
	QString inName = QString("oscLine") ;
	
	score->addTraject( inPos1, inID, inCID, inGID, inLay, inName, inColor, 2 );
	score->getTraject( inID )->setOrientation( 4 ) ; // Free Line
	score->getTraject( inID )->setLineDef( inPos1, inPos2 ) ;
	updateIndexes();
	controlCenter->updateItems( 1, true, inID );
	
	score->maxOfIndexes() ;
}
void NxGLScore::oscTrajectRadius( int inID, float inRadius )
{
	if( score->isThereTraject( inID ) && inRadius!=0 ) { // Test of existence
		if( ((NxTraject*)score->getTraject( inID ))->getType() == 1 ) { // Circle only
			NxTraject* radTraj = score->getTraject( inID ) ;
			Circle newC ;
			newC.radius = inRadius ;
			newC.p = radTraj->getPosition() ;
			radTraj->setCircleDef( newC ) ; 
			radTraj->evalBoundingBox( radTraj->getPosition() , 0 ) ;
			/////////////// Adjust cursors offsets ///////////////
			NxCursor *cur ;
			QList<int> indexes = score->cursorsIndexes();
			for( int i=0; i< nbCursors; i++ ) {	
				cur = score->getCursor( indexes.at(i) ) ;
				if ( cur->getParentTrajectory()->getID() == inID )	
					cur->reEvalOffsets() ;
			}
			/////////////////////////////////////////////////////
			controlCenter->updateParameters( Trajectory, inID ) ;
		}
		else printf("OSC Radius - Line ?? \n") ;
	}
	else printf("OSC Radius - Circle doesn't exist (or = 0)\n") ;
}
void NxGLScore::oscTrajectLength( int inID, float inLength )
{
	if( score->isThereTraject( inID ) && inLength!=0 ) { // Test of existence
		NxTraject* lenTraj = score->getTraject( inID ) ;
		
		lenTraj->setLength( fabs(inLength) ) ;
	
		lenTraj->evalBoundingBox( lenTraj->getPosition() ,  BBsize  ) ;
		/////////////// Adjust cursors offsets ///////////////
		NxCursor *cur ;
		QList<int> indexes = score->cursorsIndexes();
		for( int i=0; i< nbCursors; i++ ) {	
			cur = score->getCursor( indexes.at(i) ) ;
			if ( cur->getParentTrajectory()->getID() == inID )	
				cur->reEvalOffsets() ;
		}
		/////////////////////////////////////////////////////	
		controlCenter->updateParameters( Trajectory, inID ) ;
		assignCursors() ;
	}
	else printf("OSC Length - Traject doesn't exist (or = 0)\n") ;
}
void NxGLScore::oscTrajectPoint( int inID, int firstPt, float inX, float inY, float inZ )
{
	if( score->isThereTraject( inID ) ) { // Test of existence
		if( ((NxTraject*)score->getTraject( inID ))->getType() == 2 ) { // Line only
			NxPoint inPos ;
			inPos.ID = 0 ;
			inPos.x = inX ;
			inPos.y = inY ;
			inPos.z = inZ ;
		
			switch(firstPt) {
				case 0: // first point
					((NxTraject*)score->getTraject( inID ))->setInitialPoint( inPos ) ;
				break ;
				case 1: // 2nd point
					((NxTraject*)score->getTraject( inID ))->setFinalPoint( inPos ) ;
				break ;
			}
		}
		else printf("OSC Line Pts - Circle ?? \n") ;
	}
	else printf("OSC Line Pts - Line doesn't exist\n") ;
}
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////// CURSORS PARAMETERS
void NxGLScore::oscNewCursor( int inID, int inCID, int inGID, float inSpeed, float inWidth, int inForward, int inMode, int inR, int inG, int inB )
{
	if( score->isThereTraject( inID ) ) { // Test of existence
		NxTraject *pTraj = score->getTraject( inID ) ; // Parent
		
		NxPoint inPos ;
		inPos.ID = 0 ;
		inPos.x = inPos.y = inPos.z = 0.0 ; // Init only
		Color inColor ;
		inColor.ID = 0 ;
		inColor.c = QColor(inR,inG,inB) ;
		
		int inID = score->getCptCursor() ;
		int inLay = pTraj->getLayer() ;
		QString inName = QString("oscCursor") ;
		bool forw = (inForward==1) ;
		float inOff1 = 0.0 ;
		float inOff2 = pTraj->getLength() ;
		
		pause() ; // Stop playback
		
		score->addCursor( inPos, inID, inCID, pTraj, inGID, inLay, inName, inColor, forw, inMode, inWidth, inOff1, inOff2 );
		score->getCursor(inID)->reEvalOffsets() ;
		score->getCursor(inID)->setSpeed( inSpeed );
		score->getCursor(inID)->setWidth( inWidth );
		score->getCursor(inID)->evalBoundingBox(inPos, BBsize ) ;
		updateIndexes();
		assignCursors();
		controlCenter->updateItems( 2, true, inID );	
		timeScheduler->addNewCursor() ;		
		score->maxOfIndexes() ;
	}
	else printf("OSC New Cursor - Traject doesn't exist\n") ;
}
void NxGLScore::oscCursorOffset( int inID, float inOffsetPC )
{
	if( score->isThereCursor( inID ) ) { // Test of existence
		float len = ((NxTraject*)((NxCursor*)score->getCursor( inID ))->getParentTrajectory())->getLength() ;
		((NxCursor*)score->getCursor( inID ))->setOffset( inOffsetPC * len / 100.0 ) ;
	}
	else printf("OSC Offset - Cursor doesn't exist\n") ;
}
void NxGLScore::oscCursorForward( int inID, int playb )
{
	if( score->isThereCursor( inID ) && (playb==0||playb==1) ) { // Test of existence
		((NxCursor*)score->getCursor( inID ))->setForward( (playb==1) ) ;
		controlCenter->updateParameters( Cursor, inID ) ;
	}
	else printf("OSC Forward - Cursor doesn't exist (or wrong value)\n") ;
}
void NxGLScore::oscCursorMode( int inID, int mode )
{
	if( score->isThereCursor( inID ) && (mode==0||mode==1||mode==2) ) { // Test of existence
		((NxCursor*)score->getCursor( inID ))->setMode( mode ) ;
		controlCenter->updateParameters( Cursor, inID ) ;
	}
	else printf("OSC Mode - Cursor doesn't exist (or wrong value)\n") ;
}
void NxGLScore::oscCursorWidth( int inID, float wid )
{
	if( score->isThereCursor( inID ) ) { // Test of existence
		((NxCursor*)score->getCursor( inID ))->setWidth( wid ) ;
		controlCenter->updateParameters( Cursor, inID ) ;
	}
	else printf("OSC Width - Cursor doesn't exist\n") ;
}
void NxGLScore::oscCursorSpeed( int inID, float speed )
{
	if( score->isThereCursor( inID ) ) { // Test of existence
		((NxCursor*)score->getCursor( inID ))->setSpeed( speed ) ;
		controlCenter->updateParameters( Cursor, inID ) ;
	}
	else printf("OSC Speed - Cursor doesn't exist\n") ;
}
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////// OBJET GENERAL PARAMETERS
void NxGLScore::oscDeleteObject( int type, int inID ) 
{
	switch(type) {
		case Trajectory:
			if( score->isThereTraject( inID ) )
				destroyObject( (NxTraject*)score->getTraject( inID ) ) ;
			else printf("OSC rm - Traject doesn't exist\n") ;
		break ;
		case Cursor:
			if( score->isThereCursor( inID ) )
				destroyObject( (NxCursor*)score->getCursor( inID ) ) ;
			else printf("OSC rm - Cursor doesn't exist\n") ;
		break ;
		case Curve:
			if( score->isThereCurve( inID ) )
				destroyObject( (NxCurve*)score->getCurve( inID ) ) ;
			else printf("OSC rm - Curve doesn't exist\n") ;
		break ;
		case Trigger:
			if( score->isThereTrigger( inID ) )
				destroyObject( (NxTrigger*)score->getTrigger( inID ) ) ;
			else printf("OSC rm - Trigger doesn't exist\n") ;
		break ;
	}
}
void NxGLScore::oscObjectLayer( int type, int inID, int inLay )
{
	switch(type) {
		case Trajectory:
			if( score->isThereTraject( inID ) ) {
				score->getTraject( inID )->setLayer( inLay ) ;
				controlCenter->updateParameters( Trajectory, inID ) ;
			}
			else printf("OSC Layer - Trajectory doesn't exist\n") ;
		break ;
		case Cursor:
			if( score->isThereCursor( inID ) ) {
				score->getCursor( inID )->setLayer( inLay ) ;
				controlCenter->updateParameters( Cursor, inID ) ;
			}
			else printf("OSC Layer - Cursor doesn't exist\n") ;
		break ;
		case Curve:
			if( score->isThereCurve( inID ) ) {
				score->getCurve( inID )->setLayer( inLay ) ;
				controlCenter->updateParameters( Curve, inID ) ;
			}
			else printf("OSC Layer - Curve doesn't exist\n") ;
		break ;
		case Trigger:
			if( score->isThereTrigger( inID ) ) {
				score->getTrigger( inID )->setLayer( inLay ) ;
				controlCenter->updateParameters( Trigger, inID ) ;
			}
			else printf("OSC Layer - Trigger doesn't inLay\n") ;
		break ;
	}
}
void NxGLScore::oscObjectName( int type, int inID, const char *inName )
{
	QString inNam = QString( inName ) ;
	switch(type) {
		case Trajectory:
			if( score->isThereTraject( inID ) ) {
				score->getTraject( inID )->setName( inNam ) ;
				controlCenter->updateParameters( Trajectory, inID ) ;
			}
			else printf("OSC Name - Trajectory doesn't exist\n") ;
		break ;
		case Cursor:
			if( score->isThereCursor( inID ) ) {
				score->getCursor( inID )->setName( inNam ) ;
				controlCenter->updateParameters( Cursor, inID ) ;
			}
			else printf("OSC Name - Cursor doesn't exist\n") ;
		break ;
		case Curve:
			if( score->isThereCurve( inID ) ) {
				score->getCurve( inID )->setName( inNam ) ;
				controlCenter->updateParameters( Curve, inID ) ;
			}
			else printf("OSC Name - Curve doesn't exist\n") ;
		break ;
		case Trigger:
			if( score->isThereTrigger( inID ) ) {
				score->getTrigger( inID )->setName( inNam ) ;
				controlCenter->updateParameters( Trigger, inID ) ;
			}
			else printf("OSC Name - Trigger doesn't exist\n") ;
		break ;
	}
}
void NxGLScore::oscObjectCID( int type, int inID, int inCID )
{
	switch(type) {
		case Trajectory:
			if( score->isThereTraject( inID ) ) {
				score->getTraject( inID )->setCustomID( inCID ) ;
				controlCenter->updateParameters( Trajectory, inID ) ;
			}
			else printf("OSC CID - Trajectory doesn't exist\n") ;
		break ;
		case Cursor:
			if( score->isThereCursor( inID ) ) {
				score->getCursor( inID )->setCustomID( inCID ) ;
				controlCenter->updateParameters( Cursor, inID ) ;
			}
			else printf("OSC CID - Cursor doesn't exist\n") ;
		break ;
		case Curve:
			if( score->isThereCurve( inID ) ) {
				score->getCurve( inID )->setCustomID( inCID ) ;
				controlCenter->updateParameters( Curve, inID ) ;
			}
			else printf("OSC CID - Curve doesn't exist\n") ;
		break ;
		case Trigger:
			if( score->isThereTrigger( inID ) ) {
				score->getTrigger( inID )->setCustomID( inCID ) ;
				controlCenter->updateParameters( Trigger, inID ) ;
			}
			else printf("OSC CID - Trigger doesn't exist\n") ;
		break ;
	}
}
void NxGLScore::oscObjectGID( int type, int inID, int inGID )
{
	switch(type) {
		case Trajectory:
			if( score->isThereTraject( inID ) ) {
				score->getTraject( inID )->setGroupID( inGID ) ;
				controlCenter->updateParameters( Trajectory, inID ) ;
			}
			else printf("OSC GID - Trajectory doesn't exist\n") ;
		break ;
		case Cursor:
			if( score->isThereCursor( inID ) ) {
				score->getCursor( inID )->setGroupID( inGID ) ;
				controlCenter->updateParameters( Cursor, inID ) ;
			}
			else printf("OSC GID - Cursor doesn't exist\n") ;
		break ;
		case Curve:
			if( score->isThereCurve( inID ) ) {
				score->getCurve( inID )->setGroupID( inGID ) ;
				controlCenter->updateParameters( Curve, inID ) ;
			}
			else printf("OSC GID - Curve doesn't exist\n") ;
		break ;
		case Trigger:
			if( score->isThereTrigger( inID ) ) {
				score->getTrigger( inID )->setGroupID( inGID ) ;
				controlCenter->updateParameters( Trigger, inID ) ;
			}
			else printf("OSC GID - Trigger doesn't exist\n") ;
		break ;
	}
}
void NxGLScore::oscObjectPosition( int type, int inID, float inx, float iny, float inz )
{
	NxPoint newPos ;
	newPos.ID = 0 ;
	newPos.x = inx ;
	newPos.y = iny ;
	newPos.z = inz ;
	switch(type) {
		case Trajectory:
			if( score->isThereTraject( inID ) ) {
				score->getTraject( inID )->setPosition( newPos ) ;
				score->getTraject( inID )->evalBoundingBox( newPos, BBsize ) ; }
			else printf("OSC pos - Trajectory doesn't exist\n") ;
		break ;
		case Cursor:
			printf("OSC pos - Cursor's position cannot be modified\n") ;
		break ;
		case Curve:
			if( score->isThereCurve( inID ) ) {
				score->getCurve( inID )->setPosition( newPos ) ;
				score->getCurve( inID )->evalBoundingBox( newPos, BBsize ) ; }
			else printf("OSC pos - Curve doesn't exist\n") ;
		break ;
		case Trigger:
			if( score->isThereTrigger( inID ) ) {
				score->getTrigger( inID )->setPosition( newPos ) ;
				score->getTrigger( inID )->evalBoundingBox( newPos, BBsize ) ; }
			else printf("OSC pos - Trigger doesn't exist\n") ;
		break ;
	}
}
void NxGLScore::oscObjectColor( int type, int inID, int inR, int inG, int inB )
{
	Color newCol ;
	newCol.ID = 0 ;
	newCol.c = QColor( inR, inG, inB ) ;
	switch(type) {
		case Trajectory:
			if( score->isThereTraject( inID ) )
				score->getTraject( inID )->setColor( newCol ) ;
			else printf("OSC Color - Trajectory doesn't exist\n") ;
		break ;
		case Cursor:
			if( score->isThereCursor( inID ) )
				score->getCursor( inID )->setColor( newCol ) ;
			else printf("OSC Color - Cursor doesn't exist\n") ;
		break ;
		case Curve:
			if( score->isThereCurve( inID ) )
				score->getCurve( inID )->setColor( newCol ) ;
			else printf("OSC Color - Curve doesn't exist\n") ;
		break ;
		case Trigger:
			if( score->isThereTrigger( inID ) )
				score->getTrigger( inID )->setColor( newCol ) ;
			else printf("OSC Color - Trigger doesn't exist\n") ;
		break ;
	}
}
void NxGLScore::oscObjectActivity( int type, int inID, int chooser )
{
	switch(type) {
		case Trajectory:
			if( score->isThereTraject( inID ) )
				score->getTraject( inID )->setActive( (chooser==1) ) ;
			else printf("OSC Act - Trajectory doesn't exist\n") ;
		break ;
		case Cursor:
			if( score->isThereCursor( inID ) )
				score->getCursor( inID )->setActive( (chooser==1) ) ;
			else printf("OSC Act - Cursor doesn't exist\n") ;
		break ;
		case Curve:
			if( score->isThereCurve( inID ) )
				score->getCurve( inID )->setActive( (chooser==1) ) ;
			else printf("OSC Act - Curve doesn't exist\n") ;
		break ;
		case Trigger:
			if( score->isThereTrigger( inID ) )
				score->getTrigger( inID )->setActive( (chooser==1) ) ;
			else printf("OSC Act - Trigger doesn't exist\n") ;
		break ;
	}
}
void NxGLScore::oscObjectHost( int type, int inID, const char *inHost )
{
	QString inHos = QString( inHost ) ;
	switch(type) {
		case Curve:
			if( score->isThereCurve( inID ) ) {
				score->getCurve( inID )->setHost( inHos ) ;
				controlCenter->updateParameters( Curve, inID ) ;
			}
			else printf("OSC Host - Curve doesn't exist\n") ;
		break ;
		case Trigger:
			if( score->isThereTrigger( inID ) ) {
				score->getTrigger( inID )->setHost( inHos ) ;
				controlCenter->updateParameters( Trigger, inID ) ;
			}
			else printf("OSC Host - Trigger doesn't exist\n") ;
		break ;
	}
}
void NxGLScore::oscObjectPort( int type, int inID, int inPort )
{
	switch(type) {
		case Curve:
			if( score->isThereCurve( inID ) ) {
				score->getCurve( inID )->setPort( inPort ) ;
				controlCenter->updateParameters( Curve, inID ) ;
			}
			else printf("OSC Port - Curve doesn't exist\n") ;
		break ;
		case Trigger:
			if( score->isThereTrigger( inID ) ) {
				score->getTrigger( inID )->setPort( inPort ) ;
				controlCenter->updateParameters( Trigger, inID ) ;
			}
			else printf("OSC Port - Trigger doesn't exist\n") ;
		break ;
	}
}
void NxGLScore::oscShowHideObject( int type, int chooser)
{
	switch(type) {
		case Trajectory:
			showTrajects = (chooser==1) ;
		break ;
		case Cursor:
			showCursors = (chooser==1) ;
		break ;
		case Curve:
			showCurves = (chooser==1) ;
		break ;
		case Trigger:
			showTriggers = (chooser==1) ;
		break ;
	}
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////
// OpenGL Render Method
////////////////////////////////////
void NxGLScore::paintGL()
{
	QFont textFont("Verdana") ;
	
	mouseXPixPos = int(floor(timePosPix.x)) ;
	mouseYPixPos = int(floor(timePosPix.y)) ;
	
	glClearColor( bColor.c.red()/255.0, bColor.c.green()/255.0, bColor.c.blue()/255.0, bColor.c.alpha()/255.0 ) ;
  	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  	glLoadIdentity( );
  	glTranslatef( decX, decY, -10.0 );
	
	/*printf("\n\n///////////////////// DEBUG ////// PRINT //////////////////////////\n") ;
	printf("decXdecY : %f et %f\n", decX, decY) ;
	// bX = length / (RScaleX*scaleX*10.0) ;
  	// bY = nHeight / (RScaleY*scaleY*10.0) ;
	printf("Taille fenetre en reel /2 : bx by : %f et %f\n", bX, bY) ;
	// nHeight = (float)length*initSelfH/initSelfW
	printf("nHeight : %f\n", nHeight) ;
	printf("trans : %f et %f\n", transX, transY ) ;
	printf("real trans : %f et %f\n", transX/(RScaleX*scaleX), transY/(RScaleY*scaleY) ) ;
	printf("--------\n") ;
	printf("p : %f et %f\n", p.x, p.y) ;
	printf("timePos : %f et %f\n", timePos.x, timePos.y) ;
	//printf("timePosPix : %f et %f\n", timePosPix.x, timePosPix.y) ;
	printf("mousePixPos : %d et %d\n", mouseXPixPos, mouseYPixPos) ;
	printf("------------------------\n") ;
	printf("scale : %f et %f\n", scaleX, scaleY) ;
	//selfW = (GLint)wf;
	//selfH = (GLint)hf;
	printf("TAILLE : %d et %d\n", selfW, selfH) ;
	printf("TAILLE INITIALE : %d et %d\n", initSelfW, initSelfH) ;
	//RScaleX = (float)initSelfW/selfW ;
	//RScaleY = (float)initSelfH/selfH ;
	printf("Rscale : %f et %f\n", RScaleX, RScaleY) ;*/
	
	glRotatef( angleX, 1, 0, 0 ) ; 	
  	glRotatef( angleY, 0, 1, 0 ) ;
  	glScalef( (float)RScaleX*scaleX, (float)RScaleY*scaleY, scaleZ );
	glTranslatef( -(float)length/2, -(float)height/2, 0 );
	
  	////////////////////
  	// Score Box
  	////////////////////
	if (scoreDraw)
	{
		//Rectangle
		glColor4fv( colorConv( sColor ) );
		glBegin( GL_QUADS );
		glVertex3f( 0, 0, 0.0 );
		glVertex3f( 0, height, 0.0 );
		glVertex3f( length, height, 0.0 );
		glVertex3f( length, 0, 0.0 );
		glEnd();
		//Contour
		GLfloat colorFond2[] 	= {1.0, 1.0, 1.0, 0.4};	
		glColor4fv( colorFond2 );
		glBegin( GL_LINE_LOOP );
		glVertex3f( 0, 0, 0.0 );
		glVertex3f( 0, height, 0.0 );
		glVertex3f( length, height, 0.0 );
		glVertex3f( length, 0, 0.0 );
		glEnd();
		//Red (0,0) cross
		GLfloat colorFond3[] 	= {1.0, 0, 0, 1.0};	
		glColor4fv( colorFond3 );
		glBegin( GL_LINES );
		glVertex3f( 0, 0 - length/(160*scaleX), 0.0 );
		glVertex3f( 0, 0 + length/(160*scaleX), 0.0 );
		glVertex3f( 0 - length/(160*scaleX), 0, 0.0 );
		glVertex3f( 0 + length/(160*scaleX), 0, 0.0 );
		glEnd();
	}

	//////// draw a grid on the score ////////
	if(showGrid) { // TODO must be better (because of bad performances while big zoom -> big for()-loop...)
		if(autoGridStep) {
			if(gridStep > 0.01) gridStepInt = (int)floor(gridStep*100) ;
			else gridStepInt = 1 ;
		}
		//TODO glTranslatef( gridDecX, gridDecY, 0 );
		glColor4f((GLfloat)gColor.c.red()/255, (GLfloat)gColor.c.green()/255, (GLfloat)gColor.c.blue()/255, gridOpacity);
		glBegin(GL_LINES);
		for(int i=0; i<=height*100.0; i+= (int)gridStepInt) {
			glVertex2f(0, i/100.0);
			glVertex2f(length, i/100.0);
		}
		for(int j=0; j<=length*100.0; j+= (int)gridStepInt) {
			glVertex2f(j/100.0, 0);
			glVertex2f(j/100.0, height);
		}
		glEnd() ;
		if(snapGrid)
		{
			glColor4f(1.0, 1.0, 1.0, gridOpacity*4);
			glBegin(GL_POINTS);
			for (int i=0; i<=height*100.0; i+= (int)gridStepInt) {
				for (int j=0; j<=length*100.0; j+= (int)gridStepInt) {
					glVertex2f(j/100.0, i/100.0);
				}	
			}
			glEnd() ;
		}
		//TODO glTranslatef( -gridDecX, -gridDecY, 0 );
	}
	
  	/////////////////////
  	// Zoom Box
  	/////////////////////
  	if (zoomMode && onDrag )
	{
		glColor4f(1.0, 0.7, 0.0, 0.8);
		glBegin( GL_LINE_LOOP );
		glVertex3f( clickPos.x, clickPos.y, 0.0 );
		glVertex3f( clickPos.x, p.y, 0.0 );
		glVertex3f( p.x, p.y, 0.0 );
		glVertex3f( p.x, clickPos.y, 0.0 );
		glEnd();
	}
	
  	/////////////////////////////////////
  	// Rendering Objects
  	/////////////////////////////////////
  	if(showTriggers) renderTriggers();
  	renderTrajects();
  	if(showCurves) renderCurves();

  	////////////////////////////////////////////////////////////////	
  	// SPECIAL CASE : CURVE EDITION - 1st point
  	/////////////////////////////////////////////////////////////// 
  	if (firstPoint) {
		float dimPt = (float)length/(1500*scaleX) ;
		glBegin( GL_QUADS );
		glVertex3f( firstCurvPos.x-dimPt, firstCurvPos.y-dimPt, 0.0 );
		glVertex3f( firstCurvPos.x+dimPt, firstCurvPos.y-dimPt, 0.0 );
		glVertex3f( firstCurvPos.x+dimPt, firstCurvPos.y+dimPt, 0.0 );
		glVertex3f( firstCurvPos.x-dimPt, firstCurvPos.y+dimPt, 0.0 );
		glEnd();
	}
  	////////////////////////////////////////////////////////////////	
  	// MOVE A POINT ON A CURVE
  	/////////////////////////////////////////////////////////////// 	
	if (curveEdition && curvEdPtChoosed && pointID!=-1 && !curvEdOnlyClic) { 
		glColor4f(1.0, 1.0, 1.0, 1.0);
		float dimPt = (float)length/(1500*scaleX) ;
		glBegin( GL_LINES );
		if( !curvEdfirst ) {
			glVertex3f( curvEd1x, curvEd1y, 0.0 ) ;
			glVertex3f( tempP.x + curvEdPosX, timePos.y, 0.0 ) ;
		}
		if( !curvEdlast ) {
			glVertex3f( tempP.x + curvEdPosX, timePos.y, 0.0 ) ;
			glVertex3f( curvEd2x, curvEd2y, 0.0 ) ;
			}
		glEnd();
		glBegin( GL_QUADS );
		glVertex3f( tempP.x + curvEdPosX-dimPt, timePos.y-dimPt, 0.0 );
		glVertex3f( tempP.x + curvEdPosX+dimPt, timePos.y-dimPt, 0.0 );
		glVertex3f( tempP.x + curvEdPosX+dimPt, timePos.y+dimPt, 0.0 );
		glVertex3f( tempP.x + curvEdPosX-dimPt, timePos.y+dimPt, 0.0 );
		glEnd();
	}
  	////////////////////////////////////////////////////////////////	
  	// LINE RULE RENDER
  	///////////////////////////////////////////////////////////////
  	if (lineRuleMode)
	{
		glColor4f( 1.0, 0.5, 0.0, 1.0 );
		glBegin( GL_LINES);
		glVertex3f( lineRulePt1.x, lineRulePt1.y, 0.0 );
		glVertex3f( timePos.x, timePos.y, 0.0 );
		glEnd();
	}
  	////////////////////////////////////////////////////////////////	
  	// TEXT RENDERING
  	///////////////////////////////////////////////////////////////
  	glColor4f( 1.0, 1.0, 1.0, 1.0 );
	if ( displayOn )
	{	
		renderText( selfW - 210, selfH - 10, QString("Grid Step : %1").arg( gridStepInt/100.0 ), textFont ) ;
		renderText( selfW - 100, selfH - 50, QString("X = %1 s").arg(timePos.x), textFont );
		renderText( selfW - 100, selfH - 40, QString("Y = %1 s").arg(timePos.y), textFont );
		renderText( selfW - 100, selfH - 30, QString("Z = %1 s").arg(0), textFont );
			
		//////////// Angle display (in 3D mode only)
		if (frustum)
		{
			renderText( selfW - 100, selfH - 10, QString("3D Mode"), textFont ); 
			renderText( selfW - 100, selfH - 80, QString("Angle (X) : %1").arg(angleX), textFont ); 
			renderText( selfW - 100, selfH - 70, QString("Angle (Y) : %1").arg(angleY), textFont ); 		 
		}
		else
			renderText( selfW - 100, selfH - 10, QString("2D Mode"), textFont ); 
		
		////////////////////////////////
		// SELECTION DISPLAY
		////////////////////////////////
		if (selected != NULL)
		{	
			switch ( selected->getObjectType() ) {
				case 0: ////// CURVE
					renderText( 10, selfH - 170, QString("Curve : %1").arg( selected->getName() ), textFont ) ; 	
					renderText( 10, selfH - 150, QString("Real ID (unique) : %1").arg( selected->getID() ), textFont ) ;
					renderText( 10, selfH - 140, QString("Custom ID : %1").arg( selected->getCustomID() ), textFont ) ;
					renderText( 10, selfH - 130, QString("Group ID : %1").arg( selected->getGroupID() ), textFont ) ;
					renderText( 10, selfH - 120, QString("Layer : %1").arg( selected->getLayer() ), textFont ) ;
					renderText( 10, selfH - 100, QString("OSC : %1:%2").arg( ((NxCurve*)selected)->getHost() ).arg( ((NxCurve*)selected)->getPort() ), textFont );
					renderText( 10, selfH - 90, QString("Scan Period : %1 ms").arg( ( (NxCurve*)selected)->getSpatialPeriod() ), textFont ) ;
					renderText( 10, selfH - 70, QString("Control Points : %1").arg( ((NxCurve*)selected)->size()), textFont ) ;
					renderText( 10, selfH - 60, QString("Values in [ %1 , %2 ]").arg( ((NxCurve*)selected)->getRealMin() ).arg( ((NxCurve*)selected)->getRealMax() ), textFont ) ;
					renderText( 10, selfH - 50, QString("Duration : %1 s").arg( ( (NxCurve*)selected)->getXRange() ), textFont ) ;
					renderText( 10, selfH - 30, QString("X: %1").arg( ((NxCurve*)selected)->getPosition().x), textFont );
					renderText( 10, selfH - 20, QString("Y: %1").arg( ((NxCurve*)selected)->getPosition().y), textFont );
					renderText( 10, selfH - 10, QString("Z: %1").arg( ((NxCurve*)selected)->getPosition().z), textFont );	
				break;
				case 1: ////// TRAJECTORY
					renderText( 10, selfH - 140, QString("Trajectory : %1").arg( selected->getName() ), textFont ) ; 	
					renderText( 10, selfH - 120, QString("Real ID (unique) : %1").arg( selected->getID() ), textFont ) ;
					renderText( 10, selfH - 110, QString("Custom ID : %1").arg( selected->getCustomID() ), textFont ) ;
					renderText( 10, selfH - 100, QString("Group ID : %1").arg( selected->getGroupID() ), textFont ) ;
					renderText( 10, selfH - 90, QString("Layer : %1").arg( selected->getLayer() ), textFont ) ;
					renderText( 10, selfH - 70, QString("Child Cursors : %1").arg( ((NxTraject*)selected)->getNbCursors() ), textFont );
					renderText( 10, selfH - 30, QString("X: %1").arg( ((NxTraject*)selected)->getPosition().x), textFont );
					renderText( 10, selfH - 20, QString("Y: %1").arg( ((NxTraject*)selected)->getPosition().y), textFont );
					renderText( 10, selfH - 10, QString("Z: %1").arg( ((NxTraject*)selected)->getPosition().z), textFont );
					if ( ((NxTraject*)selected)->getType() == 1) // Circle
						renderText( 10, selfH - 50, QString("Radius: %1").arg( ((NxTraject*)selected)->getCircle().radius), textFont ); 
					renderText( 10, selfH - 60, QString("Length: %1").arg( ((NxTraject*)selected)->getLength()), textFont ); 
				break;	
				case 2: ////// CURSOR
					renderText( 10, selfH - 110, QString("Cursor : %1").arg( selected->getName() ) , textFont ) ; 	
					renderText( 10, selfH - 90, QString("Real ID (unique) : %1").arg( selected->getID() ), textFont ) ;
					renderText( 10, selfH - 80, QString("Custom ID : %1").arg( selected->getCustomID() ), textFont ) ;
					renderText( 10, selfH - 70, QString("Group ID : %1").arg( selected->getGroupID() ), textFont ) ;
					renderText( 10, selfH - 60, QString("Layer : %1").arg( selected->getLayer() ), textFont ) ;
					renderText( 10, selfH - 40, QString("Parent ID : %1").arg( ((NxCursor*)selected)->getParentTrajectory()->getID() ), textFont );  
					renderText( 10, selfH - 20, QString("Offset = %1 s").arg( ((NxCursor*)selected)->getOffset() ), textFont ); 
					renderText( 10, selfH - 10, QString("[ %1 - %2 ]").arg( ((NxCursor*)selected)->getInitOffset() ).arg( ((NxCursor*)selected)->getFinalOffset() ), textFont );   
				break;	
				case 3: ////// TRIGGER
					renderText( 10, selfH - 120, QString("Trigger : %1").arg( selected->getName() ), textFont ) ;
					renderText( 10, selfH - 100, QString("Real ID (unique) : %1").arg( selected->getID() ), textFont ) ;
					renderText( 10, selfH - 90, QString("Custom ID : %1").arg( selected->getCustomID() ), textFont ) ;
					renderText( 10, selfH - 80, QString("Group ID : %1").arg( selected->getGroupID() ), textFont ) ;
					renderText( 10, selfH - 70, QString("Layer : %1").arg( selected->getLayer() ), textFont ) ;
					renderText( 10, selfH - 50, QString("OSC : %1:%2").arg( ((NxTrigger*)selected)->getHost() ).arg( ((NxTrigger*)selected)->getPort() ), textFont );
					renderText( 10, selfH - 30, QString("X: %1").arg( ((NxTrigger*)selected)->getPosition().x), textFont );
					renderText( 10, selfH - 20, QString("Y: %1").arg( ((NxTrigger*)selected)->getPosition().y), textFont );
					renderText( 10, selfH - 10, QString("Z: %1").arg( ((NxTrigger*)selected)->getPosition().z), textFont ); 
				break;
			}	
		}
	}	
	///////////////////////////////////////////////
	// MANUAL RULE INFOS DISPLAY - angle and length
	///////////////////////////////////////////////
	if(lineRuleMode) {
		renderText( mouseXPixPos + 10,  mouseYPixPos , QString("Length : %1 s").arg( lineRuleLength ), textFont ) ; 
		renderText( mouseXPixPos + 10,  mouseYPixPos-15 , QString("Angle : %1").arg( lineRuleAngle ), textFont ) ; 
	}
	///////////////////////////////////////////////
	// TRAJECT EDITION - angle display
	///////////////////////////////////////////////
	if( (trajectEdition && selTrajectPoint != -1) || trajectCreation ) {
		if( trajectCreation )  editedTraject = createdTraject ;
		renderText( mouseXPixPos + 10,  mouseYPixPos , QString("Length : %1 s").arg(  editedTraject->getLength() ), textFont ) ;
		if( editedTraject->getOrientation() == 4) // Free line -> display angle too
			renderText( mouseXPixPos + 10,  mouseYPixPos-15 , QString("Angle : %1").arg( editedTrajectAngle ), textFont ) ; 
	}
	///////////////////////////////////////////////
	// CURVE EDITION - local coordinates
	///////////////////////////////////////////////
	if(curveEdition && curvEdPtChoosed && pointID!=-1 && !curvEdOnlyClic) { ///// Local Coordinates of Edited Curve Point 
		renderText( mouseXPixPos + 10, mouseYPixPos-15 , QString("X = %1").arg( curvEdInfoX ), textFont );
		renderText( mouseXPixPos + 10, mouseYPixPos , QString("Y = %1").arg( curvEdInfoY ), textFont );
	}
}

////////////////////////////////////
// OpenGL initializer
////////////////////////////////////
void NxGLScore::initializeGL()
{
  	qglClearColor( Qt::black );
	glLineWidth( 1.5 );
	glBlendFunc( GL_SRC_ALPHA, GL_ONE );	
  	glEnable( GL_BLEND );
  	setAntialiasing( antialiasing );
}
////////////////////////////////////////////////
// Antialiasing
////////////////////////////////////////////////
void NxGLScore::updateAntialiasing()
{
	makeCurrent() ;
	setAntialiasing( mParent->getAntialiasing() );
}
void NxGLScore::setAntialiasing( bool state )
{
  	if ( state )  {
		glEnable( GL_POLYGON_SMOOTH );
		glEnable( GL_POINT_SMOOTH );
		glEnable( GL_LINE_SMOOTH );
		glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
	}
	else  {
		glDisable( GL_LINE_SMOOTH );
		glDisable( GL_POLYGON_SMOOTH );
		glDisable( GL_POLYGON_SMOOTH );
		glHint(GL_LINE_SMOOTH_HINT, GL_DONT_CARE);
	}
}
//////////////////////////////////////////////////
// OpenGL Modelview setup
//////////////////////////////////////////////////
void NxGLScore::resizeGL( int w, int h )
{
	float hf = (float ) h;
	float wf = (float ) w;
	
	//hf = 3.0 * wf / 4.0 ; 
	//printf( "W %f H %f \n", wf,  hf );
	glViewport( 0, 0, (GLint)wf, (GLint)hf );
  	selfW = (GLint)wf;
	selfH = (GLint)hf;
	//////////////////////////////////// depends on initial window size
	RScaleX = (float)initSelfW/selfW ;
	RScaleY = (float)initSelfH/selfH ;
	frustumView();
}
///////////////////////////////////
// Model view switcher
///////////////////////////////////
void NxGLScore::frustumView()
{
  	glMatrixMode( GL_PROJECTION );
  	glLoadIdentity();
	
  	if ( frustum )
		//////////// 3D MODE - Frustum view (no selection available)
    	glFrustum( -length/10.0, length/10.0, -nHeight/10.0, nHeight/10.0, 5.0, 200.0 );
	else
		//////////// 2D MODE - Orthographic view	
		glOrtho( -length/10.0, length/10.0, -nHeight/10.0, nHeight/10.0, 5.0, 200.0 );	
			
  	glMatrixMode( GL_MODELVIEW );
  	glClear( 1 );
}

///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
//////////////////////// Reset view
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
void NxGLScore::resetView()
{	
	mParent->setWindowTitle( QString("IanniX Viewer - %1 %2").arg( floor(100*scaleX/0.2)).arg('%') );
	curveEdition = false ;
	trajectEdition = false ;
	
  	posX = posY = offX = offY = selX = selY = 0.0;
	transX = transY = 0 ;
  	cursX = cursY = diffX = diffY = decX = decY = 0.0;
  	mouseX = mouseY = 0;
  	IDselected = 0;
  	pointID = -1;
	
	// Grid
	if(autoGridStep) gridStep = length/10.0 ;
	gridDecX = 0 ;
	gridDecY = 0 ;
	// Line Rule Mode
	lineRuleMode = false ;
	lineRuleSetFirstPt = true ;	
	
  	// Curve edition variables
	selected = NULL;
  	currentPoints = 1;
	
  	scaleX = scaleY = scaleZ = 0.17;
  	bX = length / (RScaleX*scaleX*10.0);
  	bY = nHeight / (RScaleY*scaleY*10.0);
  	p.x = p.y = 0.0;
  	clickPos.x = clickPos.y = 0.0;
  	angleX = angleY = 0.0 ;
	
	updateTrigg_LinesBB() ; // useful here ?
}
void NxGLScore::resetZoom( bool withBB )
{
  	posX = posY = offX = offY = selX = selY = 0.0;
	transX = transY = 0 ;
  	cursX = cursY = diffX = diffY = decX = decY = 0.0;
  	mouseX = mouseY = 0;
  	IDselected = 0;
  	pointID = -1;
	
	// Grid
	if(autoGridStep) gridStep = length/10.0 ;
	gridDecX = 0 ;
	gridDecY = 0 ;
	
	scaleX = scaleY = scaleZ = 0.17;
  	bX = length / (RScaleX*scaleX*10.0);
  	bY = nHeight / (RScaleY*scaleY*10.0);
  	p.x = p.y = 0.0;
  	clickPos.x = clickPos.y = 0.0;
	
	if(withBB) updateTrigg_LinesBB() ;
}
void NxGLScore::fitToPageView()
{
	angleX = angleY = 0.0 ; // 3D View reset
	
	/*
	1)) want  bX = length
	donc scaleX =  1/RSCaleX 
	2)) want by = height
	donc scaleX = nHeight/(RScaleY*height  )
	donc setzomm(oldscaleX/saclex)
	scaleX/= factor;
  	scaleY/= factor;
  	scaleZ/= factor;
  	bX = length / (RScaleX*scaleX*10.0);
  	bY = nHeight / (RScaleY*scaleX*10.0);
	*/
	
	resetZoom( false ) ; 
	float oldScaleX = scaleX ;
	
	if(  (float)(length/height) > (float)(selfW/selfH)  )
		setZoom(oldScaleX*RScaleX*5) ;
	else
		setZoom(oldScaleX*RScaleY*height*5/nHeight) ;
	frustumView() ;
	
	setZoom( 1.05 ) ; // a little less to see the borders
	
	updateTrigg_LinesBB() ;
}

void NxGLScore::zoomBy( float inZoom )
{	
	if(!onClick) {
		NxPoint initCenter ; // center of view before zoom
		float transDecX, transDecY ;
		float oldScaleX ;
	
		initCenter.x = (float)transX/(RScaleX*scaleX) ;
		initCenter.y = (float)transY/(RScaleY*scaleY) ;
	
		//printf("avant centre : %f et %f\n",initCenter.x,initCenter.y) ;
	
		oldScaleX = scaleX ;
		resetZoom( false ) ; 
		setZoom(scaleX*inZoom/oldScaleX) ;
		frustumView() ;
		
		transDecX = initCenter.x*(RScaleX*scaleX) ;
		transDecY = initCenter.y*(RScaleY*scaleY) ;
		setTranslation( transDecX, transDecY, true ) ;
	
		//printf("apres : %f et %f\n",transX/(RScaleX*scaleX),transY/(RScaleY*scaleY)) ;
	
		updateTrigg_LinesBB() ;
	}
}

void NxGLScore::zoomRect( NxPoint inP1, NxPoint inP2 )
{
	float rLen = fabs(inP2.x - inP1.x) ;
	float rHei = fabs(inP2.y - inP1.y) ;
	float oldScaleX ;
	float transDecX, transDecY ;
	NxPoint initCenter ; // center of view before zoom
	NxPoint wantedCenter ;
	
	initCenter.x = (float)transX/(RScaleX*scaleX) ;
	initCenter.y = (float)transY/(RScaleY*scaleY) ;
	wantedCenter.x = (inP2.x + inP1.x)/2 - length/2 ;
	wantedCenter.y = (inP2.y + inP1.y)/2 - height/2 ;
	
	//printf("avant centre : %f et %f\n",initCenter.x,initCenter.y) ;
	//printf("wanted : %f et %f\n",wantedCenter.x,wantedCenter.y) ;

	resetZoom( false ) ; 
	oldScaleX = scaleX ;
	
	if(  (float)(rLen/rHei) > (float)(selfW/selfH)  ) {
		setZoom(oldScaleX*RScaleX*5*rLen/length) ;
		//printf("FIT longueur\n") ;
		}
	else {
		setZoom(oldScaleX*RScaleY*rHei*5/nHeight) ;
		//printf("FIT largeur\n") ;
		}
	
	frustumView() ;

	transDecX = - wantedCenter.x*(RScaleX*scaleX) ;
	transDecY = - wantedCenter.y*(RScaleY*scaleY) ;	
	//transDecX = (wantedCenter.x - initCenter.x)*(RScaleX*scaleX) ;
	//transDecY = (wantedCenter.y - initCenter.y)*(RScaleY*scaleY) ;
	setTranslation( transDecX, transDecY, true ) ;
	//printf("aprestrans : %f et %f\n",transX/(RScaleX*scaleX),transY/(RScaleY*scaleY)) ;

	updateTrigg_LinesBB() ;
}

void NxGLScore::hardResetView() // when score's length or height change
{
	scaleX = scaleY = scaleZ = 0.17 ;
	RScaleX = RScaleY = 1 ;
	selfW = initSelfW = frameSize().width() ;
	selfH = initSelfH = frameSize().height() ;
	//initSelfH = selfH ;
	nHeight = (float)length*initSelfH/initSelfW ;
	bX = length / (RScaleX*scaleX*10.0) ;
  	bY = nHeight / (RScaleY*scaleY*10.0) ;
	//nHeight = (float)length*selfH/selfW  ;
	
	frustumView() ;
	
	fitToPageView() ;
}

/////////////////////////////////////////////////////
// Translation on X, Y & Z axis
/////////////////////////////////////////////////////
void NxGLScore::setTranslation( float dX, float dY, bool trans )
{
  	decX += dX;
  	decY += dY;
  	if (trans) {
		transX += dX;
		transY += dY;
	}
  	// Updating Coords
  	timePos.x-= dX;
  	timePos.y-= dY;
}

/////////////
// Zoom
/////////////
void NxGLScore::setZoom( float factor )
{
  	scaleX /= factor;
  	scaleY /= factor;
  	scaleZ /= factor;
	
  	bX = length / (RScaleX*scaleX*10.0);
  	bY = nHeight / (RScaleY*scaleX*10.0);
	
	if(autoGridStep) gridStep *= factor ;
	 	
  	timePos.x*= factor;
  	timePos.y*= factor;
	
  	// Test Slices
  	NbSLICES = (int) ( NbSLICES / factor );
  	if (NbSLICES < 60 ) NbSLICES = 60; // Threshold
  	if (NbSLICES > 100 ) NbSLICES = 100;
	slice = 360.0 / NbSLICES;
	
	mParent->setWindowTitle( QString("IanniX Viewer - %1 %2").arg( getZoomScale() ).arg('%') );
}

/////////////
// BB calcul
/////////////
void NxGLScore::updateTrigg_LinesBB()
{
	NxTrigger *trig ;
	NxTraject *traj ;
	NxPoint pointpp ;
	
	QList<int> ind1 = score->triggersIndexes();
	QList<int> ind2 = score->trajectsIndexes();
		
	BBsize = (float)length/(600*scaleX) ;
	
	//printf("length : %d scale : %f\n", length, scaleX ) ;
	
	for(int im=0 ; im<ind1.count() ; im++) {
		trig = score->getTrigger( ind1.at(im) ) ;
		pointpp = trig->getPosition() ;
		trig->evalBoundingBox( pointpp  , BBsize ) ;
	}
	for(int in=0 ; in<ind2.count() ; in++) {
		traj = score->getTraject( ind2.at(in) ) ;
		if( ((NxTraject*)traj)->getType() == 2 ) { // Lines
			pointpp = traj->getPosition() ;
			traj->evalBoundingBox( pointpp  , BBsize ) ;
		}
	}
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////
// Object Management Section
///////////////////////////////////////////////////////
/////////////////////////
// New Traject
/////////////////////////
void NxGLScore::addNewTraject( NxPoint inPos, int inID, int incID, int inGroupID, int inLay, QString inName, Color inColor, int inChx, int inAxis, float inRadius )
{
	switch (inChx) {
		case 1 : ////// Circle
	  		newC.p = inPos;
	  		newC.radius = inRadius;
	 		score->addTraject( inPos, inID, incID, inGroupID, inLay, inName, inColor, 1 );
	  		score->getTraject( inID )->setCircleDef( newC );
	  		score->getTraject( inID )->evalBoundingBox( inPos , 1.0);
		break;		
		case 2 : ////// Line
	    	trajID = inID;
			score->addTraject( inPos, inID, incID, inGroupID, inLay, inName, inColor, 2 );
			score->getTraject( inID )->setOrientation( inAxis );
	    	score->getTraject( inID )->setLineDef( inPos, inPos ); // ??? - 2nd point later...
			createdTraject = score->getTraject( inID ) ;
			trajectCreation = true;
			trajectEdPt1 = inPos ;
		break;
	}
	updateIndexes();
	controlCenter->updateItems( 1, true, inID );
}
///////////////////////
// New Trigger
///////////////////////
void NxGLScore::addNewTrigger( NxPoint inPos, int inID, int incID, int inGroupID, int inLay, QString inName, Color inColor, QString inHost, int inPort, int inIntValue )
{
	//pause() ;
	
  	score->addTrigger( inPos, inID, incID, inGroupID, inLay, inName, inColor, inHost, inPort, inIntValue );
  	updateIndexes();
  	controlCenter->updateItems( 3, true, inID );
	score->getTrigger( inID )->evalBoundingBox(inPos, BBsize ) ;
	timeScheduler->addNewTrigger() ;
}
//////////////////////
// New Cursor
//////////////////////
void NxGLScore::addNewCursor( NxPoint inPos, int inID, int incID, int inGroupID, int inLay, QString inName, Color inColor, bool inForward, int inMode, float inFactor, float inWidth, float inInitOffset, float inFinalOffset )
{
  	if (cursorMode) {
		pause() ;
		score->addCursor( inPos, inID, incID, (NxTraject*)edited, inGroupID, inLay, inName, inColor, inForward, inMode, inWidth, inInitOffset, inFinalOffset );
		score->getCursor(inID)->reEvalOffsets() ;
		score->getCursor(inID)->setSpeed( inFactor );
		score->getCursor(inID)->setWidth( inWidth );
		score->getCursor(inID)->evalBoundingBox(inPos, BBsize ) ;
		updateIndexes();
		assignCursors();
		controlCenter->updateItems( 2, true, inID);	
		cursorMode = false;
		timeScheduler->addNewCursor() ;
	}
}
//////////////////////////////////////////////////
// ADD CURVES
//////////////////////////////////////////////////
void NxGLScore::setDrawMode( int inMode )
{
	drawMode = inMode;
}
void NxGLScore::addRandomCurve( NxPoint inPos, int inID, int incID, int inGroupID, int inLay, QString inName, Color inColor1, Color inColor2, int nbCtrlPoints, float rangeMin, float rangeMax, int newDuration, float rMin, float rMax, float inScale, int inSpatialPeriod, QString inHost, int inPort )
{	
	//pause() ;
	
	QVector<NxPoint> newDef;
	// Creating random curve		
	newDef = randomCurve( nbCtrlPoints, rangeMin, rangeMax, newDuration ) ;
	float inAngle = 0.0 ;
	score->addCurve( inPos, inID, incID, inGroupID, inLay, inName, inColor1, inColor2, newDef, nbCtrlPoints, rMin, rMax, inScale, inAngle, inSpatialPeriod, inHost, inPort );
	score->getCurve(inID)->setDrawType(drawMode);
	  
	// Updating Indexes
	updateIndexes();
	controlCenter->updateItems( 0, true, inID );
	timeScheduler->addNewCurve() ;
}
void NxGLScore::addEditCurve( NxPoint inPos, int inID, int incID, int inGroupID, int inLay, QString inName, Color inColor1, Color inColor2, float rMin, float rMax, float inScale, int inSpatialPeriod, QString inHost, int inPort )
{
	//pause() ;
	
	// Manual Edition -> saving 1st point & wait extra points with mouse clicks....
	curveCreation = true ;
	savID = inID;
	firstCurvPos = inPos ;
	savGroupID = inGroupID;
	savName = inName;
	realMin = rMin;
	realMax = rMax;

	// Allocating first point
	NxPoint fPt ;
	inPos.ID = 0;
	fPt.ID	 = 0 ;
	fPt.x	 = 0.0 ;
	fPt.y	 = fabs(rMax-rMin)/2 ;
	fPt.z	 = 0.0 ;
	
	editedCurve.append( fPt );
	firstPoint = true;
	
	float inAngle = 0.0 ;
	score->addCurve( inPos, savID, incID, savGroupID, inLay, savName, inColor1, inColor2, editedCurve, editedCurve.count(), realMin, realMax, inScale, inAngle, inSpatialPeriod, inHost, inPort ) ;
	score->getCurve(inID)->setDrawType(drawMode) ;
	timeScheduler->addNewCurve() ;
}

//////////////////////////////////////////////
// RENDERING SECTION
//////////////////////////////////////////////
//////////////
// LINES
//////////////
void NxGLScore::drawLine( NxPoint inP1, NxPoint inP2, Color inColor )
{
	glColor4fv( colorConv(inColor) );
  	glBegin(GL_LINES);
  		glVertex3f( inP1.x, inP1.y, inP1.z );
  		glVertex3f( inP2.x, inP2.y, inP2.z );
  	glEnd();
}
////////// Render cursor impact zone
void NxGLScore::drawCursorArea( NxPoint inP1, NxPoint inP2, float inWid )
{
	float lineAng = 180.0*atan( (inP1.y - inP2.y) / (inP1.x - inP2.x) )/PI ;
	float lineLen = sqrt( pow(inP1.y - inP2.y, 2) + pow(inP1.x - inP2.x, 2) ) ;

	glColor4f( 1.0, 1.0, 1.0, cursZonesOpacity ) ; /// Cursor Area Color
	glPushMatrix() ;
	glTranslatef( (inP1.x+inP2.x)/2, (inP1.y+inP2.y)/2, inP1.z ) ;
	glRotatef( lineAng, 0, 0, 1 ) ;
  	glBegin(GL_QUADS);
	glVertex3f( -lineLen/2, inWid/2, 0.0 ) ;
	glVertex3f( -lineLen/2, -inWid/2, 0.0 ) ;
	glVertex3f( lineLen/2, -inWid/2, 0.0 ) ;
	glVertex3f( lineLen/2, inWid/2, 0.0 ) ;
	glEnd();
	glPopMatrix() ;
}
/////////////////////////////////////
// CIRCLES & DISKS
/////////////////////////////////////
// Mode values		0 -> empty	|	1 -> fill 
void NxGLScore::drawCircle( GLfloat R, GLfloat cX, GLfloat cY, GLfloat cZ, int mode, Color inColor )
{
	glColor4fv( colorConv(inColor) );
  	glPushMatrix();
  		glTranslatef( cX, cY, cZ );
		if ( mode == 1 ) {
			glBegin( GL_LINE_LOOP );
				for (GLfloat i=0.0; i<NbSLICES; i++) {
	  				glVertex3f( R * cos( i * slice * PI / 180.0 ), R * sin( i * slice * PI / 180.0 ), 0.0 );
	  				glVertex3f( R * cos( (i + 1) * slice * PI / 180.0 ), R * sin( (i + 1) * slice * PI / 180.0 ), 0.0 );
				}
			glEnd();
		}
  		else {
			glBegin( GL_TRIANGLES );
				for (GLfloat i=0.0; i<NbSLICES; i++) {
	 				glVertex3f( 0.0, 0.0, 0.0 );
	  				glVertex3f( R * cos( i * slice * PI / 180.0 ), R * sin( i * slice * PI / 180.0 ), 0.0 );
	  				glVertex3f( R * cos( (i + 1) * slice * PI / 180 ), R * sin( (i + 1) * slice * PI / 180.0 ), 0.0 );
				}
			glEnd(); 
		}
	glPopMatrix();
}
////////////////////////
// CURSORS
////////////////////////
void NxGLScore::drawCursor( float inX, float inY, float inZ, float inWidth, Color inColor, bool flip )
{
	float Csize = (float)length/(300*scaleX) ;
	
	glColor4fv( colorConv(inColor) );
  	glPushMatrix();
  		glTranslatef( inX, inY, inZ );
  		if (flip) 
			glRotatef(180, 0, 1, 0); 
		////// Triangle
  		glBegin(GL_TRIANGLES);
  			glVertex3f(0, -0.4*Csize, 0.0);
  			glVertex3f(0,  0.4*Csize, 0.0);
  			glVertex3f((float)Csize, 0.0, 0.0);		
  		glEnd();
  		////// Cursor's width
  		glBegin(GL_LINES);
  			glVertex3f( 0.0, -inWidth/2, 0.0);
  			glVertex3f( 0.0, inWidth/2, 0.0); 
  		glEnd();
  	glPopMatrix();	
}
////////////////////////
// TRIGGERS
////////////////////////
void NxGLScore::drawTrigger( float inX, float inY, float inZ, Color inColor )
{
	float Tsize = BBsize ;
	
	glColor4fv( colorConv(inColor) );
  	glPushMatrix();
  		glTranslatef( inX, inY, inZ );
  		glBegin(GL_QUADS);
  			glVertex3f( 0.0, (float)Tsize, 0.0 );
  			glVertex3f( -0.5*Tsize, 0.0, 0.0 );
  			glVertex3f( 0.0, -(float)Tsize, 0.0 );
  			glVertex3f( 0.5*Tsize, 0.0, 0.0 ); 		
  		glEnd();
  	glPopMatrix();
}

/////////////////////////////
// Bounding Box
////////////////////////////
void NxGLScore::drawBoundingBox( BBox inBox )
{
  	glColor4fv( colorConv( BBColor )); // BoundingBox Color
  	glBegin( GL_LINE_LOOP );
  		glVertex3f( inBox.p1.x, inBox.p1.y, inBox.p1.z );
  		glVertex3f( inBox.p1.x, inBox.p2.y, inBox.p1.z );
  		glVertex3f( inBox.p2.x, inBox.p2.y, inBox.p1.z );
  		glVertex3f( inBox.p2.x, inBox.p1.y, inBox.p1.z );
  	glEnd();
}
void NxGLScore::drawRect( NxPoint p1, NxPoint p2 )
{
  	glBegin( GL_LINE_LOOP );
  		glVertex3f( p1.x, p1.y, p1.z );
  		glVertex3f( p1.x, p2.y, p1.z );
  		glVertex3f( p2.x, p2.y, p1.z );
  		glVertex3f( p2.x, p1.y, p1.z );
  	glEnd();
}
void NxGLScore::drawCross( NxPoint p1, float inSize ) ///// circle center crosse
{	
	glColor4f( 1.0, 1.0, 1.0, 0.15 ); //// crosses color
  	glBegin( GL_LINES );
  		glVertex3f( p1.x - inSize, p1.y, p1.z );
  		glVertex3f( p1.x + inSize, p1.y, p1.z );
  		glVertex3f( p1.x, p1.y - inSize, p1.z );
  		glVertex3f( p1.x, p1.y + inSize, p1.z );
  	glEnd();
}
//////////////////////////////////////
// Triggers Rendering
//////////////////////////////////////
void NxGLScore::renderTriggers() 
{
	Color triggColor ;
	NxPoint pt ;
  	QList<int> trigindexes 	= score->triggersIndexes();
  	
	for (int i=0; i<trigindexes.count(); i++) {
		renderedTrig	= score->getTrigger( trigindexes.at(i) );
		pt				= renderedTrig->getPosition();
		
		///////////// Normal / Inactive / Highlighted Color /////////////
		triggColor		= renderedTrig->getColor() ;
		if(!renderedTrig->isActive())		triggColor = inactiveColor ;
		if (renderedTrig->isHighlighted())	triggColor = selectedColor ;
		
		///////////// Drawing Bounding Box /////////////
		if ( renderedTrig->isSelected() ) {
	 		inBox = renderedTrig->getBoundingBox() ;
	  		drawBoundingBox( inBox ) ; 
		}
			
		drawTrigger( pt.x, pt.y, pt.z , triggColor );
	} 
}

/////////////////////////////////////////////
// Trajectories Rendering
/////////////////////////////////////////////
void NxGLScore::renderTrajects()
{
  	QList<int> indexes 	= score->trajectsIndexes();

  	///////////// Trajects rendering
  	for (int i=0; i<indexes.count(); i++) {	
		int idx = indexes.at(i);
		renderedTraj = score->getTraject(idx) ;
		
		/////////////// INACTIVE / HIGHLIGHTED / NORMAL COLORS
		Color coull = score->getTraject(idx)->getColor() ;
		if( !renderedTraj->isActive() )		coull = inactiveColor ;
		if( renderedTraj->isHighlighted() ) coull = selectedColor ;
		
		trajShape = renderedTraj->getType();
			
		switch ( trajShape ) {
			case 1:  ////////// Circle //////////
				if ( renderedTraj->isSelected() ) { // BBox
					bb = renderedTraj->getBoundingBox();
					drawBoundingBox( bb );
				}
  				if (showTrajects) {
	  				circ = renderedTraj->getCircle();
	  				drawCircle( circ.radius, circ.p.x, circ.p.y, circ.p.z, 1, coull);	
					if (crossesDraw) drawCross( circ.p, BBsize ) ;
				}
	  		break;
			case 2:  ////////// Line //////////
				////////////////// Draw BBox if selected ///////////////
				if ( renderedTraj->isSelected() ) {
					if( renderedTraj->getOrientation() != Both ) { // Horizontal or Vertical
						bb = renderedTraj->getBoundingBox();
						drawBoundingBox( bb );
					}
					else { // Free Line Case
						float trajAng = renderedTraj->getAngle() ;
						float trajLen = renderedTraj->getLength() ;
						if(trajLen<BBsize) trajLen = BBsize ;
						NxPoint initPt = renderedTraj->getInitialPoint() ;
						NxPoint finalPt = renderedTraj->getFinalPoint() ;
						if(finalPt.y<initPt.y) {
							trajAng = 180 - trajAng ;
							trajLen = - trajLen ;
						}
						glColor4fv( colorConv( BBColor )); // BoundingBox Color
						glPushMatrix() ;
						glTranslatef( initPt.x, initPt.y, initPt.z ) ;
						glRotatef( trajAng, 0, 0, 1 ) ;
						glBegin( GL_LINE_LOOP );
						glVertex3f( 0, BBsize, 0 );
						glVertex3f( 0, -BBsize, 0 );
						glVertex3f( trajLen, -BBsize, 0 );
						glVertex3f( trajLen, BBsize, 0 );
						glEnd();
						glPopMatrix();
					}
				}
				////////// LINE rendering
	  			if (showTrajects) drawLine( renderedTraj->getInitialPoint(), renderedTraj->getFinalPoint(), coull );
				if( trajectEdition ) { // traject control points
					if( renderedTraj == (NxTraject*)selected ) {
						NxPoint pp1 = renderedTraj->getInitialPoint() ;
						NxPoint pp2 = renderedTraj->getFinalPoint() ;
						float dimPt = (float)length/(1500*scaleX) ;
						glColor4f( 1.0, 1.0, 1.0, 1.0 ); //// First Point Color
						glBegin( GL_QUADS );
						glVertex3f( pp1.x-dimPt, pp1.y-dimPt, 0.0 );
						glVertex3f( pp1.x+dimPt, pp1.y-dimPt, 0.0 );
						glVertex3f( pp1.x+dimPt, pp1.y+dimPt, 0.0 );
						glVertex3f( pp1.x-dimPt, pp1.y+dimPt, 0.0 );
						glEnd();
						glColor4f( 1.0, 1.0, 1.0, 1.0 ); //// Last Point Color
						glBegin( GL_QUADS );
						glVertex3f( pp2.x-dimPt, pp2.y-dimPt, 0.0 );
						glVertex3f( pp2.x+dimPt, pp2.y-dimPt, 0.0 );
						glVertex3f( pp2.x+dimPt, pp2.y+dimPt, 0.0 );
						glVertex3f( pp2.x-dimPt, pp2.y+dimPt, 0.0 );
						glEnd();
					}
				}
	  		break;
		} // end switch	
		if(showCursors) renderCursors( idx, trajShape );
	}
}

/////////////////////////////////////////////////////////////////////////////
// Curves Rendering with custom B-splines
/////////////////////////////////////////////////////////////////////////////
void NxGLScore::renderCurves()
{
  	GLfloat controlPtsColor[] = {0.8, 0.8, 0.8, 1.0} ;
	GLfloat curvesRectColor[] = {1.0, 1.0, 1.0, 0.4} ;
	GLfloat curvesZeroColor[] = {1.0, 1.0, 1.0, 0.2} ;
	GLfloat curvesLineColor[] = {0.8, 0.7, 0.2, 0.4} ;
	
  	//////////////////// Rendering Curves ////////////////////
  	QList<int> indexes = score->curvesIndexes();
	
	for (int i=0; i<indexes.count(); i++) {
		NxPoint curvPos ;
		NxPoint mMpt ; // for the min/max rectangle
		int idx = indexes.at(i);
		renderedCurv = score->getCurve(idx) ;
		
		tmpCurves 	= renderedCurv->getCurve();
		lengthC 	= renderedCurv->size();	
		radius 		= renderedCurv->getRadius();
		tmpPos 		= renderedCurv->getCenter();
		curvPos		= renderedCurv->getPosition() ;
		float cvSc	= renderedCurv->getScale() ;
		float cvMin = renderedCurv->getRealMin() ;
		float cvMax = renderedCurv->getRealMax() ;
		float ang	= renderedCurv->getAngle() ;
		float xRange = renderedCurv->getXRange() ;
		float yRange = renderedCurv->getYRange() ;
		Color coulCir = renderedCurv->getCircleColor() ;
		Color coulGraph = renderedCurv->getColor() ;
		bool beingEdited = (curveEdition && (renderedCurv == selected) ) ;
		
		/////////////////////////// INACTIVITY ///////////////////////////
		if( !renderedCurv->isActive() ) {
			coulCir = inactiveColor ;	
			coulGraph = inactiveColor ;	
		}	
		
		/////////////////////////// BOUNDING BOX ///////////////////////////
		if ( renderedCurv->isSelected() ) {
			if( ang==0 ) {
				bb = renderedCurv->getBoundingBox();
				drawBoundingBox( bb ) ;
			}
			else {
				float ll = xRange ;
				float hh = yRange*cvSc ;
				if(ll<BBsize) ll = BBsize ;
				if(hh<BBsize) hh = BBsize ;
				glColor4fv( colorConv( BBColor )); ///// BoundingBox Color
				glPushMatrix() ;
				glTranslatef( tmpPos.x, tmpPos.y, tmpPos.z ) ;
				glRotatef( ang, 0, 0, 1 ) ;
				glBegin( GL_LINE_LOOP );
				glVertex3f( -ll/2.0,	hh/2.0,		0 ) ;
				glVertex3f( -ll/2.0,	-hh/2.0,	0 ) ;
				glVertex3f( ll/2.0,		-hh/2.0,	0 ) ;
				glVertex3f( ll/2.0,		hh/2.0,		0 ) ;
				glEnd();
				glPopMatrix();
			}
		}
		
		if (circlesDraw) ////////////////////// Circle (circle algorithm)
			drawCircle( radius, tmpPos.x, tmpPos.y, tmpPos.z, 1, coulCir );
			
		if (crossesDraw) ////////////////////// Center crosses
			drawCross( renderedCurv->getCenter(), BBsize ) ;
			
		if (!beingEdited) { ////////////////////// If edited
			glTranslatef( tmpPos.x, tmpPos.y, tmpPos.z) ;	
			glRotatef( ang, 0, 0, 1 ) ;
			glTranslatef( -tmpPos.x, -tmpPos.y, -tmpPos.z) ;
		}
		
		if (rectDraw) {
			if(lengthC>1) { //// Normal CASE
				//////////////////// Draws the min & max (rect)
				if ( !renderedCurv->isSelected() ) {
					glColor4fv( curvesRectColor ) ;
					if ( renderedCurv->isHighlighted() ) 
						glColor4fv( colorConv( selectedColor ) ) ;
					if( !renderedCurv->isActive() ) glColor4fv( colorConv( inactiveColor ) ) ;
					BBox box = renderedCurv->getBoundingBox() ;
					mMpt = box.p1 ;
					mMpt.z = tmpPos.z ;
					drawRect( mMpt, box.p2 ) ;
				}
				//////////////////// Draws the 0 line horizontally (same color) (only if the 0 is in the rect...)
				glColor4fv( curvesZeroColor );
				if( !renderedCurv->isActive() ) glColor4fv( colorConv( inactiveColor ) ) ;
				if( cvMax>=0.0 && cvMin<=0.0 ) {
					glPushMatrix();
					glTranslatef( curvPos.x, curvPos.y + (cvMin + yRange/2 )*cvSc, curvPos.z ) ;
					glBegin( GL_LINE_STRIP );	
					glVertex3f( 0.0, 0.0, 0.0 );
					glVertex3f( 2*radius, 0.0, 0.0 );	
					glEnd();
					glPopMatrix();	
				}	
				//////////////////// Draws a simple line to see the segment (collisions)
				//if(!beingEdited) {
					glColor4fv( curvesLineColor );
					if( !renderedCurv->isActive() ) glColor4fv( colorConv( inactiveColor ) ) ;
					glPushMatrix();
					glTranslatef( tmpPos.x, tmpPos.y, tmpPos.z ); // tmpPos is the curve Center
					//glRotatef( ang, 0, 0, 1 );
					glBegin( GL_LINE_STRIP );		
					//glLineStipple(6, 0xAAAA);  			
					glVertex3f( -radius, 0.0, 0.0 );
					glVertex3f( radius, 0.0, 0.0 );	
					glEnd();
					glPopMatrix();
				//}
				////////////////////////////////////////////////////////////
			}
			else { //// Curve Creation CASE
				NxPoint opp1, opp2 ;
				glColor4fv( curvesRectColor );
				opp1.ID = opp2.ID = 0 ;	// initialisation
				opp1.z = opp2.z = 0 ;	// initialisation
				opp1.x = curvPos.x ; 
				opp1.y = curvPos.y + yRange*cvSc/2 ;
				opp2.x = curvPos.x + yRange*cvSc/2 ; 
				opp2.y = curvPos.y - yRange*cvSc/2 ;
				drawRect( opp1, opp2 ) ;
			}
		}
			
		/////////////////////// RENDERING CURVE POINTS ////////////////////////////////
		glColor4fv( colorConv(coulGraph) );		
		////////////// Drawing Curve or Lines
		glBegin( GL_LINE_STRIP );
			switch ( renderedCurv->getDrawType() ) {
				case LineStrip :
					for ( int j=0; j<lengthC;j++ )
						glVertex3f( curvPos.x + tmpCurves[j].x, curvPos.y + cvSc*((tmpCurves[j].y - cvMin) - yRange/2 ), curvPos.z );
				break;
				case SplineStrip :
					glTranslatef( 0, 0, curvPos.z) ;
					renderedCurv->getSpline()->draw() ;
					glTranslatef( 0, 0, -curvPos.z) ;
				break;
			}
		glEnd();	
		
		///////////// Re-rotate if not being edited
		if (!beingEdited) {
			glTranslatef( tmpPos.x, tmpPos.y, tmpPos.z) ;
			glRotatef( -ang, 0, 0, 1 ) ;
			glTranslatef( -tmpPos.x, -tmpPos.y, -tmpPos.z) ;
		}
		
		///////////// Drawing controlPoints
		if ( renderedCurv->displayPoints() ) {
	  		glColor4fv( controlPtsColor );
	  		for( int j=0; j<lengthC; j++) {
				float dimPt = (float)length/(1500*scaleX) ;
				float dimPt2 = 2*(float)length/(1500*scaleX) ;
				glBegin( GL_QUADS );
	      			glVertex3f( curvPos.x + tmpCurves[j].x-dimPt, curvPos.y + cvSc*((tmpCurves[j].y-cvMin) - yRange/2 ) - dimPt, 0.0 );
	      			glVertex3f( curvPos.x + tmpCurves[j].x+dimPt, curvPos.y + cvSc*((tmpCurves[j].y-cvMin) - yRange/2 ) - dimPt, 0.0 );
	      			glVertex3f( curvPos.x + tmpCurves[j].x+dimPt, curvPos.y + cvSc*((tmpCurves[j].y-cvMin) - yRange/2 ) + dimPt, 0.0 );
	      			glVertex3f( curvPos.x + tmpCurves[j].x-dimPt, curvPos.y + cvSc*((tmpCurves[j].y-cvMin) - yRange/2 ) + dimPt, 0.0 );
								
	      			////////////// Display selected one (if any)
	      			if (tmpCurves[j].ID == pointID) {	
						GLfloat ccc[] 	= {1.0, 0, 0, 1.0};	
						glColor4fv( ccc );
		  				glVertex3f( curvPos.x + tmpCurves[j].x-dimPt2, curvPos.y + cvSc*((tmpCurves[j].y-cvMin) - yRange/2 ) - dimPt2, 0.0 );
		  				glVertex3f( curvPos.x + tmpCurves[j].x+dimPt2, curvPos.y + cvSc*((tmpCurves[j].y-cvMin) - yRange/2 ) - dimPt2, 0.0 );
		  				glVertex3f( curvPos.x + tmpCurves[j].x+dimPt2, curvPos.y + cvSc*((tmpCurves[j].y-cvMin) - yRange/2 ) + dimPt2, 0.0 );
		  				glVertex3f( curvPos.x + tmpCurves[j].x-dimPt2, curvPos.y + cvSc*((tmpCurves[j].y-cvMin) - yRange/2 ) + dimPt2, 0.0 );
						glColor4fv( controlPtsColor );
					}
				glEnd();
	    	}
		} // end if (display points )
	} // end for (curves )
}
//////////////////////////////////////
// Cursors Rendering
//////////////////////////////////////
void NxGLScore::renderCursors( int ind, int inShape )
{
  	QList<int> IDs 		= score->getTraject(ind)->getCursorIDs();
  	float rad 			= score->getTraject(ind)->getCircle().radius ;
  	NxPoint cnt 		= score->getTraject(ind)->getCircle().p;
  	float len, angleC, nX, nY;
	NxPoint l1 = score->getTraject(ind)->getInitialPoint();
  	NxPoint l2 = score->getTraject(ind)->getFinalPoint();
  	NxPoint orig;
	Color coul ;
	orig.ID = 0 ;
  	orig.x = orig.y = orig.z = 0.0 ; 
	
  	for ( int i=0; i<IDs.count(); i++ ) {
		renderedCurs = score->getCursor( IDs.at(i) );

		NxPoint cBox ;
		BBox newBox ;
		cBox.ID = 0 ;
		cBox.x = cBox.y = cBox.z = 0 ; // initialisation
		newBox.p1 = newBox.p2 = cBox ; // initialisation
		float localCursX ;

		if ( renderedCurs != NULL ) {	
			///////////////////////////////////// COLORS
			coul = renderedCurs->getColor() ;
			if( !renderedCurs->isActive() )		coul = inactiveColor ;
			if( renderedCurs->isHighlighted() )	coul = selectedColor ;
			
	  		switch (inShape) {	
				case 1 : //////////// CIRCLE ////////////
					glPushMatrix();
	      			len = 2.0 * rad * PI;
	      		
					// Interpolation between mouse movement & circle perimeter
	      			// Calculating angle
	      			angleC = ( 360.0*renderedCurs->getOffset() )/ len;
	      			glTranslatef( cnt.x, cnt.y, cnt.z );
	      			glRotatef( angleC, 0, 0, 1 );
	      			glTranslatef( rad, 0.0, 0.0 );
	      			if ( renderedCurs->isSelected()  && (selected != NULL) && ( renderedCurs->getID() == selected->getID() ) ) {
		  				renderedCurs->evalBoundingBox( orig, BBsize );
		  				drawBoundingBox( renderedCurs->getBoundingBox() );
						if (onClick) /////// Cursor dragged
		    				if ( cursX+savOffset > len ) renderedCurs->setOffset( len );
		    				else
		      					if (  cursX + savOffset < 0 ) renderedCurs->setOffset( 0 );
								else renderedCurs->setOffset( savOffset+cursX );	
							
		  				// Recalculate box
		  				nX = rad * cos( angleC * PI / 180.0 );
		  				nY = rad * sin ( angleC * PI / 180.0 );
						
		  				cBox.x = cnt.x + nX;
		  				cBox.y = cnt.y + nY;
		  				renderedCurs->evalBoundingBox( cBox , BBsize );
					}
	      			glRotatef( -90, 0, 0, 1 ) ; 
					
	      			drawCursor( 0.0, 0.0, 0.0, renderedCurs->getWidth(), coul , renderedCurs->getForward() );
					glPopMatrix();
			break;
	    	case 2 : //////////// LINE ////////////
	      		len = sqrt( pow( l2.x - l1.x, 2.0 ) + pow( l2.y - l1.y, 2.0 ) );
				
				////// To avoid Windows Bug
				if( len==0 || fabs((l2.x-l1.x)-len)<0.0001) angleC = 0.0 ;
	      		else angleC = acos( (l2.x-l1.x)/len )*180/PI ;  
				
	      		if (len<0) len = -len ; ////// ??
					
				////////// Impact area
				if(showCursorsArea) drawCursorArea( l1, l2, renderedCurs->getWidth() ) ;
										
	      		////////// Adjusting angle value with line orientation
	      		if ( ( l1.x > l2.x && !( l1.y < l2.y ) ) || l1.y > l2.y )
					angleC	= - angleC;

	      		////////// Cursors need specific bounding boxes, as they can be rotated!
	      		if ( renderedCurs->isSelected() && (selected != NULL) && ( renderedCurs->getID() == selected->getID() ) ) {
		  			// CursX must be positive !!
		  			if (onClick) { /////// Cursor dragged
						localCursX = cursX ;
		      			if (l1.x > l2.x) localCursX = - cursX ;
		      	
						if (cursX+savOffset < 0 ) renderedCurs->setOffset( 0 );
						else
							if (cursX+savOffset > len) renderedCurs->setOffset( len );
							else renderedCurs->setOffset( savOffset+cursX );
					}
					
					float taillCur = BBsize ;
		  			newBox.p1.x = renderedCurs->getOffset() - 2.5 * taillCur;
		  			newBox.p1.y = 1.7 * taillCur;
		  			newBox.p2.x = renderedCurs->getOffset() + 2.5 * taillCur;
		  			newBox.p2.y = - 1.7 * taillCur;
		  			newBox.p1.z = newBox.p2.z = 0.0;
					
		  			///////////// Update bounding box coords
		  			dec.x = l1.x + ( renderedCurs->getOffset() ) * cos( angleC * PI / 180.0 );
		  			dec.y = l1.y + ( renderedCurs->getOffset() ) * sin( angleC * PI / 180.0 );
					dec.z = 0.0 ;
					
		  			///////////// Recalculate BBox
		  			renderedCurs->evalBoundingBox( dec , BBsize ) ;
					glPushMatrix();
		  				glTranslatef( l1.x, l1.y, 0.0 ); ///// l1.z -> 0.0
		  				glRotatef( angleC, 0, 0, 1 );
		  				drawBoundingBox( newBox );
		  			glPopMatrix();
				}
				glPushMatrix();
	      			glTranslatef( l1.x, l1.y, l1.z );
					glRotatef( angleC, 0, 0, 1 );
	      			drawCursor( renderedCurs->getOffset(), 0.0, 0.0, renderedCurs->getWidth(), coul , !renderedCurs->getForward() );
 				glPopMatrix();
				break;
			}
		}
	}
}

/////////////////////////////////////////////
// Rotation around Y-axis (3D View)
/////////////////////////////////////////////
void NxGLScore::RotationX( float offset )
{
  	angleX += offset;
  	if (angleX == 360 || angleX == -360 ) angleX = 0 ;
	oldAngleX = angleX ;
}
void NxGLScore::RotationY( float offset )
{
  	angleY += offset;
  	if (angleY == 360 || angleY == -360 ) angleY = 0 ;
	oldAngleY = angleY ;
}
///////////////////////////////////////////////////
// Random curves generator
///////////////////////////////////////////////////
QVector<NxPoint> NxGLScore::randomCurve( int nbpoints, float inRangeMin, float inRangeMax, int duration )
{
	// Initializing randomizer
  	srand( randomizer );
  	QVector<NxPoint> newCurve;
  	NxPoint newPt ;
  	duration /= 1000 ;
  	// Debug prints
  	printf( "Generating random curve : RMIN = %f, RMAX = %f, Controls = %i, Duration = %i\n", inRangeMin, inRangeMax, nbpoints, duration );
  	for (int i=0;i<nbpoints;i++) {	
		newPt.ID = i;
		newPt.x = ( (float)( i*duration ) / (float) nbpoints);
		newPt.y = rand()%( (int)(inRangeMax-inRangeMin)*100 );
		newPt.y = newPt.y / 100 + inRangeMin ;
		newPt.z = 0.0 ;
		newCurve.append(newPt);
	}
  	// Incrementing seed (TO UPDATE : check size)
  	randomizer++;
	
  	return newCurve;
}

//////////////////////////////////////////
// EVENTS HANDLERS
//////////////////////////////////////////
///////////////////////////////  
// Keyboard Events
//////////////////////////////
void NxGLScore::keyPressEvent( QKeyEvent* e )
{
  	switch ( e->key() ) {		
		case Qt::Key_A :
			if(!frustum) addCurvePoint() ;
		break;		
		case Qt::Key_C :
			showHideCurvesCircles() ;
		break;			
    	case Qt::Key_E : // Curve / Traject Edition Mode On
      		if (selected!= NULL && !frustum) {
				if( selected->getObjectType() == 0 ) { // Curve
					trajectEdition = false ;
					curveEdition = !curveEdition;
					( (NxCurve*) selected )->setPoints( curveEdition ) ;
					( (NxCurve*) selected )->setSelected( false ) ;
				}
				if( selected->getObjectType() == 1 ) { // Traject
					if( ((NxTraject*)selected)->getType() == 2 ) { // LINE
						curveEdition = false ;
						trajectEdition = !trajectEdition ;
						editedTraject = (NxTraject*) selected ;
					}
				}
			}
		break;
		case Qt::Key_F :
			showHideCursorsArea() ;
      	break;
		case Qt::Key_G :
			showHideGrid() ;
		break;	
		case Qt::Key_I :
			snapLockX = !snapLockX ;
      	break;
		case Qt::Key_J :
			snapLockY = !snapLockY ;
      	break;	
		case Qt::Key_K :
			if(!frustum) removeCurvePoint() ;
		break ;		
		case Qt::Key_M :
			showHideScreenInfos() ;
		break;		
		case Qt::Key_P :
			showHideScore() ;
      	break;					
		case Qt::Key_S :
      		switchSnapToGrid() ;
      	break;		
    	case Qt::Key_V :	// Switch between Ortho / Frustum Projection view	
			change2D3DView() ;
      	break;			
	 	case Qt::Key_W :	// Stop Playback and Reset Cursors	
			stopAndReset() ;
		break;		
		case Qt::Key_X :
			if(!frustum) {
				lineRuleMode = true;
				if(lineRuleSetFirstPt) {
					lineRulePt1.x = p.x ;
					lineRulePt1.y = p.y ;
					lineRuleSetFirstPt = false ;
				}
			}
		break;		    	
		case Qt::Key_Z :
      		if(!frustum) zoomMode = true;
      	break;		
    	case Qt::Key_Space :
			playPause() ;
   		break;
    	case Qt::Key_Plus :	// Zoom In / Out
      		zoomBy( 0.95 ) ;
      	break;		
		case Qt::Key_Minus :
      		zoomBy( 1.05 ) ;
      	break;	
		case Qt::Key_Shift :
      		if (!curveEdition && !trajectEdition && !frustum) {
	  			selectionMode = true;
	  			if(!lockedSelection) checkSelected( mouseX, mouseY );
	  			//printf( "Selection Mode : On\n" );
			}
		break;				
    	case Qt::Key_Left : // Moving with keypad arrows
      		setTranslation( 0.05, 0.0, true );
      	break;		
    	case Qt::Key_Right :
      		setTranslation( -0.05, 0.0, true );
      	break;		
    	case Qt::Key_Up :
      		setTranslation( 0.0, -0.05, true );
      	break;
    	case Qt::Key_Down :
      		setTranslation( 0.0, 0.05, true );
      	break;
		case Qt::Key_Less : // Mouse rotation for 3D view
      		if (frustum) mouseRotation = true ;
      	break;
		case Qt::Key_1: //	Y rotation for 3D view
      		if (frustum) RotationY(2.0);
      	break;		
    	case Qt::Key_2 :
      		if (frustum) RotationY(-2.0);
      	break;
		case Qt::Key_3 : //	X rotation for 3D view
      		if (frustum) RotationX(2.0);
      	break;		
    	case Qt::Key_4 :
      		if (frustum) RotationX(-2.0);
      	break;

	}
}
void NxGLScore::keyReleaseEvent( QKeyEvent* e )
{
	if(!e->isAutoRepeat()) { // to avoid automatic release on some systems
		switch ( e->key() ) {	
			case Qt::Key_X :
				lineRuleSetFirstPt = true ;
				lineRuleMode = false;
			break;		
			case Qt::Key_Z :	
				zoomMode = false;
			break;
			case Qt::Key_Shift :
				if (!curveEdition && !trajectEdition && !frustum) {
					selectionMode = false;
					if (selected != NULL)
						checkSelected( mouseX, mouseY );
					lockedSelection = false ;
				}
			break;		
			case Qt::Key_Backspace :
				if (selected != NULL && !frustum)
					removeSelected() ;
			break;
			case Qt::Key_Less : //	Mouse rotation for 3D view
				mouseRotation = false ;
			break;	
		}
	}
}
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
void NxGLScore::updateIndexes() 
{
  	nbCurves 	= score->getEnvironment().nbCurves;
  	nbTrajects 	= score->getEnvironment().nbTrajects;
  	nbTriggers 	= score->getEnvironment().nbTriggers;
  	nbCursors 	= score->getEnvironment().nbCursors;
  	length 		= score->getEnvironment().length;
  	height 		= score->getEnvironment().height;
	////////////////////////////////////////////
	bX = length / (RScaleX*scaleX*10.0);
  	bY = nHeight / (RScaleY*scaleY*10.0);
}
void NxGLScore::updateScoreEnvironmentVariables()
{
	NxEnv newEnv = score->getEnvironment() ;
	////// MODES
	newEnv.visualMode		= visualMode ;
	newEnv.oscOutMode		= oscOutMode ;
	newEnv.oscInMode		= oscInMode ;
	newEnv.showTrajects		= showTrajects ;
	newEnv.showCursors		= showCursors ;
	newEnv.showCurves		= showCurves ;
	newEnv.showTriggers		= showTriggers ;
	////// DISPLAY
	newEnv.showGrid				= showGrid ;
	newEnv.crossesDraw			= crossesDraw ;
	newEnv.scoreDraw			= scoreDraw ;
	newEnv.displayOn			= displayOn ;
	newEnv.circlesDraw			= circlesDraw ;
	newEnv.rectDraw				= rectDraw ;
	newEnv.snapGrid				= snapGrid ;
	newEnv.autoGridStep			= autoGridStep ;
	newEnv.gridStep				= gridStep ;
	newEnv.showCursorsArea		= showCursorsArea ;
	////// TIME / OSC
	newEnv.useCirclesAlgo		= useCirclesAlgo ;
	newEnv.sendIFmessages		= sendIFmessages ;
	newEnv.moveCursors			= moveCursors ;
	newEnv.broadcastMode		= broadcastMode ;
	newEnv.initialTime			= timeScheduler->getInitialTime() ;
	newEnv.finalTime			= timeScheduler->getFinalTime() ;
	
	score->setEnvData( newEnv ) ;
}
void NxGLScore::updateCheckedMenus() 
{	
	mParent->updateCheckedMenus( useCirclesAlgo, moveCursors, broadcastMode, visualMode, sendIFmessages, oscOutMode, oscInMode, snapGrid, showGrid, autoGridStep, showTrajects, showCursors, showCurves, showTriggers, crossesDraw, circlesDraw, rectDraw, showCursorsArea, displayOn, scoreDraw, frustum ) ;
}
void NxGLScore::switchCirclesAlgo() {	
	stopAndReset() ;
	useCirclesAlgo	= !useCirclesAlgo ;
	circlesDraw		= useCirclesAlgo ;
	timeScheduler->useCirclesAlgorithm( useCirclesAlgo, true ) ; // (with reset)
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::switchSendIFmessages() {	
	sendIFmessages = !sendIFmessages ;
	timeScheduler->sendIFmessages( sendIFmessages ) ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::switchMoveCursors() {
	moveCursors = !moveCursors ;
	timeScheduler->moveCursors( moveCursors ) ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::switchBroadcastMode() {	
	broadcastMode = !broadcastMode ;
	timeScheduler->useBroadcastMode( broadcastMode ) ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::switchOSCoutMode() {	
	oscOutMode = !oscOutMode ;
	timeScheduler->setOSCOutActive( oscOutMode ) ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::switchOSCinMode() {	
	oscInMode = !oscInMode ;
	updateUDPListener( oscInMode ) ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::switchVisualMode() {	
	visualMode = !visualMode ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::change2D3DView() {
	selected = NULL ;
	frustum = !frustum;
	frustumView();
	resetView() ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
/////////////////// OBJECT RENDER //////////////////////////
void NxGLScore::showHideTrajects() {
	showTrajects = !showTrajects ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::showHideCursors() {
	showCursors = !showCursors ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::showHideCurves() {
	showCurves = !showCurves ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::showHideTriggers() {
	showTriggers = !showTriggers ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
} //////////////////////////////////////////////////////////
void NxGLScore::showHideCursorsArea() {
	showCursorsArea = !showCursorsArea ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::showHideGrid() {
	showGrid = !showGrid ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::switchAutoGridStep()
{
	autoGridStep = !autoGridStep ;
	if(autoGridStep) {
		gridStep = length/10.0 ;
		if(gridStep > 0.01) gridStepInt = int(floor( gridStep*100 )) ;
		else gridStepInt = 1 ;
	}
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::showHideScore() {
	scoreDraw = !scoreDraw;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::showHideCurvesCircles() {
	circlesDraw = !circlesDraw;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::showHideCurvesRect() {
	rectDraw = !rectDraw;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::showHideCrosses() {
	crossesDraw = !crossesDraw;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::showHideScreenInfos() {
	displayOn = !displayOn;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::switchSnapToGrid() {
	snapGrid = !snapGrid ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}

///////////////////////////////////  
// MOUSE EVENTS
///////////////////////////////////
//////////////////////////////////
// Mouse Press Event
//////////////////////////////////
void NxGLScore::mousePressEvent( QMouseEvent *e )
{
	cursX = 0 ;
	
  	// Converting click coordinates
	clickPos.x = pixToRealX(e->x()) ;
	clickPos.y = pixToRealY(e->y()) ;
	clickPos.z = 0 ;
	if(snapGrid) {
		clickPos.x = snapXY( clickPos.x ) ;
		clickPos.y = snapXY( clickPos.y ) ;
	}

  	// Switches to drag mode if left button is pressed
  	if ( e->button()==Qt::LeftButton && curveCreation==false && curveEdition==false && trajectEdition==false && zoomMode==false )  {
		onClick = true;
		if (selected != NULL ) {
	  		if (selected->getObjectType() != Cursor) 
				objectPos = selected->getPosition();
	  		else 
				savOffset = ( (NxCursor*) selected )->getOffset();
		}
	}	
  	if (curveEdition || trajectEdition || zoomMode) {	
		onDrag = true;
		onClick = false;
	}
	if (curveEdition) curvEdPtChoosed = true ;
}	
	
//////////////////////////////////
// Mouse Move Event
//////////////////////////////////
void NxGLScore::mouseMoveEvent ( QMouseEvent * e )
{
	p.x = pixToRealX( e->x() ) ;
	p.y = pixToRealY( e->y() ) ;
	p.z = 0 ;
	
	if(snapGrid) {
		p.x = snapXY( p.x ) ;
		p.y = snapXY( p.y ) ;
	}		
	
	timePosPix.x = e->x() ;
	timePosPix.y = e->y() ;
	timePos = p;
	
	/*if (gridMove) {
		gridDecX = clickPos.x + p.x ;
		gridDecY = clickPos.y + p.y ;
	}*/
	
  	if (!selectionMode) {
		mouseX = e->x();
		mouseY = e->y();
	}
	
	///////////////////// On Screen Values Display
	if(lineRuleMode) {
		lineRuleLength	= sqrt( pow(timePos.x-lineRulePt1.x,2) + pow(timePos.y-lineRulePt1.y,2)  ) ;
		lineRuleAngle	= 180.0*atan( (lineRulePt1.y - timePos.y) / (lineRulePt1.x - timePos.x) )/PI ; 
	}
	editedTrajectAngle = 180.0*atan( (trajectEdPt1.y - trajectEdPt2.y) / (trajectEdPt1.x - trajectEdPt2.x) )/PI ;
	
	// Flags checking
  	if (onDrag) {
		///////////////////////////////////
		// CURVE EDITION
		///////////////////////////////////
		if ( curveEdition && pointID != -1 ) {
			
			curvEdInfoX = timePos.x - curvEdPosX ;
			curvEdInfoY = (timePos.y - curvEdPosY)/curvEdScale + curvEdMin + curvEdYRange/2 ;
			
			tempP.x = p.x - curvEdPosX ;
			tempP.y = (p.y - curvEdPosY)/curvEdScale + curvEdMin + curvEdYRange/2 ;
			// Force the edited point to stay between its neighbors and in [min, max]
			// anything simpler with a 'constrain' like fonction ?
			if( !curvEdfirst ) {
				if (p.x < curvEdPosX + curvEdXmin ) {
					tempP.x = curvEdXmin ;
					curvEdInfoX = curvEdXmin ;
				}
				if( !curvEdlast && (p.x >curvEdPosX + curvEdXmax) ) {
					tempP.x = curvEdXmax ;
					curvEdInfoX = curvEdXmax ;
				}
			} else {
				tempP.x = 0 ;
				curvEdInfoX = 0 ;
			}
			
			if( p.y > curvEdYmax )	tempP.y = curvEdInfoY = curvEdMax ;
			if( p.y < curvEdYmin )	tempP.y = curvEdInfoY = curvEdMin ;
			
			curvEdOnlyClic = false ;
			// Coords are updated when mouse is released
		}
		///////////////////////////////////////////////
		// TRAJECT EDITION (LINES) // could be simpler ....
		/////////////////////////////////////////////// 
		if ( trajectEdition && selTrajectPoint != -1 ) 
		{
	  		NxPoint cP ;
			tempP.x = p.x ;
			tempP.y = p.y ;
			tempP.z = p.z ; 
			
			switch ( ((NxTraject*) selected )->getOrientation() ) {
				case Horizontal :
					tempP.y = ((NxTraject*) selected )->getInitialPoint().y ;
				break;
				case Vertical :
					tempP.x = ((NxTraject*) selected )->getInitialPoint().x ;
				break;
			}
			tempP.ID = 0 ;
			trajectEdPt2 = tempP ;
			if(selTrajectPoint == 0) { 
				( (NxTraject*) selected )->setInitialPoint( tempP );
				( (NxTraject*) selected )->setPosition(tempP) ;
			}
			if(selTrajectPoint == 1) ( (NxTraject*) selected )->setFinalPoint( tempP );
			
			cP = ((NxTraject*) selected )->getInitialPoint() ;
			((NxTraject*)selected)->evalBoundingBox( cP, BBsize );
			
			///////////////////// Update cursors offsets
			NxCursor *cur ;
			float len = ((NxTraject*)selected)->getLength() ; ;
			QList<int> indexes = score->cursorsIndexes();
			for( int i=0; i< nbCursors; i++ ) {	
				cur = score->getCursor( indexes.at(i) ) ;
				if ( selected->getID() == cur->getParentTrajectory()->getID() ) {	
					cur->reEvalOffsets() ;
					if( (cur->getOffset()>=len) ) cur->setOffset( len ) ;				
				}
			}
			
			////////////////////////////////////////////
			controlCenter->updateParameters( Trajectory, selected->getID() ) ;
		}
	}
  	else
		if (onClick) {
			if (firstClick) {
				posX = e->x();
				posY = e->y();
				firstClick = false;
			}
			
			if ( frustum && mouseRotation ) { // Mouse Rotation for the 3D View
				angleY = oldAngleY + 30*(p.x - clickPos.x) ;
				angleX = oldAngleX - 30*(p.y - clickPos.y) ;
			}
			
			// Drag & Drop		
			if ( selectionMode && selected !=NULL ) {			
				// Updating position (1st control point)
				diffX = p.x - clickPos.x ;
				diffY = p.y - clickPos.y ;
			
				if(snapGrid) {
					objectPos.x = snapXY( objectPos.x ) ;
					objectPos.y = snapXY( objectPos.y ) ;
				}
			
				newPos.x = objectPos.x + diffX ;
				newPos.y = objectPos.y + diffY ;
				newPos.z = objectPos.z ;
				if(snapLockX) newPos.x = objectPos.x ; /// to shift an object vertically (x-locked)
				if(snapLockY) newPos.y = objectPos.y ; /// to shift an object horizontally (y-locked)
			
				///////////// Updating Bounding Box;
				if (selected->getObjectType() != Cursor ) {
					selected->setPosition( newPos );
					selected->evalBoundingBox( newPos, BBsize );
				}
	    	else 
	      		cursX = diffX ;
	  	}
		else {
			if (!mouseRotation) {
				transX += ( ( e->x() )-posX )*(float)(2*length/10.0) /selfW;
				transY += ( posY-( e->y() ) )*(float)(2*nHeight/10.0) /selfH;
				setTranslation( ( ( e->x()-posX ) )*(float)(2*length/10.0) /selfW, ( posY-( e->y() ) )*(float)(2*nHeight/10.0) /selfH, false );
			
				offX += (e->x()) - posX;
				offY += (e->y()) - posY;
			}
	  	}
		//////////// Updating coordinates	
		posX = e->x();
		posY = e->y();
	}
	else
		if (trajectCreation) { ////////// Lines creation
		switch ( (score->getTraject( trajID )->getOrientation() ) ) {
			case Horizontal :
				p.y = score->getTraject( trajID )->getInitialPoint().y;
			break;
			
			case Vertical :
				p.x = score->getTraject( trajID )->getInitialPoint().x;
			break;
		}
		p.z = 0 ;
		trajectEdPt2 = p ;
		score->getTraject( trajID )->setFinalPoint( p );
		score->getTraject( trajID )->evalBoundingBox( score->getTraject( trajID )->getInitialPoint(), BBsize ) ;
		controlCenter->updateItems( 1, true, trajID ); // too soon ?
	}
	else
	if (curveCreation) 
	{
		savPos = p;
	}
	else
	if (selected != NULL && curveEdition && !curvEdPtChoosed) {
		selectCurvePoints( (NxCurve*) selected, e->x(), e->y() ) ;
	}
	else
	if(selected != NULL && trajectEdition) {
		selectTrajectPoints( (NxTraject*) selected, e->x(), e->y() ) ;
	}
	else
	{
		if(!lockedSelection && !frustum) checkSelected( e->x(), e->y() );
	}	
}

///////////////////////////////////////////////////
// Mouse DoubleClick Event
///////////////////////////////////////////////////
void NxGLScore::mouseDoubleClickEvent ( QMouseEvent* e )
{
	if(curveEdition) {
		curveEdition = false ;
		( (NxCurve*) selected )->setPoints( false ) ;
		}
	trajectEdition = false ;
	
	releaseKeyboard();
	float clX = pixToRealX( e->x() ) ;
	float clY = pixToRealY( e->y() ) ;
	if(snapGrid)  {
		clX = snapXY( clX ) ;
		clY = snapXY( clY ) ;
	}
	
  	if (curveCreation)  {
		curveCreation = false;
		
		score->getCurve(savID)->setPoints( false );
		currentPoints = 1;
		editedCurve.clear() ;
	}
  	else {
		if (selected != NULL && selected->getObjectType() == Trajectory )
			cursorMode = true;
	
		/////// Lauching Object Editor
		edited = selected ;
		objectMenu->raiseEditor( clX, clY, edited ) ;
	}
}

////////////////////////////////////////////
// Mouse Release Event
////////////////////////////////////////////
void NxGLScore::mouseReleaseEvent ( QMouseEvent*e ) 
{	
	oldAngleX = angleX ;
	oldAngleY = angleY ;
	
	gridMove = false ;
	firstClick = true ;
	curvEdPtChoosed = false ;
	
	///////////////////////// Zoom Rect Mode
  	if (onDrag && zoomMode) 	{  
		zoomRect( clickPos, p );
	}		
	
  	onClick = false;
  	onDrag = false;	
	
	if(curveEdition && !curvEdOnlyClic) {
		tempP.z = 0.0 ; 
		tempP.ID = IDindex ;
	  	selCurve[IDindex] = tempP;
		( (NxCurve*) selected )->setCurve( selCurve );
		controlCenter->updateParameters( Curve, selected->getID() ) ;
	
		NxPoint curveCenterP = ((NxCurve*)selected)->getPosition() ;
		((NxCurve*)selected)->setPosition( curveCenterP );
		((NxCurve*)selected)->evalBoundingBox( curveCenterP, BBsize );
		curvEdOnlyClic = true ;
	}
	
  	/////////////////////////////
  	// Curve Creation
  	/////////////////////////////
  	if (curveCreation) {
		currentPoints++;
		savPos.ID = currentPoints - 1 ; // The ID begin at 0
		//printf( "CurrentPoints : %i \n", currentPoints );
		float lastx = ((NxPoint)editedCurve.last()).x ;
		float rMin = score->getCurve(savID)->getRealMin() ;
		float rMax = score->getCurve(savID)->getRealMax() ;
		float cScale = score->getCurve(savID)->getScale() ;
		NxPoint ccP = score->getCurve(savID)->getPosition() ;
		
		NxPoint newCurvPt ;
		newCurvPt.ID = savPos.ID ;
		newCurvPt.x = savPos.x - ccP.x ;
		newCurvPt.y = (savPos.y - ccP.y)/cScale + rMin + fabs(rMax-rMin)/2 ;
		newCurvPt.z = 0.0 ;
		
		if( newCurvPt.x < (lastx+curveXMinStep) )		newCurvPt.x = lastx + curveXMinStep ;
		if( newCurvPt.y < rMin )						newCurvPt.y = rMin ;
		if( newCurvPt.y > rMax )						newCurvPt.y = rMax ;
		editedCurve.append( newCurvPt );
		
		//////// Creating or updating curve
		if (currentPoints == 2) {	// Creation
			score->getCurve(savID)->setDrawType( drawMode );
			firstPoint = false ;
	  		score->getCurve( savID )->setCurve( editedCurve );
			updateIndexes();
			controlCenter->updateItems( 0, true, savID );
			score->getCurve( savID )->setPoints(true);
		}
		if ( currentPoints > 2 ) {	///// Update created curve
			//printf("coords : newCurvPt	%f %f\n", newCurvPt.x, newCurvPt.y) ;	
	  		score->getCurve( savID )->setCurve( editedCurve );
			updateIndexes();
			controlCenter->updateItems( 0, true, savID );
		}
	}

  	///////////////////////////////
  	// Editing trajectory
  	///////////////////////////////
  	if ( trajectCreation ) {
		NxPoint p1;

		p.x = pixToRealX( e->x() ) ;
		p.y = pixToRealY( e->y() ) ;
		p.z = 0 ;
		if(snapGrid)  {
			p.x = snapXY( p.x ) ;
			p.y = snapXY( p.y ) ;
		}
		
		p1 = score->getTraject( trajID )->getInitialPoint();
		
		switch ( score->getTraject( trajID )->getOrientation() ) {
			case Horizontal :
				p.y = p1.y;
			break;
			case Vertical :
				p.x = p1.x;
			break;
		}
		p.ID = 0 ;
		score->getTraject( trajID )->setFinalPoint( p );
		score->getTraject( trajID )->evalBoundingBox( p1, BBsize );
		trajectCreation = false;
	}
	
  	///////////////////////////	
  	// Testing selection
  	///////////////////////////
	QList<int> indexes;  

	if ( selected !=NULL ) {
		float len, angleC, nX, nY;
		NxPoint tp1, tp2;
		switch( selected->getObjectType() ) {
			case Curve :
	  		break;
			case Trajectory :
	  			tmpSelTraj = (NxTraject*) selected;

				indexes = score->cursorsIndexes();
				
	  			for (int i=0; i<indexes.count(); i++) {
					int idx = indexes.at(i) ;
					tmpCurs = score->getCursor(idx) ;
					
	      			if ( tmpCurs->getParentTrajectory()->getID() == tmpSelTraj->getID() ) {	
		  				switch( tmpSelTraj->getType() ) {
		    				case 0 :
		      				break;
		    				case 1 :
		      					tmpDef = tmpSelTraj->getCircle();
		      					len = 2.0 * tmpDef.radius * PI;
		      					angleC = ( 360 * tmpCurs->getOffset() ) / len;
		      					nX = tmpDef.radius * cos( angleC * PI / 180.0 );
		      					nY = tmpDef.radius * sin ( angleC * PI / 180.0 );
						
		      					init.x = tmpDef.p.x + nX;
		      					init.y = tmpDef.p.y + nY;
		      					tmpCurs->setParentTrajectory( tmpSelTraj );
								tmpCurs->evalBoundingBox( init , BBsize );
		      				break;
							case 2 :
		      					tp1 =  tmpSelTraj->getInitialPoint();
		      					tp2 =  tmpSelTraj->getFinalPoint();
								
		      					len = sqrt( pow( tp2.x - tp1.x, 2.0 ) + pow( tp2.y - tp1.y, 2.0 ) );
								
								if( len==0 || fabs((tp2.x-tp1.x)-len)<0.0001) angleC = 0.0 ;
								else angleC = acos( (tp2.x-tp1.x)/len )*180/PI ; 
								
		      					if (len<0) 
									len = -len;
											
		      					////// Adjusting angle value with line orientation
		      					if ( ( tp1.x > tp2.x && !( tp1.y < tp2.y ) ) || tp1.y > tp2.y )
									angleC = -angleC;
								
		      					dec.x= tp1.x + ( tmpCurs->getOffset() ) * cos( angleC * PI / 180.0 );
		      					dec.y= tp1.y + ( tmpCurs->getOffset() ) * sin( angleC * PI / 180.0 );
		      					tmpCurs->setParentTrajectory( tmpSelTraj );
								tmpCurs->evalBoundingBox( dec , BBsize );
		      				break;
						}
					}
				}	  		
			break;
				
			case Cursor :
	  		break;
			
			case Trigger :
	  		break;
		}
		////// Restoring default
		selected->setSelected( false );
	}
}

void NxGLScore::repaint()
{}

/////////////////////////////////////////////////////////////////////////
// SELECTION ALGORITHM ( 2D )
/////////////////////////////////////////////////////////////////////////
void NxGLScore::checkSelected( int curX, int curY )
{
	if( selectedTrajectList.size()!=0 )		selectedTrajectList.clear() ;
	if( selectedCursorList.size()!=0 )		selectedCursorList.clear() ;
	if( selectedCurveList.size()!=0 )		selectedCurveList.clear() ;
	if( selectedTriggerList.size()!=0 )		selectedTriggerList.clear() ;
	
  	selected = NULL;
	
	selX = pixToRealX( curX ) ;
	selY = pixToRealY( curY ) ;
	
	if(showCursors) { ///////////////////// CURSORS TEST // cursors curves and triggers can be better (see trajects...)
		QList<int> curind = score->cursorsIndexes();
		for (int i=0; i<curind.count(); i++) {
			int idx = curind.at(i);
			curs = score->getCursor(idx);
			if ( curs->isInside(selX,selY) ) {
				selectedCursorList.append( curs ) ;
				if (selectionMode)
					curs->setSelected(true);
				selected = curs;
				curs->setHighlighted(true) ;
			}
			else {
				curs->setSelected(false);
				curs->setHighlighted(false) ;
			}
		}
	}	
	if(showTriggers) { ///////////////////// TRIGGERS TEST
		QList<int> trigind = score->triggersIndexes();
		for (int i=0; i<trigind.count(); i++) {
			int idx = trigind.at(i);
			trig = score->getTrigger(idx);
			if ( trig->isInside(selX,selY) ) {
				selectedTriggerList.append( trig ) ;
				if (selected == NULL) {
					if (selectionMode)
						trig->setSelected(true);
					selected = trig;
					trig->setHighlighted(true) ;
				}
			}	
			else {
				trig->setSelected(false);
				trig->setHighlighted(false) ;
			}
		}
	}	
	if(showCurves) { ///////////////////// CURVES TEST
		QList<int> curvind = score->curvesIndexes();
		for (int i=0; i<curvind.count(); i++) {
			int idx = curvind.at(i);
			curve = score->getCurve(idx);
			if ( curve->isInside(selX,selY) ) {
				selectedCurveList.append( curve ) ;
				if (selected == NULL) {
					if (selectionMode)
						curve->setSelected(true);
					selected = curve;
					curve->setHighlighted(true) ;
				}
			}
			else {
				curve->setSelected(false);
				curve->setHighlighted(false) ;
			}
		}
	}		
	if(showTrajects) { ///////////////////// TRAJECTS TEST
		QList<int> trajind = score->trajectsIndexes();
		for (int i=0; i<trajind.count(); i++) {
			int idx = trajind.at(i);
			traj = score->getTraject(idx);
			if ( traj->isInside(selX, selY, BBsize) ) {
				selectedTrajectList.append( traj ) ;
				if (selected == NULL) {
					if (selectionMode)
						traj->setSelected(true);
					selected = traj;
					traj->setHighlighted(true) ;
				}
			}
			else {
				traj->setSelected(false);
				traj->setHighlighted(false) ;
			}
		}
 	}
}

/////////////////////////////
// CURVE EDITION
/////////////////////////////
void NxGLScore::selectCurvePoints( NxCurve* inCurve, int inX, int inY )
{
  	pointID = -1;
	float rS = 0.002*length/scaleX ; ///// size of the selection zone

	selX = pixToRealX( inX ) ;
	selY = pixToRealY( inY ) ;

	NxPoint editedCenter = inCurve->getPosition() ;
	float cScal = inCurve->getScale() ;
	float cMin = inCurve->getRealMin() ;
	float cMax = inCurve->getRealMax() ;
  	selCurve = inCurve->getCurve();
	
  	for (int i=0; i<inCurve->size(); i++) {
		if (	(selX >= ( editedCenter.x + selCurve[i].x - rS ) ) && 
				(selX <= ( editedCenter.x + selCurve[i].x + rS ) ) && 
				(selY >= ( editedCenter.y + cScal*(selCurve[i].y-cMin-fabs(cMax-cMin)/2) - rS ) ) && 
				(selY <= ( editedCenter.y + cScal*(selCurve[i].y-cMin-fabs(cMax-cMin)/2) + rS ) )		)
		{
			pointID = selCurve[i].ID ;
			IDindex = i ;
		}
	}
	
	////////////// if a point is selected, we pre-calculate the borders ...
	if( curveEdition && pointID != -1 ) {
		curvEdYRange = ((NxCurve*)selected)->getYRange() ;
		curvEdPosX	= selected->getPosition().x ;
		curvEdPosY	= selected->getPosition().y ;
		curvEdScale = ((NxCurve*)selected)->getScale() ;
		curvEdMax	= ((NxCurve*)selected)->getRealMax() ;
		curvEdMin	= ((NxCurve*)selected)->getRealMin() ;
		curvEdYmin	= selected->getPosition().y - curvEdScale*curvEdYRange/2 ;
		curvEdYmax	= selected->getPosition().y + curvEdScale*curvEdYRange/2 ;
		curvEdfirst	= curvEdlast = true ;
		if(IDindex!= 0 ) {	
			curvEdfirst	= false ;
			curvEdXmin	= selCurve[IDindex-1].x + curveXMinStep ;
			curvEd1x	= selCurve[IDindex-1].x + curvEdPosX ;
			curvEd1y	= curvEdPosY + curvEdScale*(selCurve[IDindex-1].y - curvEdMin - curvEdYRange/2 ) ;
		}
		if(IDindex!= ((NxCurve*)selected)->size()-1 ) {
			curvEdlast	= false ;
			curvEdXmax	= selCurve[IDindex+1].x - curveXMinStep ;
			curvEd2x	= selCurve[IDindex+1].x + curvEdPosX ;
			curvEd2y	= curvEdPosY + curvEdScale*(selCurve[IDindex+1].y - curvEdMin - curvEdYRange/2 ) ;
		}
	}
}
/////////////////////////////
// TRAJECT EDITION
/////////////////////////////
void NxGLScore::selectTrajectPoints( NxTraject* inTraject, int inX, int inY )
{
	selTrajectPoint = -1 ;
	
	float rS = 0.002 * length/scaleX ; //// size of the selection zone
	
	NxPoint pt1 = inTraject->getInitialPoint() ;
	NxPoint pt2 = inTraject->getFinalPoint() ;
	
	selX = pixToRealX( inX ) ;
	selY = pixToRealY( inY ) ;

	if (	(selX >= ( pt1.x - rS )) && 
			(selX <= ( pt1.x + rS )) && 
			(selY >= ( pt1.y - rS )) && 
			(selY <= ( pt1.y + rS ))		) 
	{
		selTrajectPoint = 0 ;
		trajectEdPt1 = pt2 ; // For length /angle display calculation
	}
	if (	(selX >= ( pt2.x - rS )) && 
			(selX <= ( pt2.x + rS )) && 
			(selY >= ( pt2.y - rS )) && 
			(selY <= ( pt2.y + rS ))		) 
	{
		selTrajectPoint = 1 ;
		trajectEdPt1 = pt1 ; // For length /angle display calculation
	}
}
/////////////////////////////
// REMOVE A CURVE POINT
/////////////////////////////
void NxGLScore::removeCurvePoint()
{
	if ( curveEdition && pointID != -1 && ( ((NxCurve*)selected)->size() )>2 ) {
		NxPoint newPosi = selected->getPosition() ;
		int initialSize = ((NxCurve*)selected)->size() ;
		
		if( IDindex == 0 ) { // we have to translate all the points
			float xDec = selCurve[1].x - selCurve[0].x ;
			for(int l=0 ; l<initialSize ; l++) 
				selCurve[l].x = selCurve[l].x - xDec ;
			newPosi.x += xDec ; 
			selected->setPosition( newPosi ) ;
			}
			
		for(int l=IDindex ; l<initialSize-1 ; l++) {
			selCurve[l] = selCurve[l+1];
			selCurve[l].ID = l ;
			}
			
		selCurve.remove(initialSize-1) ;
		((NxCurve*)selected)->setCurve( selCurve );
		selected->evalBoundingBox( newPosi, BBsize ) ;
		controlCenter->updateParameters( Curve, selected->getID() ) ;	
	}
}
/////////////////////////////
// ADD A CURVE POINT
/////////////////////////////
void NxGLScore::addCurvePoint()
{
	if ( curveEdition ) {	
		NxPoint newP ;
		newP.z = 0 ;
		int initialSize = ((NxCurve*)selected)->size() ;
		int newID ;
		int localID=-1 ;
		bool IDfound = false ;
		float cvMin		= ((NxCurve*)selected)->getRealMin() ;
		float cvMax		= ((NxCurve*)selected)->getRealMax() ;
		float cvScale	= ((NxCurve*)selected)->getScale() ;
		float yRange	= ((NxCurve*)selected)->getYRange() ;
		NxPoint cvP		= selected->getPosition() ;
		
		bool noSpaceLeft = false ; // to avoid adding a point if there isn't any space left
		
		while(!IDfound) { /////// Search new point location
			localID++ ;
			if( (timePos.x < (cvP.x + selCurve[localID].x) ) || (localID == initialSize) ) IDfound = true ;
		}
		
		if( localID!=0 ) noSpaceLeft = ( fabs(selCurve[localID-1].x-selCurve[localID].x)<=curveXMinStep*2 ) ;
		////// we don't do anything if there isn't any x-space left
		if( !noSpaceLeft ) 
		{		
			selCurve.append(newP) ; //// enlarge the array size
			newID = localID ;
		
			for(int l=initialSize-1 ; l>=newID ; l--) {
				selCurve[l+1] = selCurve[l];
				selCurve[l+1].ID = l+1 ;
			}

			newP.x = timePos.x - cvP.x ;
			newP.y = (timePos.y - cvP.y)/cvScale + cvMin + yRange/2 ;
		
			///// Constrain the position
			if( newID != 0 )
				if (timePos.x < (cvP.x + selCurve[newID-1].x + curveXMinStep) )	newP.x = selCurve[newID-1].x + curveXMinStep ;
			if( (newID!=initialSize-1) && (newID!=initialSize) )
				if (timePos.x > (cvP.x + selCurve[newID+1].x - curveXMinStep) )	newP.x = selCurve[newID+1].x - curveXMinStep ;
			if( timePos.y > cvP.y + cvScale*yRange/2 )	newP.y = cvMax ;
			if( timePos.y < cvP.y - cvScale*yRange/2 )	newP.y = cvMin ;
		
			if( newID == 0 ) { // we have to translate all the points
				newP.x = 0.0 ;
				for(int l=1 ; l<initialSize+1 ; l++) {
					selCurve[l].x = selCurve[l].x + fabs(timePos.x - cvP.x) ;
				}
				cvP.x = timePos.x ;
				( (NxCurve*) selected )->setPosition( cvP ) ;
			}
			newP.ID = newID ;
			selCurve[newID] = newP ;
		
			( (NxCurve*) selected )->setCurve( selCurve );
			selected->evalBoundingBox( cvP, BBsize ) ;
			controlCenter->updateParameters( Curve, selected->getID() ) ;
		}
		else printf("No space left for that x - point wasn't added on the curve\n") ;
	}
}
/////////////////////////////////////////////////
// OBJECT DESTRUCTION
/////////////////////////////////////////////////
void NxGLScore::removeSelected()
{
  	destroyObject( selected ); 
	selected = NULL ;
}
void NxGLScore::destroyObject( NxObject* inSelected )
{
  	printf( "Stopping score and Removing Object - ID : %i Type %i \n", inSelected->getID(), inSelected->getObjectType() );
	pause() ; ///// STOP PLAYBACK
	
  	// Must also remove child cursors in case of trajectories...
  	if (inSelected->getObjectType() == Trajectory ) {
		NxCursor *cur ;
		QList<int> indexes = score->cursorsIndexes();
		int nbC = nbCursors ;
		for( int i=0; i< nbC; i++ )
		{	
			cur = score->getCursor( indexes.at(i) ) ;
			if ( inSelected->getID() == cur->getParentTrajectory()->getID() )	
			{	
				controlCenter->updateItems( Cursor, false, cur->getID());
				score->removeObject( cur->getID(), Cursor );
			}
		}
		updateIndexes();
		controlCenter->updateIndexes();
	}
	if (inSelected->getObjectType() == Cursor ) { // we have to remove the cursor in the parent trajectory
		NxTraject *pTraj = ((NxCursor*)inSelected)->getParentTrajectory() ;
		pTraj->removeChild( inSelected->getID() ) ;
	}
  	controlCenter->updateItems( inSelected->getObjectType(), false, inSelected->getID() );	
  	score->removeObject( inSelected->getID(), inSelected->getObjectType() );
  	updateIndexes();
  	controlCenter->updateIndexes();
	if (nbCursors > 0)
		timeScheduler->resetSchedule();
}

////////////////////////
// GUI SLOTS
////////////////////////
bool NxGLScore::save()
{
	bool savedOK = false ;
	if(neverSaved) { 
		savedOK = saveAs() ;
	}
	else {
		score->saveToXML( (score->getEnvironment().path).append(score->getEnvironment().name) ); 
		savedOK = true ;
	}
	return savedOK ;
}
bool NxGLScore::saveAs()
{
	bool savedOK = false ;
	QString XMLfileToSave ;
	fileChooser->selectFile( QString("%1.xml").arg(score->getEnvironment().name) ) ;
	XMLfileToSave = fileChooser->getSaveFileName(
					0,
					score->getEnvironment().name,
					".",
					"XML Files (*.xml)" ) ;

	if ( XMLfileToSave.length() > 1 ) {
		score->saveToXML( XMLfileToSave ) ;	
		neverSaved = false ;
		savedOK = true ;
	}
		
	/////// Set Score Tab Text
	int tFocus = mParent->myTabs->currentIndex();
	QString nStr = QString("[%1x%2] ").arg(length).arg(height) ;
	nStr.append( score->getEnvironment().name ) ;
	mParent->myTabs->setTabText(tFocus, nStr) ;
	
	return savedOK ;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Right-click contextual menu
//////////////////////////////////////////
void NxGLScore::contextMenuEvent( QContextMenuEvent* )
{
  	// Context Menu Init
  	contextMenu = new QMenu;
	contextMenu->setSeparatorsCollapsible(false) ;
  	QLabel *caption = new QLabel( "<font color=darkblue><b>IanniX Menu</b></font>");
  	caption->setAlignment( Qt::AlignCenter );
	
  	// Dynamic popup with objects list
	QMenu *subSelectMenu	= new QMenu("Select Object...") ;
	subUnderMenu			= new QMenu("Select Highlighted Object...") ;
	QMenu *subPasteMenu		= new QMenu("Paste Object...") ;
	//--------------------------------------------------------------------------// DISPLAY / TIME
	QAction *setDisplay = new QAction(tr("&Display..."), this);
	connect(setDisplay, SIGNAL(triggered()), this, SLOT(changeDisplay()) ) ; 
	//--------------------------------------------------------------------------// CURSOR
	QMenu *cursorMenu = new QMenu("Cursor") ;
	QAction *switchPlayF = new QAction(tr("&Play When Finished"), this);
	connect(switchPlayF, SIGNAL(triggered()), this, SLOT(switchPlayFinished()) ) ;
	switchPlayF->setCheckable(true) ; 
	QAction *setCurrentOffs = new QAction(tr("&Set Current Offset..."), this);
	connect(setCurrentOffs, SIGNAL(triggered()), this, SLOT(changeCurrentOffset()) ) ; 
	QAction *setInitOffs = new QAction(tr("&Set Initial Offset"), this);
	connect(setInitOffs, SIGNAL(triggered()), this, SLOT(setInitOffset()) ) ; 
	QAction *setFinalOffs = new QAction(tr("&Set Final Offset"), this);
	connect(setFinalOffs, SIGNAL(triggered()), this, SLOT(setFinalOffset()) ) ; 
	QAction *chgeOffs = new QAction(tr("&Change Offsets..."), this);
	connect(chgeOffs, SIGNAL(triggered()), this, SLOT(changeOffsets()) ) ; 
	QAction *editWidth = new QAction(tr("&Edit Width Timeline"), this);
	connect(editWidth, SIGNAL(triggered()), this, SLOT(editWidth()) ) ; 
	QAction *editSpeed = new QAction(tr("&Edit Speed Timeline"), this);
	connect(editSpeed, SIGNAL(triggered()), this, SLOT(editSpeed()) ) ; 
	//--------------------------------------------------------------------------// TRAJECT
	QMenu *trajectMenu = new QMenu("Trajectory") ;
	QAction *trajectRadius = new QAction(tr("&Change Radius..."), this);
	connect(trajectRadius, SIGNAL(triggered()), this, SLOT(changeTrajectRadius()) ) ; 	
	QAction *trajectLength = new QAction(tr("&Change Length..."), this);
	connect(trajectLength, SIGNAL(triggered()), this, SLOT(changeTrajectLength()) ) ; 
	//--------------------------------------------------------------------------// CURVE
	QMenu *curveMenu = new QMenu("Curve") ;
	QAction *curveMinMax = new QAction(tr("&Change Curve Bounds..."), this);
	connect(curveMinMax, SIGNAL(triggered()), this, SLOT(changeCurveMinMax()) ) ; 
	QAction *curveFlatten = new QAction(tr("&Flatten Curve"), this);
	connect(curveFlatten, SIGNAL(triggered()), this, SLOT(changeFlattenCurve()) ) ; 
	QAction *curveAngle = new QAction(tr("&Change Curve Angle..."), this);
	connect(curveAngle, SIGNAL(triggered()), this, SLOT(changeCurveAngle()) ) ; 	
	QAction *curveScale = new QAction(tr("&Change Curve Graphical Scale..."), this);
	connect(curveScale, SIGNAL(triggered()), this, SLOT(changeCurveScale()) ) ; 
	QAction *curveStep = new QAction(tr("&Change Curve Scan Period..."), this);
	connect(curveStep, SIGNAL(triggered()), this, SLOT(changeCurveSpatialPeriod()) ) ; 
	//--------------------------------------------------------------------------// ALL OBJECTS
	QAction *setActivity = new QAction(tr("&Active"), this);
	connect(setActivity, SIGNAL(triggered()), this, SLOT(switchActivity()) ) ;	
	setActivity->setCheckable(true) ;	
	QAction *moveObject = new QAction(tr("&Move to..."), this);
	connect(moveObject, SIGNAL(triggered()), this, SLOT(changeObjectPos()) ) ;
	QAction *setLayerAct = new QAction(tr("&Set Layer..."), this);
	connect(setLayerAct, SIGNAL(triggered()), this, SLOT(changeObjectLayer()) ) ;	
	QAction *setObjectColor = new QAction(tr("&Set Color..."), this);
	connect(setObjectColor, SIGNAL(triggered()), this, SLOT(changeObjectColor()) ) ;
	QAction *removeObject = new QAction(tr("&Delete"), this);
	connect(removeObject, SIGNAL(triggered()), this, SLOT(removeSelected()) ) ;	
	QAction *setGpActivity = new QAction(tr("&Set Group Activity..."), this);
	connect(setGpActivity, SIGNAL(triggered()), this, SLOT(changeGroupActivity()) ) ;
	QAction *translateGp = new QAction(tr("&Move or Rotate Group..."), this);
	connect(translateGp, SIGNAL(triggered()), this, SLOT(translateGroup()) ) ;	
	QAction *setGroupColor = new QAction(tr("&Set Group Color..."), this);
	connect(setGroupColor, SIGNAL(triggered()), this, SLOT(changeGroupColor()) ) ;	
	QAction *deleteGp = new QAction(tr("&Delete Group..."), this);
	connect(deleteGp, SIGNAL(triggered()), this, SLOT(deleteGroup()) ) ;
	//--------------------------------------------------------------------------// GRID
	QAction *autoStep = new QAction(tr("&Auto Grid Step"), this);
	connect(autoStep, SIGNAL(triggered()), this, SLOT(switchAutoGridStep()) ) ;	
	autoStep->setCheckable(true) ;
	QAction *chgeGridStep = new QAction(tr("&Set grid step..."), this);
	connect(chgeGridStep, SIGNAL(triggered()), this, SLOT(changeGridStep()) ) ;		
	//------------------------ Add MENUS -------------------------------------// 
	contextMenu->addMenu(subUnderMenu);
  	displayToViewUnderObjects(subUnderMenu);
	contextMenu->addMenu(subSelectMenu);
  	displayToSelectObjects(subSelectMenu);
	contextMenu->addSeparator();
	contextMenu->addMenu(subPasteMenu);
  	displayToPasteObjects(subPasteMenu);	
	///////////////////////////
	contextMenu->addSeparator();
	contextMenu->addSeparator();
	contextMenu->addSeparator();
	if (selected != NULL) {	
		QString str = selected->getName() ;
		str.append( QString(" CID:%1 GID:%2").arg( selected->getCustomID() ).arg( selected->getGroupID() ) ) ;
		QAction *objInfos = new QAction( str , this );
		contextMenu->addAction( objInfos ) ;
		objInfos->setEnabled( false ) ;
	} else {
		QAction *objEmptyInfos = new QAction( QString("Nothing Selected") , this );
		contextMenu->addAction( objEmptyInfos ) ;
		objEmptyInfos->setEnabled( false ) ;
		subUnderMenu->setEnabled( false ) ; // Under Cursor Selection
	}
	contextMenu->addSeparator();
	contextMenu->addSeparator();
	contextMenu->addSeparator();
	cursorMenu->addAction( switchPlayF ) ;
	cursorMenu->addAction( setCurrentOffs ) ;
	cursorMenu->addAction( setInitOffs );
	cursorMenu->addAction( setFinalOffs );
	cursorMenu->addAction( chgeOffs );
	//cursorMenu->addSeparator();
	//cursorMenu->addAction( editWidth );
	//cursorMenu->addAction( editSpeed );
	trajectMenu->addAction( trajectRadius );
	trajectMenu->addAction( trajectLength );
	curveMenu->addAction( curveMinMax ) ;
	curveMenu->addAction( curveFlatten ) ;
	curveMenu->addAction( curveAngle ) ;
	curveMenu->addAction( curveScale ) ;
	curveMenu->addAction( curveStep ) ;
	contextMenu->addMenu( cursorMenu ) ;
	contextMenu->addMenu( trajectMenu ) ;
	contextMenu->addMenu( curveMenu ) ;
	contextMenu->addSeparator();
	contextMenu->addAction( setActivity );
	contextMenu->addAction( moveObject );
	contextMenu->addAction( setLayerAct );
	contextMenu->addAction( setObjectColor );
	contextMenu->addAction( removeObject );	
	contextMenu->addSeparator();
	contextMenu->addAction( setGpActivity ) ;
	contextMenu->addAction( translateGp ) ;
	contextMenu->addAction( setGroupColor );
	contextMenu->addAction( deleteGp ) ;
	contextMenu->addSeparator();
	contextMenu->addSeparator();
	contextMenu->addAction( autoStep ) ;
	contextMenu->addAction( chgeGridStep ) ;
	contextMenu->addAction( setDisplay ) ;
	//--------------------------------------------------------------------------//
	setCurrentOffs->setEnabled( false ) ;
	setInitOffs->setEnabled( false ) ;
	setFinalOffs->setEnabled( false ) ;
	chgeOffs->setEnabled( false ) ;
	editWidth->setEnabled( false ) ;
	editSpeed->setEnabled( false ) ;
	trajectRadius->setEnabled( false ) ;
	trajectLength->setEnabled( false ) ;
	curveAngle->setEnabled( false ) ;
	curveScale->setEnabled( false ) ;
	curveMinMax->setEnabled( false ) ;
	curveFlatten->setEnabled( false ) ;
	curveStep->setEnabled( false ) ;
	setActivity->setEnabled( false ) ;
	setLayerAct->setEnabled( false ) ;
	setObjectColor->setEnabled( false ) ;
	moveObject->setEnabled( false ) ;
	removeObject->setEnabled( false ) ;
	curveMenu->setEnabled( false ) ;
	cursorMenu->setEnabled( false ) ;
	trajectMenu->setEnabled( false ) ;
	
	//--------------------------------------------------------------------------//
  	if (selected != NULL) {	
		setActivity->setEnabled( true ) ;
		setLayerAct->setEnabled(true) ;
		setObjectColor->setEnabled( true ) ;
		moveObject->setEnabled( true ) ;
		removeObject->setEnabled( true ) ;
		if ( selected->isActive() ) setActivity->setChecked(true) ;
		else setActivity->setChecked(false) ;
		//--------------------------------------------------------------------------//
		if( selected->getObjectType() == Cursor ) { // CURSOR
			cursorMenu->setEnabled( true ) ;
			editCursor = (NxCursor*) selected ;
			switchPlayF->setEnabled( true ) ;
			if ( editCursor->getPlayFinished() ) switchPlayF->setChecked( true ) ;
			else switchPlayF->setChecked(false) ;
			setCurrentOffs->setEnabled( true ) ;
			setInitOffs->setEnabled( true ) ;
			setFinalOffs->setEnabled( true ) ;
			chgeOffs->setEnabled( true ) ;
			editWidth->setEnabled( true ) ;
			editSpeed->setEnabled( true ) ;
			moveObject->setEnabled( false ) ;
		}		
		if ( ((NxObject*)selected)->getObjectType() == Trajectory ) { // TRAJECT
			setActivity->setEnabled( false ) ;
			trajectMenu->setEnabled( true ) ;
			trajectLength->setEnabled( true ) ;
			if ( ((NxTraject*)selected)->getType() == 1 ) // 1 -> Circle only
				trajectRadius->setEnabled( true ) ;
		}
		if ( selected->getObjectType() == Curve ) { // CURVE
			curveMenu->setEnabled( true ) ;
			curveAngle->setEnabled( true ) ;
			curveScale->setEnabled( true ) ;	
			curveMinMax->setEnabled( true ) ;	
			curveFlatten->setEnabled( true ) ;	
			curveStep->setEnabled( true ) ;
		}
		//--------------------------------------------------------------------------//
	}
	if(autoGridStep) autoStep->setChecked( true ) ;
	else autoStep->setChecked( false ) ;
	
  	///// Opening menu at cursor's position
  	contextMenu->exec( QCursor::pos() );
  	//delete contextMenu;
}
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////
/////////////// CONTEXTUALS
/////////////// - The first fonction makes the Dialog
/////////////// - The second applies the changes
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////
void NxGLScore::changeGlobalIPandPort()
{
	iportDialog = new QDialog(this) ;
	QString str = QString("Set Triggers/Curves IP and Port") ;
	iportDialog->setWindowTitle( str );
			
	QGroupBox* buttgroup	= new QGroupBox(iportDialog) ;
	QGroupBox* gidgroup		= new QGroupBox(iportDialog) ;
	QGroupBox* whichgroup	= new QGroupBox(iportDialog) ;
	whichgroup->setFlat(true) ;
	QGroupBox* whatgroup	= new QGroupBox(iportDialog) ;
		
	QGridLayout* gidlay		= new QGridLayout(gidgroup) ;
	QHBoxLayout* whichlay	= new QHBoxLayout(whichgroup) ;
	QGridLayout* whatlay	= new QGridLayout(whatgroup) ;
	QHBoxLayout* blay		= new QHBoxLayout(buttgroup) ;
	QVBoxLayout* genlay		= new QVBoxLayout(iportDialog) ;
	/////// WHICH GID
	alltcC					= new QRadioButton("All Objects", gidgroup) ;
	alltcC->setChecked(true) ;
	QRadioButton *onlytcC	= new QRadioButton("Only Group ID", gidgroup ) ;
	onlytcGp				= new QLineEdit("0", gidgroup ) ;
	onlytcGp->setEnabled(false) ;
	gidlay->addWidget( alltcC,		0, 0 ) ;
	gidlay->addWidget( onlytcC,		1, 0 ) ;
	gidlay->addWidget( onlytcGp,	1, 1 ) ;
	/////// WHICH OBJECTS
	ipTriggersC		= new QCheckBox("Triggers", whichgroup) ;
	ipCurvesC		= new QCheckBox("Curves", whichgroup) ;
	ipTriggersC->setChecked(true) ;
	ipCurvesC->setChecked(true) ;
	whichlay->addWidget( ipTriggersC ) ;
	whichlay->addWidget( ipCurvesC ) ;
	/////// WHAT TO DO
	setIPC		= new QCheckBox("Set IP", whatgroup) ;
	setPortC	= new QCheckBox("Set Port", whatgroup) ;
	setIPC->setChecked(true) ;
	setPortC->setChecked(true) ;
	newIPL		= new QLineEdit( whatgroup ) ;
	newPortL	= new QLineEdit( whatgroup ) ;
	newIPL->setText( score->getEnvironment().UDPhost ) ;
	newPortL->setText( QString("%1").arg(score->getEnvironment().UDPout) ) ;
	whatlay->addWidget( setIPC,		0, 0 ) ;
	whatlay->addWidget( newIPL,		0, 1 ) ;
	whatlay->addWidget( setPortC,	1, 0 ) ;
	whatlay->addWidget( newPortL,	1, 1 ) ;
	/////// BUTTONS
	QPushButton* apply = new QPushButton("Apply", buttgroup) ;
	QPushButton* cancel = new QPushButton("Cancel", buttgroup) ;
	blay->addWidget(cancel) ;
	blay->addWidget(apply) ;
	
	buttgroup->setLayout(blay) ;
	gidgroup->setLayout(gidlay) ;
	whichgroup->setLayout(whichlay) ;
	whatgroup->setLayout(whatlay) ;
	genlay->addWidget(gidgroup) ;
	genlay->addWidget(whichgroup) ;
	genlay->addWidget(whatgroup) ;
	genlay->addWidget(buttgroup) ;
	iportDialog->setLayout(genlay) ;

	connect( onlytcC,SIGNAL( toggled(bool) ),	onlytcGp,	SLOT( setEnabled(bool) ) ) ;
	connect( setIPC, SIGNAL( toggled(bool) ),	newIPL,		SLOT( setEnabled(bool) ) ) ;
	connect( setPortC, SIGNAL( toggled(bool) ),	newPortL,	SLOT( setEnabled(bool) ) ) ;
	connect( cancel, SIGNAL( clicked() ), iportDialog,		SLOT( close() ) );
	connect( apply, SIGNAL( clicked() ), this,				SLOT( setGlobalIPandPort() ) );
	
	iportDialog->show() ;
}
void NxGLScore::setGlobalIPandPort()
{
	bool applyAll	= alltcC->isChecked() ;
	int gpeID		= onlytcGp->text().toInt() ;
	bool onTriggers = ipTriggersC->isChecked() ;
	bool onCurves	= ipCurvesC->isChecked() ;
	bool setNewIP	= setIPC->isChecked() ;
	bool setNewPort = setPortC->isChecked() ;
	QString newIP	= newIPL->text() ;
	int newPort		= newPortL->text().toInt() ;
	
	NxTrigger *trigg ;
	NxCurve *curv ;
		
	if(onTriggers) { // apply on Triggers
		QList<int> indtrigg = score->triggersIndexes();
		for(int i=0; i<nbTriggers; i++) {
			trigg = score->getTrigger( indtrigg.at(i) ) ;
			if( trigg->getGroupID() == gpeID || applyAll ) {
				if(setNewIP)	trigg->setHost( newIP ) ;
				if(setNewPort)	trigg->setPort( newPort ) ;
				controlCenter->updateParameters( Trigger, trigg->getID() ) ;
			}
		}
	}
	if(onCurves) { // apply on Curves
		QList<int> indcur = score->curvesIndexes();
		for(int i=0; i<nbCurves; i++) {
			curv = score->getCurve( indcur.at(i) ) ;
			if( curv->getGroupID() == gpeID || applyAll ) {
				if(setNewIP)	curv->setHost( newIP ) ;
				if(setNewPort)	curv->setPort( newPort ) ;
				controlCenter->updateParameters( Curve, curv->getID() ) ;
			}
		}
	}
	
	iportDialog->accept() ;
}
void NxGLScore::changeGlobalSpatialPeriod()
{
	spDialog = new QDialog(this) ;
	QString str = QString("Set Curves Scan Periods") ;
	spDialog->setWindowTitle( str );
			
	QGroupBox* buttgroup = new QGroupBox(spDialog) ;
	QGroupBox* whichgroup = new QGroupBox(spDialog) ;
	QGroupBox* whatgroup = new QGroupBox(spDialog) ;
		
	QGridLayout* whichlay = new QGridLayout(whichgroup) ;
	QGridLayout* whatlay = new QGridLayout(whatgroup) ;
	QHBoxLayout* blay = new QHBoxLayout(buttgroup) ;
	QVBoxLayout* genlay = new QVBoxLayout(spDialog) ;
	/////// WHICH CURVES
	allcC = new QRadioButton("All Curves", whichgroup) ;
	allcC->setChecked(true) ;
	onlycC = new QRadioButton("Only Group ID", whichgroup ) ;
	onlycGp = new QLineEdit("0", whichgroup ) ;
	onlycGp->setEnabled(false) ;
	whichlay->addWidget( allcC,		0, 0 ) ;
	whichlay->addWidget( onlycC,	1, 0 ) ;
	whichlay->addWidget( onlycGp,	1, 1 ) ;
	/////// WHAT TO DO
	setcR = new QRadioButton("Set Value", whatgroup) ;
	setcR->setChecked(true) ;
	shiftcR = new QRadioButton("Shift by", whatgroup ) ;
	setcL = new QSpinBox( whatgroup ) ;
	setcL->setValue( score->getEnvironment().defaultSpatialPeriod ) ;
	setcL->setMinimum( 5 ) ;
	setcL->setMaximum(100000) ;
	shiftcL = new QSpinBox( whatgroup ) ;
	shiftcL->setValue( 0 ) ;
	shiftcL->setMinimum( -100000 ) ;
	shiftcL->setMaximum( 100000 ) ;
	whatlay->addWidget( setcR,		0, 0 ) ;
	whatlay->addWidget( setcL,		0, 1 ) ;
	whatlay->addWidget( shiftcR,	1, 0 ) ;
	whatlay->addWidget( shiftcL,	1, 1 ) ;
	/////// BUTTONS
	QPushButton* apply = new QPushButton("Apply", buttgroup) ;
	QPushButton* cancel = new QPushButton("Cancel", buttgroup) ;
	blay->addWidget(cancel) ;
	blay->addWidget(apply) ;
	
	buttgroup->setLayout(blay) ;
	whichgroup->setLayout(whichlay) ;
	whatgroup->setLayout(whatlay) ;
	genlay->addWidget(whichgroup) ;
	genlay->addWidget(whatgroup) ;
	genlay->addWidget(buttgroup) ;
	spDialog->setLayout(genlay) ;

	onlycGp->setEnabled(false) ;
	shiftcL->setEnabled(false) ;
	connect( onlycC,SIGNAL( toggled(bool) ),	onlycGp,	SLOT( setEnabled(bool) ) ) ;
	connect( setcR, SIGNAL( toggled(bool) ),	setcL,		SLOT( setEnabled(bool) ) ) ;
	connect( shiftcR, SIGNAL( toggled(bool) ),	shiftcL,	SLOT( setEnabled(bool) ) ) ;
	connect( cancel, SIGNAL( clicked() ), spDialog, SLOT( close() ) );
	connect( apply, SIGNAL( clicked() ), this, SLOT( setGlobalSpatialPeriod() ) );
	
	spDialog->show() ;
}
void NxGLScore::setGlobalSpatialPeriod()
{
	bool applyAll	= allcC->isChecked() ;
	int gpeID		= onlycGp->text().toInt() ;
	bool shift		= shiftcR->isChecked() ;
	int set			= setcL->value() ;
	int shif		= shiftcL->value() ;
	
	NxCurve *curv ;
	int newP ;
	
	int nbC = 0 ;
		
	if (set<5 && !shift) set = 5 ; // unuseful because there is a minimum for the spinbox
	
	QList<int> indcur = score->curvesIndexes();
	for(int i=0; i<nbCurves; i++) {
		curv = score->getCurve( indcur.at(i) ) ;
		if(shift) { ///////////////// shift value
			if( curv->getGroupID() == gpeID || applyAll ) {
				newP = curv->getSpatialPeriod() + shif ;
				if (newP<5) newP = 5 ; //////// 5 is the minimum spatial period value
				curv->setSpatialPeriod(newP) ;
				nbC++ ;
			}
		}
		else { /////////////////// set new value
			if( curv->getGroupID() == gpeID || applyAll ) {
				curv->setSpatialPeriod(set) ;
				nbC++ ;
			}
		}
	}
	
	spDialog->accept() ;
	QMessageBox::about(this, "Done", QString("%1 Curve(s) Were Modified").arg(nbC++) ) ;	
}
void NxGLScore::changeGlobalCurvesBounds()
{
	cGlobalMMDialog = new QDialog(this) ;
	QString str = QString("Change Curves Bounds") ;
	cGlobalMMDialog->setWindowTitle( str );
	
	QGroupBox* buttgroup	= new QGroupBox(cGlobalMMDialog) ;
	QGroupBox* choosegroup	= new QGroupBox(cGlobalMMDialog) ; // Groups
	QGroupBox* scagroup		= new QGroupBox(cGlobalMMDialog) ; // Text Field
	QGroupBox* rescgroup	= new QGroupBox(cGlobalMMDialog) ; // CheckBox
	scagroup->setFlat(true) ;
	rescgroup->setFlat(true) ;
		
	QGridLayout* groupLay	= new QGridLayout(choosegroup) ; // Groups
	QGridLayout* poslay		= new QGridLayout(scagroup) ;
	QVBoxLayout* checklay	= new QVBoxLayout(rescgroup) ;
	QHBoxLayout* blay		= new QHBoxLayout(buttgroup) ;
	QVBoxLayout* genlay		= new QVBoxLayout(cGlobalMMDialog) ;
	
	/////// WHICH CURVES
	allBounds		= new QRadioButton("All Curves", choosegroup) ;
	allBounds->setChecked(true) ;
	allBoundsGp		= new QRadioButton("Only Group ID", choosegroup ) ;
	allBoundsGpID	= new QLineEdit("0", choosegroup ) ;
	
	globminSF		= new QLineEdit(scagroup) ;
	globmaxSF		= new QLineEdit(scagroup) ;
	QLabel* minSL	= new QLabel("Min", scagroup) ;
	QLabel* maxSL	= new QLabel("Max", scagroup) ;
	globrescale			= new QCheckBox("Rescale Curve Points to fit the Bounds", rescgroup) ;
	globrescaleG		= new QCheckBox("Keep Graphical Aspect", rescgroup) ;
	globrescaleTrans	= new QCheckBox("Move Object if Needed", rescgroup) ;
	QPushButton* apply		= new QPushButton("Apply", buttgroup) ;
	QPushButton* cancel		= new QPushButton("Cancel", buttgroup) ;
	
	groupLay->addWidget( allBounds,		0, 0 ) ;
	groupLay->addWidget( allBoundsGp,	1, 0 ) ;
	groupLay->addWidget( allBoundsGpID,	1, 1 ) ;
	
	checklay->addWidget( globrescale ) ;
	checklay->addWidget( globrescaleG ) ;
	checklay->addWidget( globrescaleTrans ) ;
	
	poslay->addWidget(minSL ,		0, 0 ) ;
	poslay->addWidget(globminSF ,	0, 1 ) ;
	poslay->addWidget(maxSL ,		1, 0 ) ;
	poslay->addWidget(globmaxSF ,	1, 1 ) ;
	
	blay->addWidget(cancel) ;
	blay->addWidget(apply) ;
	
	choosegroup->setLayout(groupLay) ;
	buttgroup->setLayout(blay) ;
	rescgroup->setLayout(checklay) ;
	scagroup->setLayout(poslay) ;
	
	genlay->addWidget(choosegroup) ;
	genlay->addWidget(scagroup) ;
	genlay->addWidget(rescgroup) ;
	genlay->addWidget(buttgroup) ;
	cGlobalMMDialog->setLayout(genlay) ;
	
	globminSF->setText( "0") ;
	globmaxSF->setText( "1" ) ;
	globminSF->selectAll() ;

	cGlobalMMDialog->show() ;
	
	globrescaleTrans->setEnabled(false) ;
	globrescaleG->setEnabled(false) ;
	allBoundsGpID->setEnabled( false );
	
	connect( allBoundsGp,	SIGNAL( toggled(bool) ),	allBoundsGpID,		SLOT( setEnabled(bool) ) ) ;
	connect( globrescale,	SIGNAL( toggled(bool) ), globrescaleG,			SLOT( setEnabled(bool) ) );
	connect( globrescale,	SIGNAL( toggled(bool) ), globrescaleG,			SLOT( setChecked(bool) ) );
	connect( globrescaleG,	SIGNAL( toggled(bool) ), globrescaleTrans,		SLOT( setEnabled(bool) ) );
	connect( cancel,	SIGNAL( clicked() ), cGlobalMMDialog,		SLOT( close() ) );
	connect( apply,		SIGNAL( clicked() ), this,					SLOT( setGlobalCurvesBounds() ) );
}
void NxGLScore::setGlobalCurvesBounds()
{
	cGlobalMMDialog->accept() ;
	
	//// Get dialog choices
	bool applyAll	= allBounds->isChecked() ;
	int gpeID		= allBoundsGpID->text().toInt() ;
	float newMin	= globminSF->text().toFloat() ;
	float newMax	= globmaxSF->text().toFloat() ;
	bool resc		= globrescale->isChecked() ;
	bool chgescale	= globrescaleG->isChecked() ;
	bool transObj	= globrescaleTrans->isChecked() ;
	//// Initialisations
	NxCurve *curveToBound ;
	float oldRealMin, oldRealMax, oldSca, localMin, localMax, oldYR, newSca, decal ;
	bool pbZero, pbRange ;
	
	///////////// inverse Min and Max if needed
	if( newMax < newMin ) {
		float newTemp = newMin ;
		newMin = newMax ;
		newMax = newTemp ;
	}
	if( newMax == newMin ) pbRange = true ;
			
	QList<int> indcurv = score->curvesIndexes();
	for(int i=0; i<nbCurves; i++) {
		curveToBound = score->getCurve( indcurv.at(i) ) ;
		
		if( applyAll || curveToBound->getGroupID()==gpeID ) {
			pbZero		= false ; // points form an horizontal line
			pbRange		= false ; // min choosed = max choosed
			oldRealMin	= curveToBound->getRealMin() ;
			oldRealMax	= curveToBound->getRealMax() ;
			oldSca		= curveToBound->getScale() ;
				
			if(!resc) {	// Just set the new bounds...
				curveToBound->setRealMin( newMin ) ;
				curveToBound->setRealMax( newMax ) ; 
			}
			else { // Move all curve points
				QVector<NxPoint> points = ((NxCurve*)curveToBound)->getCurve();
				int cSize = ((NxCurve*)curveToBound)->size() ;

				////// Get the local Min and Max
				localMin = points[0].y ;
				localMax = points[0].y ;
				for(int i=1; i<cSize; i++) {
					if (localMin>points[i].y) localMin = points[i].y ;
					if (localMax<points[i].y) localMax = points[i].y ;
				}
				if( localMin == localMax ) pbZero = true ;
				oldYR = fabs(localMax-localMin) * oldSca ;
		
				////// Rescale all points
				if(!pbZero) for(int i=0; i<cSize; i++) points[i].y = newMin + (points[i].y - localMin)*(newMax-newMin)/(localMax-localMin) ;
				else for(int i=0; i<cSize; i++) points[i].y = newMin ;
				////// Set the curve
				curveToBound->setCurve( points ) ;
				curveToBound->setRealMin( newMin ) ;
				curveToBound->setRealMax( newMax ) ; 
	
				if(chgescale) {
					newSca = oldYR / curveToBound->getYRange() ;
					curveToBound->setScale( newSca ) ;
					if(transObj) {
						decal = ( (localMax+localMin)/2.0 - (oldRealMax+oldRealMin)/2.0 )*oldSca ;
						NxPoint oldPos = curveToBound->getPosition() ;
						oldPos.y += decal ;
						curveToBound->setPosition( oldPos ) ;
					}
				}
			}
			curveToBound->evalBoundingBox( curveToBound->getPosition() ,  BBsize  ) ;
			controlCenter->updateParameters( Curve, curveToBound->getID() ) ;
		}
	}
}
void NxGLScore::changeGlobalCurvesFlatten()
{
	globCurvFlatDialog = new QDialog(this) ;
	QString str = QString("Flatten Curves") ;
	globCurvFlatDialog->setWindowTitle( str );
			
	QGroupBox* buttgroup	= new QGroupBox(globCurvFlatDialog) ;
	QGroupBox* whichgroup	= new QGroupBox(globCurvFlatDialog) ;
	QGridLayout* whichlay		= new QGridLayout(whichgroup) ;
	QHBoxLayout* blay			= new QHBoxLayout(buttgroup) ;
	QVBoxLayout* genlay			= new QVBoxLayout(globCurvFlatDialog) ;
	/////// WHICH CURSORS
	allCurvFlat		= new QRadioButton("All Curves", whichgroup) ;
	allCurvFlat->setChecked(true) ;
	onlyCurvFlatGp	= new QRadioButton("Only Group ID", whichgroup ) ;
	onlyCurvFlatID	= new QLineEdit("0", whichgroup ) ;
	whichlay->addWidget( allCurvFlat,		0, 0 ) ;
	whichlay->addWidget( onlyCurvFlatGp,	1, 0 ) ;
	whichlay->addWidget( onlyCurvFlatID,	1, 1 ) ;
	/////// BUTTONS
	QPushButton* apply	= new QPushButton("Apply", buttgroup) ;
	QPushButton* cancel = new QPushButton("Cancel", buttgroup) ;
	blay->addWidget(cancel) ;
	blay->addWidget(apply) ;
	
	buttgroup->setLayout(blay) ;
	whichgroup->setLayout(whichlay) ;
	genlay->addWidget(whichgroup) ;
	genlay->addWidget(buttgroup) ;
	globCurvFlatDialog->setLayout(genlay) ;

	onlyCurvFlatID->setEnabled(false) ;
	connect( onlyCurvFlatGp, SIGNAL( toggled(bool) ),	onlyCurvFlatID,	SLOT( setEnabled(bool) ) ) ;
	connect( cancel, SIGNAL( clicked() ),globCurvFlatDialog,		SLOT( close() ) );
	connect( apply, SIGNAL( clicked() ), this,						SLOT( setGlobalCurvesFlatten() ) );
	
	globCurvFlatDialog->show() ;
}
void NxGLScore::setGlobalCurvesFlatten()
{
	bool applyAll	= allCurvFlat->isChecked() ;
	int gpeID		= onlyCurvFlatID->text().toInt() ;
	
	NxCurve *curveToFlatten ;
	QList<int> indcurv = score->curvesIndexes();
	for(int i=0; i<nbCurves; i++) {
		curveToFlatten = score->getCurve( indcurv.at(i) ) ;
		if( curveToFlatten->getGroupID() == gpeID || applyAll ) {
			flattenCurve( curveToFlatten ) ;
		}
	}
	
	globCurvFlatDialog->accept() ;
}
void NxGLScore::changeGlobalCursorFinished()
{
	cursFinDialog = new QDialog(this) ;
	QString str = QString("Set Cursors Playback if Finished") ;
	cursFinDialog->setWindowTitle( str );
			
	QGroupBox* buttgroup = new QGroupBox(cursFinDialog) ;
	QGroupBox* whichgroup = new QGroupBox(cursFinDialog) ;
	QGroupBox* whatgroup = new QGroupBox(cursFinDialog) ;
		
	QGridLayout* whichlay = new QGridLayout(whichgroup) ;
	QVBoxLayout* whatlay = new QVBoxLayout(whatgroup) ;
	QHBoxLayout* blay = new QHBoxLayout(buttgroup) ;
	QVBoxLayout* genlay = new QVBoxLayout(cursFinDialog) ;
	/////// WHICH CURSORS
	allcursFC = new QRadioButton("All Cursors", whichgroup) ;
	allcursFC->setChecked(true) ;
	onlycursFC = new QRadioButton("Only Group ID", whichgroup ) ;
	onlycursFGp = new QLineEdit("0", whichgroup ) ;
	whichlay->addWidget( allcursFC,		0, 0 ) ;
	whichlay->addWidget( onlycursFC,	1, 0 ) ;
	whichlay->addWidget( onlycursFGp,	1, 1 ) ;
	/////// WHAT TO DO
	playFFcR = new QRadioButton("Play When Finished", whatgroup) ;
	playFFcR->setChecked(true) ;
	muteFFcR = new QRadioButton("Mute When Finished", whatgroup ) ;
	whatlay->addWidget( playFFcR ) ;
	whatlay->addWidget( muteFFcR ) ;
	/////// BUTTONS
	QPushButton* apply = new QPushButton("Apply", buttgroup) ;
	QPushButton* cancel = new QPushButton("Cancel", buttgroup) ;
	blay->addWidget(cancel) ;
	blay->addWidget(apply) ;
	
	buttgroup->setLayout(blay) ;
	whichgroup->setLayout(whichlay) ;
	whatgroup->setLayout(whatlay) ;
	genlay->addWidget(whichgroup) ;
	genlay->addWidget(whatgroup) ;
	genlay->addWidget(buttgroup) ;
	cursFinDialog->setLayout(genlay) ;

	onlycursFGp->setEnabled(false) ;
	connect( onlycursFC, SIGNAL( toggled(bool) ),	onlycursFGp,	SLOT( setEnabled(bool) ) ) ;
	connect( cancel, SIGNAL( clicked() ),cursFinDialog,				SLOT( close() ) );
	connect( apply, SIGNAL( clicked() ), this,						SLOT( setGlobalCursorFinished() ) );
	
	cursFinDialog->show() ;
}
void NxGLScore::setGlobalCursorFinished()
{
	bool applyAll	= allcursFC->isChecked() ;
	int gpeID		= onlycursFGp->text().toInt() ;
	bool cursoPlayF = playFFcR->isChecked() ;
	
	NxCursor *curs ;
	
	QList<int> indcur = score->cursorsIndexes();
	for(int i=0; i<nbCursors; i++) {
		curs = score->getCursor( indcur.at(i) ) ;
		if( curs->getGroupID() == gpeID || applyAll ) {
			curs->setPlayFinished( cursoPlayF ) ;
		}
	}
	cursFinDialog->accept() ;
}
void NxGLScore::changePlaybackTime() 
{
	cPlaybackTimeDialog = new QDialog(this) ;
	QString str = QString("Set Playback") ;
	cPlaybackTimeDialog->setWindowTitle( str );
			
	QGroupBox* buttgroup = new QGroupBox(cPlaybackTimeDialog) ;
	QGroupBox* gridgroup = new QGroupBox(cPlaybackTimeDialog) ;
	gridgroup->setFlat(true) ;
		
	QGridLayout* poslay = new QGridLayout(gridgroup) ;
	QHBoxLayout* blay = new QHBoxLayout(buttgroup) ;
	QVBoxLayout* genlay = new QVBoxLayout(cPlaybackTimeDialog) ;
	
	QLabel* timiL = new QLabel("Initial Time (in s)", gridgroup) ;
	timiF = new QLineEdit("0", gridgroup) ;
	QLabel* timfL = new QLabel("Final Time (in s)", gridgroup) ;
	timfF = new QLineEdit("0", gridgroup) ;
	timiF->setText( QString().setNum(timeScheduler->getInitialTime()) ) ;
	timfF->setText( QString().setNum(timeScheduler->getFinalTime()) ) ;

	QPushButton* apply = new QPushButton("Apply", buttgroup) ;
	QPushButton* cancel = new QPushButton("Cancel", buttgroup) ;
		
	poslay->addWidget(timiL ,  0, 0 ) ;
	poslay->addWidget(timiF ,  0, 1 ) ;
	poslay->addWidget(timfL ,  1, 0 ) ;
	poslay->addWidget(timfF ,  1, 1 ) ;
	
	blay->addWidget(cancel) ;
	blay->addWidget(apply) ;
	
	buttgroup->setLayout(blay) ;
	gridgroup->setLayout(poslay) ;
	
	genlay->addWidget(gridgroup) ;
	genlay->addWidget(buttgroup) ;
	cPlaybackTimeDialog->setLayout(genlay) ;

	timiF->selectAll() ;
	
	cPlaybackTimeDialog->show() ;
	connect( cancel, SIGNAL( clicked() ), cPlaybackTimeDialog, SLOT( close() ) );
	connect( apply, SIGNAL( clicked() ), this, SLOT( setPlaybackTime() ) );
}
void NxGLScore::setPlaybackTime()
{
	float initTime	= timiF->text().toFloat() ;
	float finalTime = timfF->text().toFloat() ;
	if (initTime<0) initTime = 0 ;
	if (finalTime<0) finalTime = 0 ;
	if (finalTime!=0 && initTime>finalTime) {
		float temp = initTime ;
		initTime = finalTime ;
		finalTime = temp ;
	}
	
	timeScheduler->setInitialTime( initTime ) ;
	timeScheduler->setFinalTime( finalTime ) ;
	
	updateScoreEnvironmentVariables() ;
	
	timeScheduler->initScheduler() ;
	cPlaybackTimeDialog->accept() ;
}
void NxGLScore::changeDisplay()
{
	dispDialog = new QDialog(this) ;
	QString str = QString("Change Display") ;
	dispDialog->setWindowTitle( str ) ;
			
	QGroupBox* buttgroup = new QGroupBox(dispDialog) ;
	QGroupBox* disgroup = new QGroupBox(dispDialog) ;
	disgroup->setFlat(true) ;
	
	QVBoxLayout* dislay = new QVBoxLayout(disgroup) ;
	QHBoxLayout* blay = new QHBoxLayout(buttgroup) ;
	QVBoxLayout* genlay = new QVBoxLayout(dispDialog) ;
	
	sWorkspace = new QCheckBox("Show Workspace", disgroup );
	sWorkspace->setChecked( scoreDraw) ;
	sScreenInfos = new QCheckBox("Show On Screen Infos", disgroup );
	sScreenInfos->setChecked( displayOn ) ;
	sGrid = new QCheckBox("Show Grid", disgroup );
	sGrid->setChecked( showGrid ) ;
	sCrosses = new QCheckBox("Show Center Crosses", disgroup );
	sCrosses->setChecked( crossesDraw ) ;
	sCircles = new QCheckBox("Show Curves Circles", disgroup );
	sCircles->setChecked( circlesDraw ) ;
	sRectangles = new QCheckBox("Show Curves Rectangles", disgroup );
	sRectangles->setChecked( rectDraw ) ;
	sCursZones = new QCheckBox("Show Cursors Impact Zones", disgroup );
	sCursZones->setChecked( showCursorsArea ) ;

	QPushButton* apply = new QPushButton("Apply", buttgroup) ;
	QPushButton* cancel = new QPushButton("Cancel", buttgroup) ;
		
	dislay->addWidget( sWorkspace ) ;
	dislay->addWidget( sScreenInfos ) ;
	dislay->addWidget( sGrid ) ;
	dislay->addWidget( sCrosses ) ;
	dislay->addWidget( sCircles ) ;
	dislay->addWidget( sRectangles ) ;
	dislay->addWidget( sCursZones ) ;

	blay->addWidget(cancel) ;
	blay->addWidget(apply) ;
	
	buttgroup->setLayout(blay) ;
	disgroup->setLayout(dislay) ;
	
	genlay->addWidget(disgroup) ;
	genlay->addWidget(buttgroup) ;
	dispDialog->setLayout(genlay) ;

	dispDialog->show() ;
	
	connect( cancel, SIGNAL( clicked() ), dispDialog, SLOT( close() ) );
	connect( apply, SIGNAL( clicked() ), this, SLOT( setChangedDisplay() ) );
}
void NxGLScore::setChangedDisplay()
{
	dispDialog->accept() ;
	
	scoreDraw = sWorkspace->isChecked() ;
	displayOn = sScreenInfos->isChecked() ;
	showGrid = sGrid->isChecked() ;
	crossesDraw = sCrosses->isChecked() ;
	circlesDraw = sCircles->isChecked() ;
	rectDraw = sRectangles->isChecked() ;
	showCursorsArea = sCursZones->isChecked() ;
	
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::setGridOpacity( float inOp )
{
	gridOpacity = inOp ;
}
void NxGLScore::setCursorZonesOpacity( float inOp )
{
	cursZonesOpacity = inOp ;
}
void NxGLScore::changeGridStep()
{
	cGridDialog = new QDialog(this) ;
	QString str = QString("Change Grid Step") ;
	cGridDialog->setWindowTitle( str );
			
	QGroupBox* buttgroup	= new QGroupBox(cGridDialog) ;
	QGroupBox* gridgroup	= new QGroupBox(cGridDialog) ;
	QGroupBox* slidergroup	= new QGroupBox(cGridDialog) ;
	gridgroup->setFlat(true) ;
	slidergroup->setFlat(true) ;
		
	QHBoxLayout* poslay		= new QHBoxLayout(gridgroup) ;
	QHBoxLayout* slidlay	= new QHBoxLayout(slidergroup) ;
	QHBoxLayout* blay		= new QHBoxLayout(buttgroup) ;
	QVBoxLayout* genlay = new QVBoxLayout(cGridDialog) ;
	
	gridStepF			= new QLineEdit(gridgroup) ;
	QLabel* gridStepL	= new QLabel("Grid Step", gridgroup) ;
	gStepSlider		= new QSlider(slidergroup) ;
	gStepSlider->setOrientation( Qt::Horizontal );
	gStepSlider->setRange(1, 100*length); // 100*gridstep
	gStepSlider->setTickPosition(QSlider::NoTicks);
	gStepSlider->setValue( gridStepInt ) ;
	
	QPushButton* apply = new QPushButton("Apply", buttgroup) ;
	QPushButton* cancel = new QPushButton("Cancel", buttgroup) ;
		
	poslay->addWidget(gridStepL) ;
	poslay->addWidget(gridStepF) ;
	slidlay->addWidget(gStepSlider) ;
	blay->addWidget(cancel) ;
	blay->addWidget(apply) ;
	
	buttgroup->setLayout(blay) ;
	gridgroup->setLayout(poslay) ;
	slidergroup->setLayout(slidlay) ;
	
	genlay->addWidget(gridgroup) ;
	genlay->addWidget(slidergroup) ;
	genlay->addWidget(buttgroup) ;
	cGridDialog->setLayout(genlay) ;

	gridStepF->setText( QString("%1").arg( gridStepInt/100.0 ) ) ;
	gridStepF->selectAll() ;
	
	cGridDialog->show() ;
	connect( gStepSlider, SIGNAL( sliderMoved(int) ), this,	SLOT( setDynamicGridStep(int) ) );
	connect( cancel, SIGNAL( clicked() ), cGridDialog,		SLOT( close() ) );
	connect( apply, SIGNAL( clicked() ), this,				SLOT( setGridStep() ) );
}
void NxGLScore::setDynamicGridStep( int val )
{
	autoGridStep = false ;
	gridStep = val/100.0 ;
	if(gridStep > 0.01) gridStepInt = (int)floor(gridStep*100.0) ;
	else gridStepInt = 1 ;
	gridStepF->setText(QString("%1").arg( val/100.0 )) ;
	updateScoreEnvironmentVariables() ;
}
void NxGLScore::setGridStep()
{
	autoGridStep = false ;

	gridStep = floor( ( fabs(gridStepF->text().toFloat()) )*100.0 ) / 100.0 ;
	if(gridStep > 0.01) gridStepInt = (int)floor(gridStep*100.0) ;
	else gridStepInt = 1 ;
	
	updateScoreEnvironmentVariables() ;
	cGridDialog->accept() ;
}
////////// GROUPS
void NxGLScore::changeGroupActivity()
{
	gpActDialog = new QDialog(this) ;
	QString str = QString("Change Activity") ;
	gpActDialog->setWindowTitle( str );
	
	QGroupBox* choicegp		= new QGroupBox(gpActDialog) ;
	QGroupBox* typegroup	= new QGroupBox(gpActDialog) ;	
	QGroupBox* buttgroup	= new QGroupBox(gpActDialog) ;
	typegroup->setFlat(true) ;
		
	QHBoxLayout* hlay		= new QHBoxLayout(typegroup) ;
	QVBoxLayout* typeLay	= new QVBoxLayout(typegroup) ;
	QVBoxLayout* choicelay	= new QVBoxLayout(typegroup) ;
	QGridLayout* idlay		= new QGridLayout(choicegp) ;
	QVBoxLayout* genlay		= new QVBoxLayout(gpActDialog) ;
	QHBoxLayout* blay		= new QHBoxLayout(buttgroup) ;
	
	gpActF = new QLineEdit(choicegp) ;
	QLabel* gpActL = new QLabel("Group ID", choicegp) ;
	setActiveR					= new QRadioButton("Set Active", typegroup ) ;
  	QRadioButton *setInactiveR	= new QRadioButton("Set Inactive", typegroup ) ;
	cursaC	= new QCheckBox("Cursors", typegroup) ;
	curvaC	= new QCheckBox("Curves", typegroup) ; 
	triggaC	= new QCheckBox("Triggers", typegroup) ; 
	cursaC->setChecked( true ) ;
	curvaC->setChecked( true ) ;
	triggaC->setChecked( true ) ;
	QPushButton* apply	= new QPushButton("Apply", buttgroup) ;
	QPushButton* cancel = new QPushButton("Cancel", buttgroup) ;
	
	choicelay->addWidget(setActiveR) ;
	choicelay->addWidget(setInactiveR) ;
	typeLay->addWidget( cursaC ) ;
	typeLay->addWidget( curvaC ) ;
	typeLay->addWidget( triggaC ) ;
	idlay->addWidget(gpActL ,  0, 0 ) ;
	idlay->addWidget(gpActF ,  0, 1 ) ;
	blay->addWidget(cancel) ;
	blay->addWidget(apply) ;
	hlay->addLayout(choicelay) ;
	hlay->addLayout(typeLay) ;
	
	typegroup->setLayout( hlay ) ;
	buttgroup->setLayout(blay) ;
	choicegp->setLayout(choicelay) ;
	
	genlay->addWidget(choicegp) ;
	genlay->addWidget(typegroup) ;
	genlay->addWidget(buttgroup) ;
	gpActDialog->setLayout(genlay) ;

	setActiveR->setChecked(true) ;
	gpActF->setText("0") ;
	gpActF->selectAll() ;
	
	gpActDialog->show() ;
	connect( cancel, SIGNAL( clicked() ), gpActDialog, SLOT( close() ) );
	connect( apply, SIGNAL( clicked() ), this, SLOT( setGroupActivity() ) );
}
void NxGLScore::setGroupActivity()
{
	gpActDialog->accept() ;
	int gpeID = gpActF->text().toInt() ;
	bool act = setActiveR->isChecked() ;

	NxCurve *curv ;
	NxTrigger *trigg ;
	NxCursor *curs ;
	//NxTraject *traj ;
	
	if(cursaC->isChecked()) { ////// Cursors
		QList<int> indcurs = score->cursorsIndexes();
		for(int i=0; i<nbCursors; i++) {
			curs = score->getCursor( indcurs.at(i) ) ;
			if( curs->getGroupID() == gpeID ) {
				curs->setActive(act) ;
			}
		}
	}
	if(curvaC->isChecked()) { ////// Curves
		QList<int> indcur = score->curvesIndexes();
		for(int i=0; i<nbCurves; i++) {
			curv = score->getCurve( indcur.at(i) ) ;
			if( curv->getGroupID() == gpeID ) {
				curv->setActive(act) ;
			}
		}
	}
	if(triggaC->isChecked()) { ////// Triggers
		QList<int> indtrig = score->triggersIndexes();
		for(int i=0; i<nbTriggers; i++) {
			trigg = score->getTrigger( indtrig.at(i) ) ;
			if( trigg->getGroupID() == gpeID ) {
				trigg->setActive(act) ;
			}
		}
	}
}
void NxGLScore::translateGroup()
{
	transDialog = new QDialog(this) ;
	QString str = QString("Move or Rotate Group") ;
	transDialog->setWindowTitle( str );	
	// General GROUPS
	QGroupBox* typegroup = new QGroupBox(transDialog) ;	
	QGroupBox* buttgroup = new QGroupBox(transDialog) ;
	QGroupBox* layergroup = new QGroupBox("New Layer and Group ID", transDialog) ;
	layergroup->setAlignment( Qt::AlignHCenter );
	QGroupBox* choicegroup = new QGroupBox(transDialog) ;
	typegroup->setFlat(true) ;
	
	QGroupBox* GP1 = new QGroupBox("Move", transDialog) ;
	GP1->setAlignment( Qt::AlignHCenter );
	QGroupBox* GP2 = new QGroupBox("Rotate (Curves)", transDialog) ;
	GP2->setAlignment( Qt::AlignHCenter );
	
	QGridLayout* typeLay	= new QGridLayout(typegroup) ;
	QHBoxLayout* gpLay		= new QHBoxLayout(choicegroup) ;
	QGridLayout* layerlay	= new QGridLayout(layergroup) ;
	QHBoxLayout* h1Lay = new QHBoxLayout(GP1) ; // move
	QHBoxLayout* h2Lay = new QHBoxLayout(GP2) ; // rotate
	QVBoxLayout* chpLay		= new QVBoxLayout() ;
	QGridLayout* posLay		= new QGridLayout() ;
	QVBoxLayout* chaLay		= new QVBoxLayout() ;
	QHBoxLayout* angLay		= new QHBoxLayout() ;
	QHBoxLayout* blay	= new QHBoxLayout(buttgroup) ;
	QVBoxLayout* genlay = new QVBoxLayout(transDialog) ;

	trajC	= new QCheckBox("Trajectories"	, typegroup) ;
	cursC	= new QCheckBox("Cursors"		, typegroup) ;
	curvC	= new QCheckBox("Curves"		, typegroup) ; 
	triggC	= new QCheckBox("Triggers"		, typegroup) ; 
	trajC->setChecked( true ) ;
	cursC->setChecked( true ) ;
	curvC->setChecked( true ) ;
	triggC->setChecked( true ) ;
	moveC	= new QRadioButton("Move to", GP1) ; 
	transC	= new QRadioButton("Shift by", GP1) ;	
	rotR	= new QRadioButton("Set Angle", GP2) ;	
	rotsR	= new QRadioButton("Shift Angle by", GP2) ;	
	transC->setChecked(true) ;
	rotsR->setChecked(true) ;
	QLabel* gpL		= new QLabel("Group ID", choicegroup) ;
	gpF				= new QLineEdit(choicegroup) ;
	layerMTC		= new QCheckBox("Set Layer", layergroup) ; 
	layerMTL		= new QLineEdit(layergroup) ;
	groupidMTC		= new QCheckBox("Change Group ID", layergroup) ; 
	groupidMTL		= new QLineEdit(layergroup) ;
	xtF			= new QLineEdit("0", GP1) ;
	ytF			= new QLineEdit("0", GP1) ;
	ztF			= new QLineEdit("0", GP1) ;
	xtF->setMaximumSize( 80, 21 );
	ytF->setMaximumSize( 80, 21 );
	ztF->setMaximumSize( 80, 21 );
	xtC		= new QCheckBox("X", GP1) ; 
	ytC		= new QCheckBox("Y", GP1) ; 
	ztC		= new QCheckBox("Z", GP1) ; 
	xtC->setChecked(true) ;
	ytC->setChecked(true) ;
	ztC->setChecked(true) ;
	angGF				= new QLineEdit("0", GP2) ;
	QLabel* angGL		= new QLabel("Angle", GP2) ;
	angGF->setMaximumSize( 50, 21 );
	QPushButton* apply	= new QPushButton("Apply", buttgroup) ;
	QPushButton* cancel	= new QPushButton("Cancel", buttgroup) ;
	typeLay->addWidget( trajC ,  0, 0 ) ;
	typeLay->addWidget( cursC ,  1, 0 ) ;
	typeLay->addWidget( curvC ,  0, 1 ) ;
	typeLay->addWidget( triggC , 1, 1 ) ;
	gpLay->addWidget(gpL) ;
	gpLay->addWidget(gpF) ;
	chpLay->addWidget(moveC) ;
	chpLay->addWidget(transC) ;
	layerlay->addWidget(layerMTC,	0, 0 ) ;
	layerlay->addWidget(layerMTL,	0, 1 ) ;
	layerlay->addWidget(groupidMTC, 1, 0 ) ;
	layerlay->addWidget(groupidMTL, 1, 1 ) ;
	posLay->addWidget(xtC ,  0, 0 ) ;
	posLay->addWidget(ytC ,  1, 0 ) ;
	posLay->addWidget(ztC ,  2, 0 ) ;
	posLay->addWidget(xtF ,  0, 1 ) ;
	posLay->addWidget(ytF ,  1, 1 ) ;
	posLay->addWidget(ztF ,  2, 1 ) ;
	
	chaLay->addWidget(rotR) ;
	chaLay->addWidget(rotsR) ;
	angLay->addWidget(angGL) ;
	angLay->addWidget(angGF) ;
	blay->addWidget(cancel) ;
	blay->addWidget(apply) ;
	
	typegroup->setLayout( typeLay ) ;
	buttgroup->setLayout(blay) ;
	layergroup->setLayout(layerlay) ;
	choicegroup->setLayout(gpLay) ;
	h1Lay->addLayout( chpLay ) ;
	h1Lay->addLayout( posLay ) ;
	h2Lay->addLayout( chaLay ) ;
	h2Lay->addLayout( angLay ) ;
	GP1->setLayout(h1Lay) ;
	GP2->setLayout(h2Lay) ;
	
	genlay->addWidget(choicegroup) ;
	genlay->addWidget(typegroup) ;
	genlay->addWidget(GP1) ;
	genlay->addWidget(GP2) ;
	genlay->addWidget(layergroup) ;
	genlay->addWidget(buttgroup) ;
	transDialog->setLayout(genlay) ;
	
	gpF->setText("0") ;
	gpF->selectAll() ;
	groupidMTL->setText("0") ;
	groupidMTC->setChecked( false ) ;
	groupidMTL->setEnabled (false ) ;
	layerMTL->setText("0") ;
	layerMTC->setChecked( false ) ;
	layerMTL->setEnabled (false ) ;
	apply->setShortcut(Qt::Key_Return) ;
	transDialog->show() ;
	
	connect( xtC, SIGNAL( toggled(bool) ),		xtF,			SLOT( setEnabled(bool) ) );
	connect( ytC, SIGNAL( toggled(bool) ),		ytF,			SLOT( setEnabled(bool) ) );
	connect( ztC, SIGNAL( toggled(bool) ),		ztF,			SLOT( setEnabled(bool) ) );
	connect( curvC, SIGNAL( toggled(bool) ),	GP2,			SLOT( setEnabled(bool) ) );
	connect( groupidMTC, SIGNAL( toggled(bool) ), groupidMTL,	SLOT( setEnabled(bool) ) );
	connect( layerMTC, SIGNAL( toggled(bool) ), layerMTL,		SLOT( setEnabled(bool) ) );
	connect( cancel, SIGNAL( clicked() ),		transDialog,	SLOT( close() ) );
	connect( apply, SIGNAL( clicked() ),		this,			SLOT( setTranslatedGroup() ) );
}
void NxGLScore::setTranslatedGroup() ///// Translate all the objects of the same ID group
{
	transDialog->accept() ;
	int gpeID			= gpF->text().toInt() ;
	bool trans			= transC->isChecked() ;
	bool setAngle		= rotR->isChecked() ;
	bool setNewLayer	= layerMTC->isChecked() ;
	bool setNewGroupID	= groupidMTC->isChecked() ;
	int newLayer		= layerMTL->text().toInt() ;
	int newGroupID		= groupidMTL->text().toInt() ;
	NxTraject *traj ;
	NxCursor *curs ;
	NxCurve *curv ;
	NxTrigger *trigg ;
	NxPoint oldP ;
	bool setX	= xtC->isChecked() ;
	bool setY	= ytC->isChecked() ;
	bool setZ	= ztC->isChecked() ;
	float tx = xtF->text().toFloat() ;
	float ty = ytF->text().toFloat() ;
	float tz = ztF->text().toFloat() ;
	float tr = angGF->text().toFloat() ;
	
	if(curvC->isChecked()) { ////// Curves
		QList<int> indcur = score->curvesIndexes();
		for(int i=0; i<nbCurves; i++) {
			curv = score->getCurve( indcur.at(i) ) ;
			if( curv->getGroupID() == gpeID ) {
				oldP = curv->getPosition() ;
				if(trans) {
					if(setX) oldP.x += tx ; 
					if(setY) oldP.y += ty ; 
					if(setZ) oldP.z += tz ;
				}
				else {
					if(setX) oldP.x = tx ; 
					if(setY) oldP.y = ty ; 
					if(setZ) oldP.z = tz ;
				}
				curv->setPosition( oldP ) ;
				/////// ANGLES
				float newAngle = tr ;
				if(!setAngle) newAngle += curv->getAngle() ; // angle shifting
				curv->setAngle( newAngle ) ;
				curv->evalBoundingBox( oldP , BBsize ) ; 
				if(setNewLayer)		curv->setLayer(newLayer) ;
				if(setNewGroupID)	curv->setGroupID(newGroupID) ;
				controlCenter->updateParameters( Curve, curv->getID() ) ;
			}
		}
	}
	if(trajC->isChecked()) { ////// Trajects
		QList<int> indtraj = score->trajectsIndexes();
		for(int i=0; i<nbTrajects; i++) {
			traj = score->getTraject( indtraj.at(i) ) ;
			if( traj->getGroupID() == gpeID ) {
				oldP = traj->getPosition() ;
				if(trans) {
					if(setX) oldP.x += tx ; 
					if(setY) oldP.y += ty ; 
					if(setZ) oldP.z += tz ;
				}
				else {
					if(setX) oldP.x = tx ; 
					if(setY) oldP.y = ty ; 
					if(setZ) oldP.z = tz ;
				}
				traj->setPosition( oldP ) ;
				traj->evalBoundingBox( oldP , BBsize ) ; 
				if(setNewLayer)		traj->setLayer(newLayer) ;
				if(setNewGroupID)	traj->setGroupID(newGroupID) ;
				controlCenter->updateParameters( Trajectory, traj->getID() ) ;
			}
		}
	}
	if(triggC->isChecked()) { ////// Triggers
		QList<int> indtrig = score->triggersIndexes();
		for(int i=0; i<nbTriggers; i++) {
			trigg = score->getTrigger( indtrig.at(i) ) ;
			if( trigg->getGroupID() == gpeID ) {
				oldP = trigg->getPosition() ;
				if(trans) {
					if(setX) oldP.x += tx ; 
					if(setY) oldP.y += ty ; 
					if(setZ) oldP.z += tz ;
				}
				else {
					if(setX) oldP.x = tx ; 
					if(setY) oldP.y = ty ; 
					if(setZ) oldP.z = tz ;
				}
				trigg->setPosition( oldP ) ;
				trigg->evalBoundingBox( oldP , BBsize ) ; 
				if(setNewLayer)		trigg->setLayer(newLayer) ;
				if(setNewGroupID)	trigg->setGroupID(newGroupID) ;
				controlCenter->updateParameters( Trigger, trigg->getID() ) ;
			}
		}
	}
	if(cursC->isChecked()) { ////// Cursors (only layer and group ID modification)
		QList<int> indcurs = score->cursorsIndexes();
		for(int i=0; i<nbCursors; i++) {
			curs = score->getCursor( indcurs.at(i) ) ;
			if( curs->getGroupID() == gpeID ) {
				if(setNewLayer)		curs->setLayer(newLayer) ;
				if(setNewGroupID)	curs->setGroupID(newGroupID) ;
				controlCenter->updateParameters( Cursor, curs->getID() ) ;
			}
		}
	}
	assignCursors() ;
}
void NxGLScore::changeGroupColor()
{
	groupColorDialog = new QDialog(this) ;
	QString str = QString("Change Group Color");
	groupColorDialog->setWindowTitle( str );
	
	QGroupBox* choicegp		= new QGroupBox(groupColorDialog) ;
	QGroupBox* typegp		= new QGroupBox(groupColorDialog) ;
	QGroupBox* colgroup		= new QGroupBox(groupColorDialog) ;
	QGroupBox* buttgroup	= new QGroupBox(groupColorDialog) ;
	typegp->setFlat(true) ;
	colgroup->setFlat(true) ;
	
	QHBoxLayout* choicelay	= new QHBoxLayout(choicegp) ;
	QGridLayout* typelay	= new QGridLayout(typegp) ;
	QHBoxLayout* collay		= new QHBoxLayout(colgroup) ;
	QHBoxLayout* blay		= new QHBoxLayout(buttgroup) ;
	QVBoxLayout* genlay		= new QVBoxLayout(groupColorDialog) ;

	QPushButton* color	= new QPushButton("Color", colgroup) ;
	QPushButton* apply	= new QPushButton("Apply", buttgroup) ;
	QPushButton* cancel = new QPushButton("Cancel", buttgroup) ;
		
	gpColorF			= new QLineEdit(choicegp) ;
	QLabel* gpColorL	= new QLabel("Group ID", choicegp) ;
	trajColC			= new QCheckBox("Trajects", typegp) ; 
	cursColC			= new QCheckBox("Cursors", typegp) ;
	curvColC			= new QCheckBox("Curves", typegp) ; 
	triggColC			= new QCheckBox("Triggers", typegp) ; 
	trajColC->setChecked( true ) ;
	cursColC->setChecked( true ) ;
	curvColC->setChecked( true ) ;
	triggColC->setChecked( true ) ;
	
	choicelay->addWidget(gpColorL ,  0, 0 ) ;
	choicelay->addWidget(gpColorF ,  1, 0 ) ;
	typelay->addWidget(trajColC ,	0, 0 ) ;
	typelay->addWidget(cursColC ,	1, 0 ) ;
	typelay->addWidget(curvColC ,	0, 1 ) ;
	typelay->addWidget(triggColC ,  1, 1 ) ;
	collay->addWidget(color) ;
	blay->addWidget(cancel) ;
	blay->addWidget(apply) ;
	
	choicegp->setLayout(choicelay) ;
	typegp->setLayout(typelay) ;
	colgroup->setLayout(collay) ;
	buttgroup->setLayout(blay) ;
	genlay->addWidget(choicegp) ;
	genlay->addWidget(typegp) ;
	genlay->addWidget(colgroup) ;
	genlay->addWidget(buttgroup) ;
	groupColorDialog->setLayout(genlay) ;
	
	gpColorF->setText("0") ;
	gpColorF->selectAll() ;
	dialogColorPicked = false ;
	apply->setShortcut(Qt::Key_Return) ;
	groupColorDialog->show() ;
	
	connect( color, SIGNAL( clicked() ), this, SLOT( pickObjectColor() ) ) ;
	connect( cancel, SIGNAL( clicked() ), groupColorDialog, SLOT( close() ) ) ;
	connect( apply, SIGNAL( clicked() ), this, SLOT( setChangedGroupColor() ) ) ;
}
void NxGLScore::setChangedGroupColor()
{
	groupColorDialog->accept() ;
	
	int gpeID = gpColorF->text().toInt() ;
	
	if(dialogColorPicked) {
		NxCurve *curv ;
		NxTrigger *trigg ;
		NxCursor *curs ;
		NxTraject *traj ;
		
		if(cursColC->isChecked()) { ////// Cursors
			QList<int> indcurs = score->cursorsIndexes();
			for(int i=0; i<nbCursors; i++) {
				curs = score->getCursor( indcurs.at(i) ) ;
				if( curs->getGroupID() == gpeID ) {
					curs->setColor( objectPickedColor ) ;
				}
			}
		}
		if(curvColC->isChecked()) { ////// Curves
			QList<int> indcur = score->curvesIndexes();
			for(int i=0; i<nbCurves; i++) {
				curv = score->getCurve( indcur.at(i) ) ;
				if( curv->getGroupID() == gpeID ) {
					curv->setColor( objectPickedColor ) ;
				}
			}
		}
		if(triggColC->isChecked()) { ////// Triggers
			QList<int> indtrig = score->triggersIndexes();
			for(int i=0; i<nbTriggers; i++) {
				trigg = score->getTrigger( indtrig.at(i) ) ;
				if( trigg->getGroupID() == gpeID ) {
					trigg->setColor( objectPickedColor ) ;
				}
			}
		}
		if(trajColC->isChecked()) { ////// Trajects
			QList<int> indtraj = score->trajectsIndexes();
			for(int i=0; i<nbTrajects; i++) {
				traj = score->getTraject( indtraj.at(i) ) ;
				if( traj->getGroupID() == gpeID ) {
					traj->setColor( objectPickedColor ) ;
				}
			}
		}
	}
	else
		printf("Color wasn't Choosed\n") ;
}
void NxGLScore::deleteGroup() 
{	
	gpDeleteDialog = new QDialog(this) ;
	QString str = QString("Delete Group") ;
	gpDeleteDialog->setWindowTitle( str );
	
	QGroupBox* buttgroup	= new QGroupBox(gpDeleteDialog) ;
	QGroupBox* typegroup	= new QGroupBox(gpDeleteDialog) ;
	QGroupBox* gpgroup		= new QGroupBox(gpDeleteDialog) ;
	typegroup->setFlat(true) ;
		
	QGridLayout* poslay		= new QGridLayout(gpgroup) ;
	QHBoxLayout* blay		= new QHBoxLayout(buttgroup) ;
	QGridLayout* typeLay	= new QGridLayout(typegroup) ;
	QVBoxLayout* genlay		= new QVBoxLayout(gpDeleteDialog) ;
	
	gpDelF = new QLineEdit(gpgroup) ;
	QLabel* gpDelL = new QLabel("Group ID to delete", gpgroup) ;
	trajdC	= new QCheckBox("Trajectories", typegroup) ;
	cursdC	= new QCheckBox("Cursors", typegroup) ;
	curvdC	= new QCheckBox("Curves", typegroup) ; 
	triggdC	= new QCheckBox("Triggers", typegroup) ; 
	trajdC->setChecked( true ) ;
	cursdC->setChecked( true ) ;
	curvdC->setChecked( true ) ;
	triggdC->setChecked( true ) ;
	QPushButton* apply	= new QPushButton("Apply", buttgroup) ;
	QPushButton* cancel = new QPushButton("Cancel", buttgroup) ;
	
	poslay->addWidget(gpDelL ,  0, 0 ) ;
	poslay->addWidget(gpDelF ,  0, 1 ) ;
	typeLay->addWidget( trajdC,		0, 0 ) ;
	typeLay->addWidget( cursdC,		1, 0 ) ;
	typeLay->addWidget( curvdC,		0, 1 ) ;
	typeLay->addWidget( triggdC,	1, 1 ) ;
	blay->addWidget(cancel) ;
	blay->addWidget(apply) ;
	
	gpgroup->setLayout(typeLay) ;
	buttgroup->setLayout(blay) ;
	gpgroup->setLayout(poslay) ;
	genlay->addWidget(gpgroup) ;
	genlay->addWidget(typegroup) ;
	genlay->addWidget(buttgroup) ;
	gpDeleteDialog->setLayout(genlay) ;

	gpDelF->setText("0") ;
	gpDelF->selectAll() ;

	gpDeleteDialog->show() ;
	connect( cancel, SIGNAL( clicked() ), gpDeleteDialog, SLOT( close() ) );
	connect( apply, SIGNAL( clicked() ), this, SLOT( setDeletedGroup() ) );
}
void NxGLScore::setDeletedGroup() 
{
	gpDeleteDialog->accept() ;
	int gpeID = gpDelF->text().toInt() ;
	NxObject *obj ;
	
	if(curvdC->isChecked()) { ////// Curves
		QList<int> indcurv = score->curvesIndexes();
		for(int i=nbCurves-1 ; i>=0; i--) {
			obj = score->getCurve( indcurv.at(i) ) ;
			if( obj->getGroupID() == gpeID ) destroyObject( obj ) ;
		}
	}
	if(cursdC->isChecked()) { ////// Cursors
		QList<int> indcurs = score->cursorsIndexes();
		for(int i=nbCursors-1 ; i>=0; i--) {
			obj = score->getCursor( indcurs.at(i) ) ;
			if( obj->getGroupID() == gpeID ) destroyObject( obj ) ;
		}
	}
	if(trajdC->isChecked()) { ////// Trajects
		QList<int> indtraj = score->trajectsIndexes();
		for(int i=nbTrajects-1 ; i>=0; i--) {
			obj = score->getTraject( indtraj.at(i) ) ;
			if( obj->getGroupID() == gpeID ) destroyObject( obj ) ;
		}
	}
	if(triggdC->isChecked()) { ////// Triggers
		QList<int> indtrig = score->triggersIndexes();
		for(int i=nbTriggers-1 ; i>=0; i--) {
			obj = score->getTrigger( indtrig.at(i) ) ;
			if( obj->getGroupID() == gpeID ) destroyObject( obj ) ;
		}
	}
}
/////////// OBJECTS
void NxGLScore::switchActivity()
{
	((NxObject*)selected)->setActive( !((NxObject*)selected)->isActive() ) ;
}
void NxGLScore::changeObjectPos()
{
	gotoDialog = new QDialog(this) ;
	QString str = QString("ID = %2 / Name = %3").arg(selected->getObjectType()).arg(selected->getCustomID()).arg(selected->getName());
	toBeMoved = selected ;
	gotoDialog->setWindowTitle( str );
			
	QGroupBox* buttgroup	= new QGroupBox(gotoDialog) ;
	QGroupBox* posgroup		= new QGroupBox(gotoDialog) ;
	posgroup->setFlat(true) ;
	
	QGridLayout* poslay		= new QGridLayout(posgroup) ;
	QHBoxLayout* blay		= new QHBoxLayout(buttgroup) ;
	QVBoxLayout* genlay		= new QVBoxLayout(gotoDialog) ;
	
	xF = new QLineEdit(posgroup) ;
	yF = new QLineEdit(posgroup) ;
	zF = new QLineEdit(posgroup) ;
	QLabel* xL = new QLabel("X", posgroup) ;
	QLabel* yL = new QLabel("Y", posgroup) ;
	QLabel* zL = new QLabel("Z", posgroup) ;
	QPushButton* apply = new QPushButton("Apply", buttgroup) ;
	QPushButton* cancel = new QPushButton("Cancel", buttgroup) ;
		
	poslay->addWidget(xL ,  0, 0 ) ;
	poslay->addWidget(yL ,  1, 0 ) ;
	poslay->addWidget(zL ,  2, 0 ) ;
	poslay->addWidget(xF ,  0, 1 ) ;
	poslay->addWidget(yF ,  1, 1 ) ;
	poslay->addWidget(zF ,  2, 1 ) ;
	blay->addWidget(cancel) ;
	blay->addWidget(apply) ;
	
	buttgroup->setLayout(blay) ;
	posgroup->setLayout(poslay) ;
	
	genlay->addWidget(posgroup) ;
	genlay->addWidget(buttgroup) ;
	gotoDialog->setLayout(genlay) ;
	
	// TEXT
	xF->setText( QString("%1").arg(selected->getPosition().x) ) ;
	yF->setText( QString("%1").arg(selected->getPosition().y) ) ;
	zF->setText( QString("%1").arg(selected->getPosition().z) ) ;
	xF->selectAll() ;
	
	gotoDialog->show() ;
	
	connect( cancel, SIGNAL( clicked() ), gotoDialog, SLOT( close() ) );
	connect( apply, SIGNAL( clicked() ), this, SLOT( setChangeObjectPos() ) );
}
void NxGLScore::setChangeObjectPos()
{
	gotoDialog->accept() ;

	if ( toBeMoved->getObjectType() != Cursor )
	{
		NxPoint newPosition ;
		newPosition.x = xF->text().toFloat() ;
		newPosition.y = yF->text().toFloat() ;
		newPosition.z = zF->text().toFloat() ;
		newPosition.ID = 0 ;

		toBeMoved->setPosition( newPosition ) ;
		toBeMoved->evalBoundingBox( newPosition , BBsize ) ; 
	}
	assignCursors() ;
}
void NxGLScore::changeObjectLayer()
{
	toChgLayer = selected ;
	
	cLayDialog = new QDialog(this) ;
	QString str = QString("Change Layer") ;
	cLayDialog->setWindowTitle( str );
	
	QGroupBox* buttgroup	= new QGroupBox(cLayDialog) ;
	QGroupBox* laygroup		= new QGroupBox(cLayDialog) ;
	laygroup->setFlat(true) ;
		
	QGridLayout* poslay		= new QGridLayout(laygroup) ;
	QHBoxLayout* blay		= new QHBoxLayout(buttgroup) ;
	QVBoxLayout* genlay		= new QVBoxLayout(cLayDialog) ;
	
	layF			= new QLineEdit(laygroup) ;
	QLabel* layL	= new QLabel("Layer", laygroup) ;
	QPushButton* apply	= new QPushButton("Apply", buttgroup) ;
	QPushButton* cancel = new QPushButton("Cancel", buttgroup) ;
	
	poslay->addWidget(layL ,  0, 0 ) ;
	poslay->addWidget(layF ,  0, 1 ) ;
	
	blay->addWidget(cancel) ;
	blay->addWidget(apply) ;
	
	buttgroup->setLayout(blay) ;
	laygroup->setLayout(poslay) ;
	
	genlay->addWidget(laygroup) ;
	genlay->addWidget(buttgroup) ;
	cLayDialog->setLayout(genlay) ;
		
	layF->setText( QString("%1").arg( toChgLayer->getLayer() ) ) ;
	layF->selectAll() ;

	cLayDialog->show() ;
	
	connect( cancel, SIGNAL( clicked() ), cLayDialog, SLOT( close() ) );
	connect( apply, SIGNAL( clicked() ), this, SLOT( setChangedObjectLayer() ) );
}
void NxGLScore::setChangedObjectLayer()
{
	cLayDialog->accept() ;
	int newLay = layF->text().toInt() ;
	toChgLayer->setLayer( newLay ) ; 

	controlCenter->updateParameters( toChgLayer->getObjectType(), toChgLayer->getID() ) ;
}
void NxGLScore::changeObjectColor()
{
	NxObject *toBeColored = selected ;
	objectPickedColor.c = colDialogPicker->getColor();	
	if( !(objectPickedColor.c.red()==0 && objectPickedColor.c.green()==0 && objectPickedColor.c.blue()==0) ) 
		toBeColored->setColor( objectPickedColor ) ;
	else
		printf("Color wasn't Choosed\n") ;
}
void NxGLScore::pickObjectColor() 
{
	objectPickedColor.c = colDialogPicker->getColor();	
	if( !(objectPickedColor.c.red()==0 && objectPickedColor.c.green()==0 && objectPickedColor.c.blue()==0) ) 
		dialogColorPicked = true ;
	else {
		dialogColorPicked = false ;
		printf("Color wasn't Choosed\n") ;
	}
}
//////// CURSORS
void NxGLScore::switchPlayFinished()
{
	((NxCursor*)selected)->setPlayFinished( !((NxCursor*)selected)->getPlayFinished() ) ; 
}
void NxGLScore::changeCurrentOffset()
{
	toChgOffsetsCursor = (NxCursor*)selected ;
	
	curOffsDialog = new QDialog(this) ;
	QString str = QString("Set Current Offset") ;
	curOffsDialog->setWindowTitle( str );
			
	QGroupBox* buttgroup	= new QGroupBox(curOffsDialog) ;
	QGroupBox* posgroup		= new QGroupBox(curOffsDialog) ;
	posgroup->setFlat(true) ;
	
	QGridLayout* poslay		= new QGridLayout(posgroup) ;
	QHBoxLayout* blay		= new QHBoxLayout(buttgroup) ;
	QVBoxLayout* genlay		= new QVBoxLayout(curOffsDialog) ;
	
	curOffF			= new QLineEdit(posgroup) ;
	QLabel* curOffL = new QLabel("Offset", posgroup) ;
	QPushButton* apply	= new QPushButton("Apply", buttgroup) ;
	QPushButton* cancel = new QPushButton("Cancel", buttgroup) ;
		
	poslay->addWidget(curOffL ,	0, 0 ) ;
	poslay->addWidget(curOffF ,	1, 0 ) ;

	blay->addWidget(cancel) ;
	blay->addWidget(apply) ;
	
	buttgroup->setLayout(blay) ;
	posgroup->setLayout(poslay) ;
	
	genlay->addWidget(posgroup) ;
	genlay->addWidget(buttgroup) ;
	curOffsDialog->setLayout(genlay) ;
	
	curOffF->setText( QString("%1").arg(toChgOffsetsCursor->getOffset() ) ) ;
	curOffF->selectAll() ;
	
	curOffsDialog->show() ;
	
	connect( cancel, SIGNAL( clicked() ), curOffsDialog, SLOT( close() ) );
	connect( apply, SIGNAL( clicked() ), this, SLOT( setChangedCurrentOffset() ) );
}
void NxGLScore::setChangedCurrentOffset()
{
	curOffsDialog->accept() ;
	float newOffs = curOffF->text().toFloat()  ;
	toChgOffsetsCursor->setOffset( newOffs ) ;
	toChgOffsetsCursor->reEvalOffsets() ; // reajust offsets / length
	
	assignCursors() ; ///// to re-eval BBoxes
}
void NxGLScore::setInitOffset()
{
	float newOffs = ((NxCursor*)selected)->getOffset() ;
	if ( ((NxCursor*)selected)->getForward() )
		((NxCursor*)selected)->setInitOffset( newOffs ) ;
	else {
		float len = ((NxTraject*)((NxCursor*)selected)->getParentTrajectory())->getLength() ;
		((NxCursor*)selected)->setInitOffset( len - newOffs ) ;
	}
	((NxCursor*)selected)->reEvalOffsets() ; // reajust offsets / length
}
void NxGLScore::setFinalOffset()
{
	float newOffs = ((NxCursor*)selected)->getOffset() ;
	if ( ((NxCursor*)selected)->getForward() )
		((NxCursor*)selected)->setFinalOffset( newOffs ) ;
	else {
		float len = ((NxTraject*)((NxCursor*)selected)->getParentTrajectory())->getLength() ;
		((NxCursor*)selected)->setFinalOffset( len - newOffs ) ;
	}
	((NxCursor*)selected)->reEvalOffsets() ; // reajust offsets / length
}
void NxGLScore::changeOffsets()
{
	offsDialog = new QDialog(this) ;
	QString str = QString("Change Offsets") ;
	toChgOffsetsCursor = (NxCursor*)selected ;
	offsDialog->setWindowTitle( str );
			
	QGroupBox* buttgroup	= new QGroupBox(offsDialog) ;
	QGroupBox* posgroup		= new QGroupBox(offsDialog) ;
	posgroup->setFlat(true) ;
	
	QGridLayout* poslay = new QGridLayout(posgroup) ;
	QHBoxLayout* blay	= new QHBoxLayout(buttgroup) ;
	QVBoxLayout* genlay = new QVBoxLayout(offsDialog) ;
	
	initoF	= new QLineEdit(posgroup) ;
	finaloF = new QLineEdit(posgroup) ;
	QLabel* initoL	= new QLabel("Initial Offset", posgroup) ;
	QLabel* finaloL = new QLabel("Final Offset", posgroup) ;
	QPushButton* apply	= new QPushButton("Apply", buttgroup) ;
	QPushButton* cancel = new QPushButton("Cancel", buttgroup) ;
		
	poslay->addWidget(initoL ,		0, 0 ) ;
	poslay->addWidget(finaloL ,		1, 0 ) ;
	poslay->addWidget(initoF ,		0, 1 ) ;
	poslay->addWidget(finaloF ,		1, 1 ) ;
	blay->addWidget(cancel) ;
	blay->addWidget(apply) ;
	
	buttgroup->setLayout(blay) ;
	posgroup->setLayout(poslay) ;
	
	genlay->addWidget(posgroup) ;
	genlay->addWidget(buttgroup) ;
	offsDialog->setLayout(genlay) ;
	
	initoF->setText( QString("%1").arg(toChgOffsetsCursor->getInitOffset() ) ) ;
	finaloF->setText( QString("%1").arg(toChgOffsetsCursor->getFinalOffset() ) ) ;
	initoF->selectAll() ;
	
	offsDialog->show() ;
	
	connect( cancel, SIGNAL( clicked() ), offsDialog, SLOT( close() ) );
	connect( apply, SIGNAL( clicked() ), this, SLOT( setChangedOffsets() ) );
}
void NxGLScore::setChangedOffsets()
{
	offsDialog->accept() ;
	float newInitOffs = initoF->text().toFloat()  ;
	float newFinalOffs = finaloF->text().toFloat() ;
	
	if(newInitOffs>newFinalOffs) {
		float tempOff = newInitOffs ;
		newInitOffs = newFinalOffs ;
		newFinalOffs = tempOff ;
	}
	
	if ( toChgOffsetsCursor->getForward() ) {
		toChgOffsetsCursor->setInitOffset( newInitOffs ) ;
		toChgOffsetsCursor->setFinalOffset( newFinalOffs ) ;
	}
	else {
		float len = ((NxTraject*)toChgOffsetsCursor->getParentTrajectory())->getLength() ;
		toChgOffsetsCursor->setInitOffset( len - newInitOffs ) ;
		toChgOffsetsCursor->setFinalOffset( len - newFinalOffs ) ;
	}
	
	toChgOffsetsCursor->reEvalOffsets() ; ///// reajust offsets / length
	assignCursors() ; ///// to re-eval BBoxes
}
////////////// CURVES
void NxGLScore::changeCurveMinMax()
{
	toChgMMCurve = (NxCurve*)selected ;
	
	cMMDialog = new QDialog(this) ;
	QString str = QString("Change Curve Bounds") ;
	cMMDialog->setWindowTitle( str );
	
	QGroupBox* buttgroup	= new QGroupBox(cMMDialog) ;
	QGroupBox* scagroup		= new QGroupBox(cMMDialog) ; // Text Field
	QGroupBox* rescgroup	= new QGroupBox(cMMDialog) ; // CheckBox
	scagroup->setFlat(true) ;
	rescgroup->setFlat(true) ;
		
	QGridLayout* poslay		= new QGridLayout(scagroup) ;
	QVBoxLayout* checklay	= new QVBoxLayout(rescgroup) ;
	QHBoxLayout* blay		= new QHBoxLayout(buttgroup) ;
	QVBoxLayout* genlay		= new QVBoxLayout(cMMDialog) ;
	
	minSF = new QLineEdit(scagroup) ;
	maxSF = new QLineEdit(scagroup) ;
	QLabel* minSL = new QLabel("Min", scagroup) ;
	QLabel* maxSL = new QLabel("Max", scagroup) ;
	rescale			= new QCheckBox("Rescale Curve Points to fit the Bounds", rescgroup) ;
	rescaleG		= new QCheckBox("Keep Graphical Aspect", rescgroup) ;
	rescaleTrans	= new QCheckBox("Move Object if Needed", rescgroup) ;
	QPushButton* apply = new QPushButton("Apply", buttgroup) ;
	QPushButton* cancel = new QPushButton("Cancel", buttgroup) ;
	
	checklay->addWidget( rescale ) ;
	checklay->addWidget( rescaleG ) ;
	checklay->addWidget( rescaleTrans ) ;
	
	poslay->addWidget(minSL ,  0, 0 ) ;
	poslay->addWidget(minSF ,  0, 1 ) ;
	poslay->addWidget(maxSL ,  1, 0 ) ;
	poslay->addWidget(maxSF ,  1, 1 ) ;
	
	blay->addWidget(cancel) ;
	blay->addWidget(apply) ;
	
	buttgroup->setLayout(blay) ;
	rescgroup->setLayout(checklay) ;
	scagroup->setLayout(poslay) ;
	
	genlay->addWidget(scagroup) ;
	genlay->addWidget(rescgroup) ;
	genlay->addWidget(buttgroup) ;
	cMMDialog->setLayout(genlay) ;
	
	minSF->setText( QString("%1").arg( toChgMMCurve->getRealMin() ) ) ;
	maxSF->setText( QString("%1").arg( toChgMMCurve->getRealMax() ) ) ;
	minSF->selectAll() ;

	cMMDialog->show() ;
	
	rescaleTrans->setEnabled(false) ;
	rescaleG->setEnabled(false) ;
	
	connect( rescale,	SIGNAL( toggled(bool) ), rescaleG,			SLOT( setEnabled(bool) ) );
	connect( rescale,	SIGNAL( toggled(bool) ), rescaleG,			SLOT( setChecked(bool) ) );
	connect( rescaleG,	SIGNAL( toggled(bool) ), rescaleTrans,		SLOT( setEnabled(bool) ) );
	connect( cancel,	SIGNAL( clicked() ), cMMDialog,				SLOT( close() ) );
	connect( apply,		SIGNAL( clicked() ), this,					SLOT( setChangedMinMax() ) );
}
void NxGLScore::setChangedMinMax()
{
	cMMDialog->accept() ;
	float newMin	= minSF->text().toFloat() ;
	float newMax	= maxSF->text().toFloat() ;
	bool resc		= rescale->isChecked() ;
	bool chgescale	= rescaleG->isChecked() ;
	bool transObj	= rescaleTrans->isChecked() ;
	bool pbZero		= false ; // points are an horizontal line
	bool pbRange	= false ; // min choosed = max choosed
	float oldRealMin = toChgMMCurve->getRealMin() ;
	float oldRealMax = toChgMMCurve->getRealMax() ;
	float oldSca	= toChgMMCurve->getScale() ;
	float localMin, localMax, oldYR, newSca, decal ;
	
	/////////// inverse Min and Max if Min > Max
	if( newMax < newMin ) {
		float newTemp = newMin ;
		newMin = newMax ;
		newMax = newTemp ;
	}
	if( newMax == newMin ) pbRange = true ;
		
	if(!resc) { // Just set the new bounds...
		toChgMMCurve->setRealMin( newMin ) ;
		toChgMMCurve->setRealMax( newMax ) ; 
	}
	else { // Move all curve points
		QVector<NxPoint> points = ((NxCurve*)toChgMMCurve)->getCurve();
		int cSize = ((NxCurve*)toChgMMCurve)->size() ;
	
		////// Get the local Min and Max
		localMin = points[0].y ;
		localMax = points[0].y ;
		for(int i=1; i<cSize; i++) {
			if (localMin>points[i].y) localMin = points[i].y ;
			if (localMax<points[i].y) localMax = points[i].y ;
		}
		if( localMin == localMax ) pbZero = true ;
		oldYR = fabs(localMax-localMin) * oldSca ;
		
		////// Rescale all points
		if(!pbZero) for(int i=0; i<cSize; i++) points[i].y = newMin + (points[i].y - localMin)*(newMax-newMin)/(localMax-localMin) ;
		else for(int i=0; i<cSize; i++) points[i].y = newMin ;
		////// Set the curve
		toChgMMCurve->setCurve( points ) ;
		toChgMMCurve->setRealMin( newMin ) ;
		toChgMMCurve->setRealMax( newMax ) ; 
	
		if(chgescale) {	
			newSca = oldYR / toChgMMCurve->getYRange() ;
			toChgMMCurve->setScale( newSca ) ;
			if(transObj) {
				decal = ( (localMax+localMin)/2.0 - (oldRealMax+oldRealMin)/2.0 )*oldSca ;
				NxPoint oldPos = toChgMMCurve->getPosition() ;
				oldPos.y += decal ;
				toChgMMCurve->setPosition( oldPos ) ;
			}
		}
	}
	toChgMMCurve->evalBoundingBox( toChgMMCurve->getPosition() ,  BBsize  ) ;
	controlCenter->updateParameters( Curve, toChgMMCurve->getID() ) ;
}
void NxGLScore::changeFlattenCurve()
{
	flattenCurve( (NxCurve*)selected ) ;
}
void NxGLScore::flattenCurve( NxCurve *toFlattenCurve)
{
	NxPoint curvPos = toFlattenCurve->getPosition() ;
	float curvMin	= toFlattenCurve->getRealMin() ;
	float curvYRge	= toFlattenCurve->getYRange() ;
	float curvScale = toFlattenCurve->getScale() ;
	QVector<NxPoint> points = toFlattenCurve->getCurve();
	int cSize = toFlattenCurve->size() ;
	float newMax, newMin ;
	
	newMin = curvPos.y - curvYRge*curvScale/2 ;
	newMax = curvPos.y + curvYRge*curvScale/2 ;
	
	////// Rescale all points
	for(int i=0; i<cSize; i++) 
		points[i].y = curvPos.y + curvScale*(points[i].y - curvMin - curvYRge/2.0) ;

	toFlattenCurve->setCurve( points ) ;
	toFlattenCurve->setRealMin( newMin ) ;
	toFlattenCurve->setRealMax( newMax ) ; 
	toFlattenCurve->setScale( 1.0 ) ;
	controlCenter->updateParameters( Curve, toFlattenCurve->getID() ) ;
}
void NxGLScore::changeCurveSpatialPeriod()
{
	toChgPeriodCurve = (NxCurve*)selected ;
	
	cPeriodDialog = new QDialog(this) ;
	QString str = QString("Change Curve Scan Period") ;
	cPeriodDialog->setWindowTitle( str );
	
	QGroupBox* buttgroup	= new QGroupBox(cPeriodDialog) ;
	QGroupBox* periodgroup	= new QGroupBox(cPeriodDialog) ;
	periodgroup->setFlat(true) ;
		
	QGridLayout* poslay = new QGridLayout(periodgroup) ;
	QHBoxLayout* blay	= new QHBoxLayout(buttgroup) ;
	QVBoxLayout* genlay = new QVBoxLayout(cPeriodDialog) ;
	
	stepF = new QSpinBox(periodgroup) ;
	stepF->setMinimum(5) ;
	stepF->setMaximum(100000) ;
	QLabel* stepL = new QLabel("Scan Period (in ms)", periodgroup) ;
	QPushButton* apply	= new QPushButton("Apply", buttgroup) ;
	QPushButton* cancel = new QPushButton("Cancel", buttgroup) ;
	
	poslay->addWidget(stepL ,  0, 0 ) ;
	poslay->addWidget(stepF ,  0, 1 ) ;
	
	blay->addWidget(cancel) ;
	blay->addWidget(apply) ;
	
	buttgroup->setLayout(blay) ;
	periodgroup->setLayout(poslay) ;
	
	genlay->addWidget(periodgroup) ;
	genlay->addWidget(buttgroup) ;
	cPeriodDialog->setLayout(genlay) ;
		
	stepF->setValue( toChgPeriodCurve->getSpatialPeriod() ) ;

	cPeriodDialog->setFixedSize(300, 125) ;
	cPeriodDialog->show() ;
	
	connect( cancel, SIGNAL( clicked() ), cPeriodDialog, SLOT( close() ) );
	connect( apply, SIGNAL( clicked() ), this, SLOT( setChangedSpatialPeriod() ) );
}
void NxGLScore::setChangedSpatialPeriod()
{
	cPeriodDialog->accept() ;
	int newPeriod = stepF->value() ;
	toChgPeriodCurve->setSpatialPeriod( newPeriod ) ; 
}
void NxGLScore::changeCurveScale()
{
	toChgScaleCurve = (NxCurve*)selected ;
	
	cScaleDialog = new QDialog(this) ;
	QString str = QString("Change Curve Scale") ;
	cScaleDialog->setWindowTitle( str );
	
	QGroupBox* buttgroup	= new QGroupBox(cScaleDialog) ;
	QGroupBox* scagroup		= new QGroupBox(cScaleDialog) ;
	scagroup->setFlat(true) ;
		
	QGridLayout* poslay = new QGridLayout(scagroup) ;
	QHBoxLayout* blay	= new QHBoxLayout(buttgroup) ;
	QVBoxLayout* genlay = new QVBoxLayout(cScaleDialog) ;
	
	scaF			= new QLineEdit(scagroup) ;
	QLabel* scaL	= new QLabel("Graphical Scale", scagroup) ;
	QPushButton* apply	= new QPushButton("Apply", buttgroup) ;
	QPushButton* cancel = new QPushButton("Cancel", buttgroup) ;
	
	poslay->addWidget(scaL ,  0, 0 ) ;
	poslay->addWidget(scaF ,  0, 1 ) ;
	
	blay->addWidget(cancel) ;
	blay->addWidget(apply) ;
	
	buttgroup->setLayout(blay) ;
	scagroup->setLayout(poslay) ;
	
	genlay->addWidget(scagroup) ;
	genlay->addWidget(buttgroup) ;
	cScaleDialog->setLayout(genlay) ;
		
	scaF->setText( QString("%1").arg( toChgScaleCurve->getScale() ) ) ;
	scaF->selectAll() ;

	cScaleDialog->show() ;
	
	connect( cancel, SIGNAL( clicked() ), cScaleDialog, SLOT( close() ) );
	connect( apply, SIGNAL( clicked() ), this, SLOT( setChangedScale() ) );
}
void NxGLScore::setChangedScale()
{
	cScaleDialog->accept() ;
	float newScale = scaF->text().toFloat() ;
	toChgScaleCurve->setScale( newScale ) ; 
	toChgScaleCurve->evalBoundingBox( toChgScaleCurve->getPosition() ,  BBsize  ) ;
}
void NxGLScore::changeCurveAngle()
{
	toChgAngleCurve = (NxCurve*)selected ;
	
	cAngleDialog = new QDialog(this) ;
	QString str = QString("Change Angle") ;
	cAngleDialog->setWindowTitle( str );
	
	QGroupBox* buttgroup	= new QGroupBox(cAngleDialog) ;
	QGroupBox* anggroup		= new QGroupBox(cAngleDialog) ;
	anggroup->setFlat(true) ;
		
	QGridLayout* poslay = new QGridLayout(anggroup) ;
	QHBoxLayout* blay	= new QHBoxLayout(buttgroup) ;
	QVBoxLayout* genlay = new QVBoxLayout(cAngleDialog) ;
	
	angF			= new QLineEdit(anggroup) ;
	QLabel* angL	= new QLabel("Angle", anggroup) ;
	QPushButton* apply	= new QPushButton("Apply", buttgroup) ;
	QPushButton* cancel = new QPushButton("Cancel", buttgroup) ;
	
	poslay->addWidget(angL ,  0, 0 ) ;
	poslay->addWidget(angF ,  0, 1 ) ;
	
	blay->addWidget(cancel) ;
	blay->addWidget(apply) ;
	
	buttgroup->setLayout(blay) ;
	anggroup->setLayout(poslay) ;
	
	genlay->addWidget(anggroup) ;
	genlay->addWidget(buttgroup) ;
	cAngleDialog->setLayout(genlay) ;
		
	angF->setText( QString("%1").arg( toChgAngleCurve->getAngle() ) ) ;
	angF->selectAll() ;

	cAngleDialog->show() ;
	
	connect( cancel, SIGNAL( clicked() ), cAngleDialog, SLOT( close() ) );
	connect( apply, SIGNAL( clicked() ), this, SLOT( setChangedAngle() ) );
}
void NxGLScore::setChangedAngle()
{
	cAngleDialog->accept() ;
	float newAng = angF->text().toFloat() ;
	toChgAngleCurve->setAngle( newAng ) ; 
}
//////// TRAJECTS
void NxGLScore::changeTrajectRadius()
{
	toChgRadiusTraject = (NxTraject*)selected ;

	cRadiusDialog = new QDialog(this) ;
	QString str = QString("Change Radius") ;
	cRadiusDialog->setWindowTitle( str );
			
	QGroupBox* buttgroup	= new QGroupBox(cRadiusDialog) ;
	QGroupBox* radgroup		= new QGroupBox(cRadiusDialog) ;
	radgroup->setFlat(true) ;
		
	QGridLayout* poslay = new QGridLayout(radgroup) ;
	QHBoxLayout* blay	= new QHBoxLayout(buttgroup) ;
	QVBoxLayout* genlay = new QVBoxLayout(cRadiusDialog) ;
	
	radF			= new QLineEdit(radgroup) ;
	QLabel* radL	= new QLabel("Radius", radgroup) ;

	QPushButton* apply	= new QPushButton("Apply", buttgroup) ;
	QPushButton* cancel = new QPushButton("Cancel", buttgroup) ;
		
	poslay->addWidget(radL ,  0, 0 ) ;
	poslay->addWidget(radF ,  0, 1 ) ;
	
	blay->addWidget(cancel) ;
	blay->addWidget(apply) ;
	
	buttgroup->setLayout(blay) ;
	radgroup->setLayout(poslay) ;
	
	genlay->addWidget(radgroup) ;
	genlay->addWidget(buttgroup) ;
	cRadiusDialog->setLayout(genlay) ;
		
	radF->setText( QString("%1").arg( toChgRadiusTraject->getCircle().radius ) ) ;
	radF->selectAll() ;

	cRadiusDialog->show() ;
	
	connect( cancel, SIGNAL( clicked() ), cRadiusDialog, SLOT( close() ) );
	connect( apply, SIGNAL( clicked() ), this, SLOT( setChangedRadius() ) );
}
void NxGLScore::setChangedRadius()
{
	cRadiusDialog->accept() ;
	Circle newC ;
	float newRadius = fabs( radF->text().toFloat() ) ;
	if(newRadius!=0) { ///// Avoid radius = 0 case
		newC.radius = newRadius ;
		newC.p = toChgRadiusTraject->getPosition() ;
		toChgRadiusTraject->setCircleDef( newC ) ; 
		toChgRadiusTraject->evalBoundingBox( toChgRadiusTraject->getPosition() , 0 ) ;
		/////////////// Adjust cursors offsets ///////////////
		NxCursor *cur ;
		QList<int> indexes = score->cursorsIndexes();
		for( int i=0; i< nbCursors; i++ ) {	
			cur = score->getCursor( indexes.at(i) ) ;
			if ( ((NxTraject*)toChgRadiusTraject)->getID() == cur->getParentTrajectory()->getID() )	
				cur->reEvalOffsets() ;
		}
		/////////////////////////////////////////////////////
		controlCenter->updateParameters( Trajectory, toChgRadiusTraject->getID() ) ;
	}
	else printf("Radius wasn't changed (=0) !\n") ;
}
void NxGLScore::changeTrajectLength()
{
	toChgLengthTraject = (NxTraject*)selected ;

	cLengthDialog = new QDialog(this) ;
	QString str = QString("Change Length") ;

	cLengthDialog->setWindowTitle( str );
			
	QGroupBox* buttgroup	= new QGroupBox(cLengthDialog) ;
	QGroupBox* lengroup		= new QGroupBox(cLengthDialog) ;
	lengroup->setFlat(true) ;
	
	QGridLayout* poslay = new QGridLayout(lengroup) ;
	QHBoxLayout* blay	= new QHBoxLayout(buttgroup) ;
	QVBoxLayout* genlay = new QVBoxLayout(cLengthDialog) ;
	
	lenF			= new QLineEdit(lengroup) ;
	QLabel* lenL	= new QLabel("Length", lengroup) ;

	QPushButton* apply	= new QPushButton("Apply", buttgroup) ;
	QPushButton* cancel = new QPushButton("Cancel", buttgroup) ;
		
	poslay->addWidget(lenL ,  0, 0 ) ;
	poslay->addWidget(lenF ,  0, 1 ) ;
	
	blay->addWidget(cancel) ;
	blay->addWidget(apply) ;
	buttgroup->setLayout(blay) ;
	lengroup->setLayout(poslay) ;
	genlay->addWidget(lengroup) ;
	genlay->addWidget(buttgroup) ;
	cLengthDialog->setLayout(genlay) ;

	lenF->setText( QString("%1").arg( toChgLengthTraject->getLength() ) ) ;
	lenF->selectAll() ;

	cLengthDialog->show() ;
	
	connect( cancel, SIGNAL( clicked() ), cLengthDialog, SLOT( close() ) );
	connect( apply, SIGNAL( clicked() ), this, SLOT( setChangedLength() ) );
}
void NxGLScore::setChangedLength()
{	
	cLengthDialog->accept() ;
	float newLen = fabs( lenF->text().toFloat() ) ;
	if (newLen != 0) {
		toChgLengthTraject->setLength( newLen ) ;
		toChgLengthTraject->evalBoundingBox( toChgLengthTraject->getPosition() ,  BBsize  ) ;
		/////////////// Adjust cursors offsets ///////////////
		NxCursor *cur ;
		QList<int> indexes = score->cursorsIndexes();
		for( int i=0; i< nbCursors; i++ ) {	
			cur = score->getCursor( indexes.at(i) ) ;
			if ( ((NxTraject*)toChgLengthTraject)->getID() == cur->getParentTrajectory()->getID() )	
				cur->reEvalOffsets() ;
		}
		/////////////////////////////////////////////////////	
		controlCenter->updateParameters( Trajectory, toChgLengthTraject->getID() ) ;
		assignCursors() ;
	}
	else printf("Length wasn't changed (=0) !\n") ;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////
////////////////////////////
// Raises IanniX Control Center
void NxGLScore::openControl()
{
	controlCenter->show();
	controlCenter->setFocus();
}
// Raises IanniX Editor
void NxGLScore::openEditor()
{
	objectMenu->raiseEditor( timePos.x, timePos.y, selected );
	objectMenu->show();
	objectMenu->setFocus() ;
}

// Raises 3D Center
void NxGLScore::open3DCenter()
{
}

void NxGLScore::editWidth()
{
	this->releaseKeyboard();
	timeWidthEditor->show();
}

void NxGLScore::editSpeed()
{
	this->releaseKeyboard();
	timeSpeedEditor->show();
}
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
// Dynamic object display in context menu + PASTE
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////
//////// SELECT MENU
////////////////////////////////////////////////////////
void NxGLScore::displayToSelectObjects( QMenu* inMenu )
{
  	QList<int> trajectIndexes 	= score->trajectsIndexes();
	QList<int> cursorIndexes 	= score->cursorsIndexes();
	QList<int> curveIndexes 	= score->curvesIndexes();
  	QList<int> triggerIndexes 	= score->triggersIndexes();
	
  	QMenu *trajMenu 	= new QMenu( QString("Trajects (%1)").arg( trajectIndexes.count() )) ;
	QMenu *cursorMenu	= new QMenu( QString("Cursors (%1)").arg( cursorIndexes.count() )) ;
	QMenu *curveMenu 	= new QMenu( QString("Curves (%1)").arg( curveIndexes.count() )) ;
  	QMenu *trigMenu 	= new QMenu( QString("Triggers (%1)").arg( triggerIndexes.count() )) ;
	
	QString str ;
	
	for(int i=0; i<trajectIndexes.count(); i++) {
		str = QString("%1 - %2").arg( score->getTraject(trajectIndexes.at(i))->getID() ).arg( score->getTraject(trajectIndexes.at(i))->getName() ) ;
		trajMenu->addAction( str );
	}
	for(int i=0; i<cursorIndexes.count(); i++) {
		str = QString("%1 - %2").arg( score->getCursor(cursorIndexes.at(i))->getID() ).arg( score->getCursor(cursorIndexes.at(i))->getName() ) ;
    	cursorMenu->addAction( str );
	}
	for(int i=0; i<curveIndexes.count(); i++) {
		str = QString("%1 - %2").arg( score->getCurve(curveIndexes.at(i))->getID() ).arg( score->getCurve(curveIndexes.at(i))->getName() ) ;
		curveMenu->addAction( str );
	}
	for(int i=0; i<triggerIndexes.count(); i++) {
		str = QString("%1 - %2").arg( score->getTrigger(triggerIndexes.at(i))->getID() ).arg( score->getTrigger(triggerIndexes.at(i))->getName() ) ;
    	trigMenu->addAction( str );
	}
	
	if(trajectIndexes.count()==0)	trajMenu->setEnabled(false) ;
	if(cursorIndexes.count()==0)	cursorMenu->setEnabled(false) ;
	if(curveIndexes.count()==0)		curveMenu->setEnabled(false) ;
	if(triggerIndexes.count()==0)	trigMenu->setEnabled(false) ;
	
	connect(trajMenu, SIGNAL(triggered( QAction* )),	this, SLOT(selectTraject( QAction* )) ) ;
	connect(cursorMenu, SIGNAL(triggered( QAction* )),	this, SLOT(selectCursor( QAction* )) ) ;
	connect(curveMenu, SIGNAL(triggered( QAction* )),	this, SLOT(selectCurve( QAction* )) ) ;
	connect(trigMenu, SIGNAL(triggered( QAction* )),	this, SLOT(selectTrigger( QAction* )) ) ;
	
	inMenu->addMenu(trajMenu);
	inMenu->addMenu(cursorMenu);
  	inMenu->addMenu(curveMenu);
  	inMenu->addMenu(trigMenu);
}
void NxGLScore::selectTraject( QAction *action ) 
{
	QString str = action->text() ;
	int tmp = str.indexOf("-") ;
	int id = str.left(tmp-1).toInt() ;
	selected = score->getTraject( id ) ;
	selected->setSelected(true);
	selected->setHighlighted(true) ;
	lockedSelection = true ;
}
void NxGLScore::selectCursor( QAction *action )
{
	QString str = action->text() ;
	int tmp = str.indexOf("-") ;
	int id = str.left(tmp-1).toInt() ;
	selected = score->getCursor( id ) ;
	selected->setSelected(true);
	selected->setHighlighted(true) ;
	lockedSelection = true ;
}
void NxGLScore::selectCurve( QAction *action )
{
	QString str = action->text() ;
	int tmp = str.indexOf("-") ;
	int id = str.left(tmp-1).toInt() ;
	selected = score->getCurve( id ) ;
	selected->setSelected(true);
	selected->setHighlighted(true) ;
	lockedSelection = true ;
}
void NxGLScore::selectTrigger( QAction *action )
{
	QString str = action->text() ;
	int tmp = str.indexOf("-") ;
	int id = str.left(tmp-1).toInt() ;
	selected = score->getTrigger( id ) ;
	selected->setSelected(true);
	selected->setHighlighted(true) ;
	lockedSelection = true ;
}
////////////////////////////////////////////////////////
//////// PASTE MENU
////////////////////////////////////////////////////////
void NxGLScore::displayToPasteObjects( QMenu* inMenu )
{
	QList<int> trajectIndexes 	= score->trajectsIndexes();
	QList<int> curveIndexes 	= score->curvesIndexes();
  	QList<int> triggerIndexes 	= score->triggersIndexes();
	
	QMenu *trajMenu 	= new QMenu( QString("Trajects (%1)").arg( trajectIndexes.count() )) ;
	QMenu *curveMenu 	= new QMenu( QString("Curves (%1)").arg( curveIndexes.count() )) ;
  	QMenu *trigMenu 	= new QMenu( QString("Triggers (%1)").arg( triggerIndexes.count() )) ;

	for(int i=0; i<trajectIndexes.count(); i++) {
		str = QString("%1 - %2").arg( score->getTraject(trajectIndexes.at(i))->getID() ).arg( score->getTraject(trajectIndexes.at(i))->getName() ) ;
    	trajMenu->addAction( str );
	}
	for(int i=0; i<curveIndexes.count(); i++) {
		str = QString("%1 - %2").arg( score->getCurve(curveIndexes.at(i))->getID() ).arg( score->getCurve(curveIndexes.at(i))->getName() ) ;
		curveMenu->addAction( str );
	}
	for(int i=0; i<triggerIndexes.count(); i++) {
		str = QString("%1 - %2").arg( score->getTrigger(triggerIndexes.at(i))->getID() ).arg( score->getTrigger(triggerIndexes.at(i))->getName() ) ;
    	trigMenu->addAction( str );
	}
	
	if(trajectIndexes.count()==0)	trajMenu->setEnabled(false) ;
	if(curveIndexes.count()==0)		curveMenu->setEnabled(false) ;
	if(triggerIndexes.count()==0)	trigMenu->setEnabled(false) ;
	
	connect(trajMenu, SIGNAL(triggered( QAction* )),	this, SLOT(pasteTraject( QAction* )) ) ;
	connect(curveMenu, SIGNAL(triggered( QAction* )),	this, SLOT(pasteCurve( QAction* )) ) ;
	connect(trigMenu, SIGNAL(triggered( QAction* )),	this, SLOT(pasteTrigger( QAction* )) ) ;
	
	inMenu->addMenu(trajMenu);
  	inMenu->addMenu(curveMenu);
  	inMenu->addMenu(trigMenu);
}
void NxGLScore::pasteTraject(QAction *action)
{
	QString str = action->text() ;
	int tmp = str.indexOf("-") ;
	int id = str.left(tmp-1).toInt() ;

	NxTraject* copiedTraject = score->getTraject( id ) ;
	int incID		= copiedTraject->getCustomID() ;
	int inGroupID	= copiedTraject->getGroupID() ;
	int layer		= copiedTraject->getLayer() ;
	QString inName	= copiedTraject->getName() ;
	Color inColor	= copiedTraject->getColor() ;
	int inID		= score->getCptTraject() ; 
	NxPoint inPos	= getCursor() ; 
	
	switch ( copiedTraject->getType() ) {
		case 1 : ////// Circle
			score->addTraject( inPos, inID, incID, inGroupID, layer, inName, inColor, 1 );
			score->getTraject( inID )->setCircleDef( copiedTraject->getCircle() );
		break;		
		case 2 : ////// Line
			score->addTraject( inPos, inID, incID, inGroupID, layer, inName, inColor, 2 );
			score->getTraject( inID )->setOrientation( copiedTraject->getOrientation() );
			score->getTraject( inID )->setLineDef( copiedTraject->getInitialPoint(), copiedTraject->getFinalPoint() );
		break;
	}
	score->getTraject( inID )->evalBoundingBox( inPos , BBsize ) ;
	updateIndexes();
	controlCenter->updateItems( Trajectory, true, inID );
	score->maxOfIndexes() ;
	
	pause() ; // Stop playback
	
	////////// We now have to copy the correspondant cursors /////////
	QList<int> indCopCur = copiedTraject->getCursorIDs() ;
	for(int i=0; i<indCopCur.count(); i++) {
		NxCursor *toCop = score->getCursor( indCopCur.at(i) ) ;
		// Get cursor infos
		int newCursID = score->getCptCursor() ;
		QString newNam = toCop->getName().append("_c") ;
		// Add cursor
		score->addCursor( toCop->getPosition(), newCursID, toCop->getCustomID(), (NxTraject*)score->getTraject(inID), toCop->getGroupID(), toCop->getLayer(), newNam, toCop->getColor(), toCop->getForward(), toCop->getMode(), 0, toCop->getInitOffset(), toCop->getFinalOffset() );
		score->getCursor(newCursID)->reEvalOffsets() ;
		score->getCursor(newCursID)->setSpeed( toCop->getSpeed() );
		score->getCursor(newCursID)->setWidth( toCop->getWidth() );
		score->getCursor(newCursID)->setPlayFinished( toCop->getPlayFinished() ) ;
		score->getCursor(newCursID)->setOffset(  toCop->getOffset() ) ;
		score->getCursor(newCursID)->evalBoundingBox( toCop->getPosition(), BBsize ) ;
		updateIndexes();
		assignCursors();
		controlCenter->updateItems( Cursor, true, newCursID );	
		score->maxOfIndexes() ;
	}
	timeScheduler->resetSchedule() ;
}
void NxGLScore::pasteCurve(QAction *action)
{
	QString str = action->text() ;
	int tmp = str.indexOf("-") ;
	int id = str.left(tmp-1).toInt() ;

	NxCurve* copiedCurve = score->getCurve( id ) ;
	int incID		= copiedCurve->getCustomID() ;
	int inGroupID	= copiedCurve->getGroupID() ;
	int layer		= copiedCurve->getLayer() ;
	QString inName	= copiedCurve->getName() ;
	int inID		= score->getCptCurve() ;
	Color inColor1	= copiedCurve->getColor() ;
	Color inColor2	= copiedCurve->getCircleColor() ;
	QVector<NxPoint> editedCurve = copiedCurve->getCurve() ;
	float realMin	= copiedCurve->getRealMin() ;
	float realMax	= copiedCurve->getRealMax() ;
	float scale		= copiedCurve->getScale() ;
	float angle		= copiedCurve->getAngle() ;
	int drawt		= copiedCurve->getDrawType() ;
	QString inHost	= copiedCurve->getHost() ;
	int inPort		= copiedCurve->getPort() ;
	int inPeriod	= copiedCurve->getSpatialPeriod() ;
	NxPoint inPos = getCursor() ; 
	
	score->addCurve( inPos, inID, incID, inGroupID, layer, inName, inColor1, inColor2, editedCurve, editedCurve.count(), realMin, realMax, scale, angle, inPeriod, inHost, inPort );
	updateIndexes() ;
	((NxCurve*)score->getCurve( inID ))->setDrawType( drawt ) ;
	controlCenter->updateItems( 0, true, inID ) ;
	((NxCurve*)score->getCurve( inID ))->setPosition(inPos) ;
	((NxCurve*)score->getCurve( inID ))->evalBoundingBox( inPos , 0 ) ;
	
	score->maxOfIndexes() ;
	timeScheduler->addNewCurve() ;
}
void NxGLScore::pasteTrigger(QAction *action)
{
	QString str = action->text() ;
	int tmp = str.indexOf("-") ;
	int id = str.left(tmp-1).toInt() ;

	NxTrigger* copiedTrigger = score->getTrigger( id ) ;
	int incID		= copiedTrigger->getCustomID() ;
	int inGroupID	= copiedTrigger->getGroupID() ;
	int layer		= copiedTrigger->getLayer() ;
	QString inName	= copiedTrigger->getName() ;
	Color inColor1	= copiedTrigger->getColor() ;
	int inID		= score->getCptTrigger() ; 
	int inIntValue	= copiedTrigger->getValue() ;
	QString inHost	= copiedTrigger->getHost() ;
	int inPort		= copiedTrigger->getPort() ;
	NxPoint inPos	= getCursor() ; 
	
	addNewTrigger( inPos, inID, incID, inGroupID, layer, inName, inColor1, inHost, inPort, inIntValue ) ;

	score->maxOfIndexes() ;
	timeScheduler->addNewTrigger() ;
}
////////////////////////////////////////////////////////
//////// UNDER MENU
////////////////////////////////////////////////////////
void NxGLScore::displayToViewUnderObjects( QMenu* inMenu )
{
	QMenu *trajMenu 	= new QMenu( QString("Trajects (%1)").arg( selectedTrajectList.count() )) ;
	QMenu *cursorMenu	= new QMenu( QString("Cursors (%1)").arg( selectedCursorList.count() )) ;
	QMenu *curveMenu 	= new QMenu( QString("Curves (%1)").arg( selectedCurveList.count() )) ;
  	QMenu *trigMenu 	= new QMenu( QString("Triggers (%1)").arg( selectedTriggerList.count() )) ;
	
	QString str ;
	
	for(int i=0; i<selectedTrajectList.count(); i++) {
		str = QString("%1 - %2").arg( ((NxTraject*)selectedTrajectList.at(i))->getID() )
								.arg( ((NxTraject*)selectedTrajectList.at(i))->getName() ) ;
		trajMenu->addAction( str );
	}
	for(int i=0; i<selectedCursorList.count(); i++) {
		str = QString("%1 - %2").arg( ((NxCursor*)selectedCursorList.at(i))->getID() )
								.arg( ((NxCursor*)selectedCursorList.at(i))->getName() ) ;
		cursorMenu->addAction( str );
	}
	for(int i=0; i<selectedCurveList.count(); i++) {
		str = QString("%1 - %2").arg( ((NxCurve*)selectedCurveList.at(i))->getID() )
								.arg( ((NxCurve*)selectedCurveList.at(i))->getName() ) ;
		curveMenu->addAction( str );
	}
	for(int i=0; i<selectedTriggerList.count(); i++) {
		str = QString("%1 - %2").arg( ((NxTrigger*)selectedTriggerList.at(i))->getID() )
								.arg( ((NxTrigger*)selectedTriggerList.at(i))->getName() ) ;
		trigMenu->addAction( str );
	}
	
	if(selectedTrajectList.count()==0)	trajMenu->setEnabled(false) ;
	if(selectedCursorList.count()==0)	cursorMenu->setEnabled(false) ;
	if(selectedCurveList.count()==0)	curveMenu->setEnabled(false) ;
	if(selectedTriggerList.count()==0)	trigMenu->setEnabled(false) ;
	
	connect(trajMenu, SIGNAL(triggered( QAction* )),	this, SLOT(selectTraject( QAction* )) ) ;
	connect(cursorMenu, SIGNAL(triggered( QAction* )),	this, SLOT(selectCursor( QAction* )) ) ;
	connect(curveMenu, SIGNAL(triggered( QAction* )),	this, SLOT(selectCurve( QAction* )) ) ;
	connect(trigMenu, SIGNAL(triggered( QAction* )),	this, SLOT(selectTrigger( QAction* )) ) ;
	
	inMenu->addMenu(trajMenu);
	inMenu->addMenu(cursorMenu);
  	inMenu->addMenu(curveMenu);
  	inMenu->addMenu(trigMenu);
}
////////////////////////////////////////////////////////
// ACCESS
////////////////////////////////////////////////////////
NxControlCenter* NxGLScore::getControlCenter()
{
	return controlCenter ;
}
NxScore* NxGLScore::getPartition()
{
	return score;
}
NxScheduler* NxGLScore::getScheduler()
{
	return timeScheduler ;
}
NxObject* NxGLScore::getSelected()
{
	if (selected != NULL) return selected ;
	else return NULL ;
}
NxPoint NxGLScore::getCursor()
{
	return timePos ;
}
////////////////////////////////////////////////////////
////////////////////////////////////////////////////////
////// Timelines for cursors
void NxGLScore::setTimeline( State* inStates, int chooser )
{
	if (chooser == 0)
		editCursor->setWidthDef( inStates );
	else
		editCursor->setSpeedDef( inStates );
}

//////////////////////////
// Deactivate flags
//////////////////////////
void NxGLScore::flagsOff()
{
  	cursorMode = false;
}

////////////////////////////////////////////
// Cursors initial assignment
////////////////////////////////////////////
void NxGLScore::assignCursors()
{
  	Circle tmpDef;
  	NxPoint pt1, pt2, boxZone ;
	boxZone.ID = pt1.ID = pt2.ID = 0 ;
	boxZone.x = boxZone.y = boxZone.z = 0.0 ; // Initialisation
  	float offsetT, angleT, lenT, nX, nY ;
	int idx ;
  	
	QList<int> indexes = score->cursorsIndexes() ;
	for (int i=0; i<indexes.count(); i++) {
		
		idx = indexes.at(i);
		assCurs		= score->getCursor(idx) ;
		assParent	= assCurs->getParentTrajectory() ;
		
		int typp = assParent->getType() ;

		switch ( typp ) { // Evaluating Bounding Box (from offset value)		
			case 1 : ///////// CIRCLE
				tmpDef	= assParent->getCircle();
				offsetT = assCurs->getOffset();
				lenT	= assParent->getLength();
				angleT	= ( 360.0 * offsetT ) / lenT;
						
				// Recalculate box
				nX = tmpDef.radius * cos( angleT * PI / 180.0 );
				nY = tmpDef.radius * sin( angleT * PI / 180.0 );
				init.x = tmpDef.p.x + nX ;
				init.y = tmpDef.p.y + nY ;	
				assCurs->evalBoundingBox( init, BBsize );
			break;
			case 2 : ///////// LINE
				offsetT = assCurs->getOffset();
				
				pt1 	= assParent->getInitialPoint();
				pt2 	= assParent->getFinalPoint();
						
				// Retrieving angle & length values
				angleT 	= assCurs->getAngle();
				lenT 	= assParent->getLength();		
				
				//printf("offset: %f et %f et %f et BB : %f\n", offsetT, angleT, lenT, BBsize) ;
						
				// Evaluating Bounding Box
				boxZone.x = pt1.x + offsetT*cos( angleT*PI/180.0 );
				boxZone.y = pt1.y + offsetT*sin( angleT*PI/180.0 );

				assCurs->evalBoundingBox( boxZone , BBsize );
			break;
		}
	}
}
/////////////////////////////////
/////////////////////////////////
void NxGLScore::refreshView()
{ 
	BBsize = (float)length/(600*scaleX) ;
	if(visualMode) updateGL();
	mParent->setToolBarTime( this,  timeScheduler->getCurrentTime() ) ;
}
void NxGLScore::setToolBarInfosTxt( QString inTxt )
{
	mParent->setToolBarInfosTxt( inTxt ) ;
}
/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
//////////////////////// PLAYBACK // Timing functions
/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
void NxGLScore::hardOffsetsReset()
{
	NxCursor *cur ;
	QList<int> indexes = score->cursorsIndexes();
	for( int i=0; i<indexes.count(); i++) {
		cur	= score->getCursor(indexes.at(i)) ;
		cur->hardOffsetsReset() ;
	}
	assignCursors();
	timeScheduler->initScheduler() ;
}
void NxGLScore::resetSchedule()
{
	NxCursor *cur ;
	float inInitOffs, inFinalOffs, len ;
	bool forw ;
	
	//pause() ; ///// Stopping playback already done
	
	////// Resetting Cursors Positions
	QList<int> indexes = score->cursorsIndexes();
	for( int i=0; i<indexes.count(); i++)
	{
		cur			= score->getCursor(indexes.at(i)) ;
		inInitOffs	= cur->getInitOffset() ;
		inFinalOffs = cur->getFinalOffset() ;
		forw		= cur->getForward() ;
		len			= ((NxTraject*)cur->getParentTrajectory())->getLength() ;
		
		if( forw ) cur->setOffset( inInitOffs );
			else cur->setOffset( len - inInitOffs );
	}
	
	timeScheduler->initScheduler() ;
	timeScheduler->resetTime() ;
	assignCursors() ;
}
bool NxGLScore::isPlaying()
{
	return ( timeScheduler->isPlaying() ) ;
}
void NxGLScore::playPause()
{
	if (timeScheduler->isReadyToPlay()) {
		if (!timeScheduler->isPlaying() && nbCursors > 0)
		{
			timeScheduler->setPlaying( true ) ;
			timeScheduler->start();
		}
		else {
			pause() ;
		}
		
		mParent->setToolBarPlayback() ; // play / pause icon refresh
	}
}
void NxGLScore::pause()
{
	if (timeScheduler->isPlaying()) timeScheduler->stop() ;
	
	assignCursors() ;
}
void NxGLScore::stopAndReset()
{
	timeScheduler->setReadyToPlay( false ) ;
	pause() ;
	usleep(100000) ;
	resetSchedule();
}
void NxGLScore::sendConfigOSC()
{
	timeScheduler->sendOSCConfigMessage() ;
}
void NxGLScore::stopGLTimer()
{
	glTimer->stop();
}
void NxGLScore::startGLTimer()
{
	glTimer->start( int(score->getEnvironment().displayStep) );
}
void NxGLScore::setTempo( float inTempo )
{
	score->setTempo( inTempo ) ;
	timeScheduler->changeTempo( (float)inTempo ) ;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////// UTILITIES /////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void NxGLScore::setCPUWarning( bool act )
{
	mParent->setCPUWarning( act ) ;
}
void NxGLScore::setCPUTimeStep( int val )
{
	mParent->setCPUTimeStep( val ) ;
}
float NxGLScore::pixToRealX(float _x)
{
	bX = length / (RScaleX*scaleX*10.0) ;
	float _r = ( _x -(selfW/2.0) )*bX / (selfW/2.0) - transX/(scaleX*RScaleX) + (float)length/2.0 ;
  	return _r ;
}
float NxGLScore::pixToRealY(float _y)
{
	bY = nHeight / (RScaleY*scaleY*10.0) ;
	float _r = ( - _y +(selfH/2.0) )*bY / (selfH/2.0) - transY/(scaleY*RScaleY) + (float)height/2.0 ;	
	return _r ;
}
float NxGLScore::snapXY(float _a)
{
  	return ( gridStepInt/100.0 * floor( ( _a / (gridStepInt/100.0) ) + 0.5 ) ) ;
}
float NxGLScore::realToPix() ////// UNUSED
{
	return 0 ;
}
int NxGLScore::getZoomScale() 
{
	return int(floor(100*scaleX/0.2)) ;
}
float NxGLScore::getGridOpacity() 
{
	return gridOpacity ;
}
float NxGLScore::getCursorZonesOpacity()
{
	return cursZonesOpacity ;
}
GLfloat* NxGLScore::colorConv(Color inColor)
{
	GLfloat *col = new GLfloat[4] ;
	int _r = inColor.c.red() ;
	int _g = inColor.c.green() ;
	int _b = inColor.c.blue() ;
	int _a = inColor.c.alpha() ;
	col[0] = (GLfloat)_r/255 ;
	col[1] = (GLfloat)_g/255 ;
	col[2] = (GLfloat)_b/255 ;
	col[3] = (GLfloat)_a/255 ;
	return col ;
}

